;;###################################################
;;###################################################
;;               program.clp                       ##
;; An Expert System to Help on Doing Projects.     ##
;;                                                 ##
;;###################################################
;;###################################################

;;***********************************
;; Set up initial facts for testing.*
;;***********************************


(deffacts init
   (question topics))

(defmodule MAIN
  (export deftemplate ?ALL))


;;****************************************************
;; Present the main menu.                            *
;;                                                   *
;; Variables:                                        *
;;   ?ml - used for retracting the menu level.       *
;;   ?response - used for binding in the users input.*
;;****************************************************

;;Most rules follow almost the same syntax so only the first time
;;a new feature is introduced in a rule, it is commented. That's why
;;not all rules have comments. 
;;In the EVALUATION module, ERROR module there are more complicated 
;;rules and so there are some more comments there

(defrule MAIN::main-menu
   ;;If the question topics has to be asked
  ?ml <- (question topics)
=>
   ;;retract the fact that the question topics has to be asked
  (retract ?ml)
   ;;crlf is for changing line
  (printout t crlf crlf crlf)
   ;;print the main menu (question topics) to the screen
  (printout t
  "        Choose one of the topics listed below" crlf
  "        by typing a number and pressing the return key." crlf crlf
  "                  1.) Choosing a research topic -" crlf 
  "                      checking an already chosen one." crlf
  "                  2.) Starting the project." crlf
  "                  3.) Programming." crlf
  "                  4.) Choosing a method of evaluation" crlf 
  "                      to support or refute the research hypothesis." crlf
  "                  5.) Quit the program." crlf crlf
  "        Choice: " )
   ;;bind the response with the output of the user 
  (bind ?response (read))
   ;;assert the fact that the answer is the ?response                        
  (assert (answer topics ?response))
   ;;assert the fact for correct answer checking             
  (assert (answer-check ?response topics 5))
   ;;focus to the module that checks for errors     
  (focus ERROR)                                  
  (printout t crlf))

;;***************************************
;;      The modules are defined         *
;;***************************************

;;each module imports facts from other modules and exports facts to
;;other modules
 
(defmodule CHOOSE-TOPIC
   (import MAIN deftemplate ?ALL)
   (export deftemplate ?ALL))

(defmodule START-PROJECT
   (import MAIN deftemplate ?ALL)
   (import CHOOSE-TOPIC deftemplate ?ALL)
   (export deftemplate ?ALL))

(defmodule PROGRAMMING
   (import MAIN deftemplate ?ALL)
   (import CHOOSE-TOPIC deftemplate ?ALL)
   (import START-PROJECT deftemplate ?ALL)
   (export deftemplate ?ALL))
  
(defmodule EVALUATION
   (import MAIN deftemplate ?ALL)
   (import CHOOSE-TOPIC deftemplate ?ALL)
   (import START-PROJECT deftemplate ?ALL)
   (import PROGRAMMING deftemplate ?ALL)
   (export deftemplate ?ALL))

(defmodule ADVICE
   (export deftemplate ?ALL)
   (import MAIN deftemplate ?ALL)
   (import CHOOSE-TOPIC deftemplate ?ALL)
   (import START-PROJECT deftemplate ?ALL)
   (import PROGRAMMING deftemplate ?ALL)
   (import EVALUATION deftemplate ?ALL))

(defmodule ERROR
   (export deftemplate ?ALL)
   (import CHOOSE-TOPIC deftemplate ?ALL)
   (import START-PROJECT deftemplate ?ALL)
   (import PROGRAMMING deftemplate ?ALL)
   (import EVALUATION deftemplate ?ALL))

;;***************************************
;; When the user enters a response of   *
;;    from the main menu,               *
;; Then go first to the module          *  
;; ADVICE for the introduction          *
;; and then to the appropriate module.  *
;;***************************************

;; If the answer to topics is 1 then assert the fact that the 
;; introduction for the first topic has to be given and focus
;; to the ADVICE module

(defrule MAIN::choose_topic
  (answer topics 1)
=>
  (focus ADVICE) 
  (assert (introduction 1)))

(defrule MAIN::start_project
  (answer topics 2)
=>
  (focus ADVICE) 
  (assert (introduction 2)))

(defrule MAIN::programming
  (answer topics 3)
=>
  (focus ADVICE) 
  (assert (introduction 3)))

(defrule MAIN::evaluation
  (answer topics 4)
 =>
  (focus ADVICE)
  (assert (introduction 4)))

;;***************************************
;;      CHOOSE-TOPIC MODULE             *
;;***************************************

(defrule CHOOSE-TOPIC::question1-1
  (answer topics 1)
=>
  (printout t crlf crlf)
  (printout t 
  "  Have you already chosen your topic of research?" crlf crlf
  "                 1.) yes" crlf
  "                 2.) no " crlf 
  "                 3.) return to the main menu " crlf crlf 
  "      Choice: " )
  (bind ?response1-1 (read))
  (printout t crlf crlf)

   ;;assert the fact that the response the user gave to question Q1-1 has
   ;;has to be checked in case it is invalid. The choices the user had
   ;;in question Q1-1 were 3.
  (assert (answer-check ?response1-1 Q1-1 3))

   ;;assert the fact that the answer to question Q1-1 is the response 
   ;;the user gave
  (assert (answer Q1-1 ?response1-1)) 

   ;;focus to the ERROR module for the error checking
  (focus ERROR))


;;If the answer to question 1-1 is 2 ask question 1-10

(defrule CHOOSE-TOPIC::question1-10
   (answer Q1-1 2)
=>
  (printout t crlf crlf)
(printout t 
  "  Are you enthusiastic about any specific area of AI research?" crlf crlf
  "                 1.) yes" crlf
  "                 2.) no " crlf 
  "                 3.) return to the main menu " crlf crlf 
  "      Choice: " )
  (bind ?response1-10 (read))
  (printout t crlf crlf)
  (assert (answer Q1-10 ?response1-10))
  (assert (answer-check ?response1-10 Q1-10 3))
  (focus ERROR))


;;If the answer to question 1-10 is 2 ask question 1-12

(defrule CHOOSE-TOPIC::question1-12
   (answer Q1-10 2)
=>
  (printout t crlf crlf)
(printout t 
  "  Are you enthusiastic about doing AI research in general?" crlf crlf
  "                 1.) yes" crlf
  "                 2.) so and so " crlf 
  "                 3.) no " crlf 
  "                 4.) return to the main menu " crlf crlf 
  "      Choice: " )
  (bind ?response1-12 (read))
  (printout t crlf crlf)
  (assert (answer Q1-12 ?response1-12))
  (assert (answer-check ?response1-12 Q1-12 4))
  (focus ERROR))

;;If the answer to the question 1-12 is 1 give advice1-1

(defrule CHOOSE-TOPIC::advice1-1
  (answer Q1-12 1)
=>
  (assert (advice 1-1))
  (focus ADVICE)
  (printout t crlf crlf))


;;If the answer to the question 1-12 is 2 or 3 or question 1-4 is completed ask
;;question 1-6

(defrule CHOOSE-TOPIC::question1-6
  (or (answer Q1-12 2)
      (answer Q1-12 3)
      (completed Q1-4))
=>
  (printout t crlf crlf)
  (printout t 
  "  Have you read some relevant literature?" crlf crlf
  "                 1.) yes" crlf
  "                 2.) no " crlf 
  "                 3.) return to the main menu " crlf crlf 
  "      Choice: " )
  (bind ?response1-6 (read))
  (printout t crlf crlf)
  (assert (answer Q1-6 ?response1-6))
  (assert (answer-check ?response1-6 Q1-6 3))
  (assert (completed Q1-6))
  (focus ERROR))
 

;;If Q1-6 is asked and the answer to Q1-1 is 2 ask question 1-7

(defrule CHOOSE-TOPIC::question1-7
  (and (completed Q1-6)
       (answer Q1-1 2))
=>
  (printout t crlf crlf)
  (printout t 
  "  Have you talked with people about AI research?" crlf crlf
  "                 1.) yes" crlf
  "                 2.) no " crlf 
  "                 3.) return to the main menu " crlf crlf 
  "      Choice: " )
  (bind ?response1-7 (read))
  (printout t crlf crlf)
  (assert (answer Q1-7 ?response1-7))
  (assert (answer-check ?response1-7 Q1-7 3))
  (focus ERROR))

;;If either Q1-6 or Q1-7 is 2 and the answer to question 1-1 is 2
;;then give advice1-2
 
(defrule CHOOSE-TOPIC::advice1-2
  (and (answer Q1-1 2)
       (or (answer Q1-6 2)
           (answer Q1-7 2)))
=>
  (assert (advice 1-2))
  (focus ADVICE)
  (printout t crlf crlf)) 


;;If both Q1-6 and Q1-7 are 1 then give advice1-3

(defrule CHOOSE-TOPIC::advice1-3
  (and (answer Q1-6 1)
       (answer Q1-7 1))
=>
  (assert (advice 1-3))
  (focus ADVICE)
  (printout t crlf crlf)) 


;;If the answer to question 1-10 is 1 ask question Q1-11

(defrule CHOOSE-TOPIC::question1-11
   (answer Q1-10 1)
=>
  (printout t crlf crlf)
(printout t 
  "  Is this area new to you?" crlf crlf
  "                 1.) yes" crlf
  "                 2.) no, it utilizes some of my already proven abilities " crlf 
  "                 3.) return to the main menu " crlf crlf 
  "      Choice: " )
  (bind ?response1-11 (read))
  (printout t crlf crlf)
  (assert (answer Q1-11 ?response1-11))
  (assert (answer-check ?response1-11 Q1-11 3))
  (focus ERROR))


;;If the answer to question 1-11 is 1 then give advice 1-4

(defrule CHOOSE-TOPIC::advice1-4
  (answer Q1-11 1)
=>
  (assert (advice 1-4))
  (focus ADVICE)
  (printout t crlf crlf)) 

;;If the advice 1-4 is given and the answer to question 1-1 is 2 
;;or the answer to question 1-11 is 2 or the question 1-2 is completed 
;;ask question 1-8

(defrule CHOOSE-TOPIC::question1-8
  (or (and (completed advice1-4)
           (answer Q1-1 2))
           (answer Q1-11 2)
           (completed Q1-2))
=>
  (printout t crlf crlf)
  (printout t 
  "  Have you found a potential supervisor?" crlf crlf
  "                 1.) yes" crlf
  "                 2.) no " crlf 
  "                 3.) return to the main menu " crlf crlf 
  "      Choice: " )
  (bind ?response1-8 (read))
  (printout t crlf crlf)
  (assert (answer Q1-8 ?response1-8))
  (assert (answer-check ?response1-8 Q1-8 3))
  (assert (completed Q1-8))
  (focus ERROR))


;;If the answer to question 1-8 is 2 and the answer to question 1-1 was 2 
;;give advice 1-5. 

(defrule CHOOSE-TOPIC::advice1-5
  (and (answer Q1-8 2)
       (answer Q1-1 2))
=>
  (assert (advice 1-5))
  (focus ADVICE)
  (printout t crlf crlf)) 


;;If the answer to question 1-8 is 1 and the answer to question 1-1 was 2 
;;give advice 1-6.

(defrule CHOOSE-TOPIC::advice1-6
  (and (answer Q1-8 1)
       (answer Q1-1 2))
=>
  (assert (advice 1-6))
  (focus ADVICE)
  (printout t crlf crlf)) 
 

;;If the answer to question 1-1 is 1 ask question 1-15

(defrule CHOOSE-TOPIC::question1-15
  (answer Q1-1 1)
=>
  (printout t crlf crlf)
  (printout t 
" Is the area of your topic new to you? " crlf crlf
"                 1.) yes" crlf
"                 2.) no, it utilizes some of my already proven abilities " crlf
"                 3.) return to the main menu " crlf crlf 
"      Choice: " ) 
  (bind ?response1-15 (read))
  (printout t crlf crlf)
  (assert (answer Q1-15 ?response1-15))
  (assert (answer-check ?response1-15 Q1-15 3))
  (focus ERROR)
  (assert (completed Q1-15)))

;;If question 1-15 is completed ask question 1-3 

(defrule CHOOSE-TOPIC::question1-3
  (completed Q1-15)
=>
  (printout t crlf crlf)
  (printout t crlf crlf crlf 
" Are you enthusiastic about this topic? " crlf crlf
"                 1.) yes, I like it very much" crlf
"                 2.) so and so, I cannot be sure right now " crlf
"                 3.) no, I don't like it so much " crlf 
"                 4.) return to the main menu " crlf crlf 
"      Choice: " ) 
  (bind ?response1-3 (read))
  (printout t crlf crlf)
  (assert (answer Q1-3 ?response1-3))
  (assert (answer-check ?response1-3 Q1-3 4))
  (focus ERROR)
  (assert (completed Q1-3)))

;;If question 1-3 is completed go on to question 1-14 

(defrule CHOOSE-TOPIC::question1-14
  (completed Q1-3)
=>
  (printout t crlf crlf)
  (printout t 
" Do you feel confident in what you are going to do? " crlf crlf
"                 1.) yes, I feel a lot confidence" crlf
"                 2.) not really " crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " ) 
  (bind ?response1-14 (read))
  (printout t crlf crlf)
  (assert (answer Q1-14 ?response1-14))
  (assert (answer-check ?response1-14 Q1-14 3))
  (focus ERROR)
  (assert (completed Q1-14)))
 
;;If question 1-14 is completed go on to question 1-2

(defrule CHOOSE-TOPIC::question1-2
  (completed Q1-14)
=>
  (printout t crlf crlf)
  (printout t 
"  Is this project topic an original work that will contribute to knowledge?" crlf crlf
"                 1.) yes" crlf
"                 2.) I'm not sure" crlf
"                 3.) no " crlf 
"                 4.) return to the main menu " crlf crlf 
"      Choice: " )
  (bind ?response1-2 (read))
  (printout t crlf crlf)
  (assert (answer Q1-2 ?response1-2))
  (assert (answer-check ?response1-2 Q1-2 4))
  (focus ERROR)
  (assert (completed Q1-2)))

;;If question 1-8 is completed and the answer to Q1-1 is 1 ask question 1-4 

(defrule CHOOSE-TOPIC::question1-4
  (and (completed Q1-8)
       (answer Q1-1 1))
=>
  (printout t crlf crlf)
  (printout t 
"  How difficult do you consider your topic to be?" crlf crlf
"                 1.) Average or less" crlf
"                 2.) Above average" crlf
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
  (bind ?response1-4 (read))
  (printout t crlf crlf)
  (assert (answer Q1-4 ?response1-4))
  (assert (answer-check ?response1-4 Q1-4 3))
  (focus ERROR)
  (assert (completed Q1-4)))

;;If Q1-4 is completed and the answer to Q1-8 was 1 ask Q1-5 first(with higher 
;;priority than the other questions).

(defrule CHOOSE-TOPIC::question1-5
  (declare (salience 50))
  (and (completed Q1-4)
       (answer Q1-8 1))
=>
  (printout t crlf crlf)
  (printout t 
" How difficult does your supervisor consider the topic to be? " crlf crlf
"                 1.) Average or less" crlf
"                 2.) Above average" crlf
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
  (bind ?response1-5 (read))
  (printout t crlf crlf)
  (assert (answer Q1-5 ?response1-5))
  (assert (answer-check ?response1-5 Q1-5 3))
  (focus ERROR))


;;If question 1-6 is completed and the answer to question 1-1 s 1 then
;;ask question 1-13.

(defrule CHOOSE-TOPIC::question1-13
  (and (completed Q1-6)
       (answer Q1-1 1))
=>
  (printout t crlf crlf)
  (printout t 
"  Have you talked to people related to your topic of research?" crlf crlf
"                 1.) yes" crlf
"                 2.) no" crlf
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
  (bind ?response1-13 (read))
  (printout t crlf crlf)
  (assert (completed Q1-13))
  (assert (answer Q1-13 ?response1-13))
  (assert (answer-check ?response1-13 Q1-13 3))
  (focus ERROR))

;;If the answers to all questions asked when the user has chosen his topic
;;are 1 and the answer to Q1-15 is 2 then give advice 1-7

(defrule CHOOSE-TOPIC::advice1-7
  (and (answer Q1-2 1)
       (answer Q1-3 1)
       (answer Q1-14 1)
       (answer Q1-4 1)
       (answer Q1-5 1)
       (answer Q1-6 1)
       (answer Q1-13 1)
       (answer Q1-15 2))
=>
  (assert (advice 1-7))
  (focus ADVICE)
  (printout t crlf crlf)) 
 

;;If the answer to question 1-3 is 2 or 3 give advice 1-8

(defrule CHOOSE-TOPIC::advice1-8
  (declare (salience 50))
  (or  (answer Q1-3 2)
       (answer Q1-3 3))
=>
  (assert (advice 1-8))
  (focus ADVICE)
  (printout t crlf crlf)) 

;;If the answer to question 1-15 is 1 then give advice 1-4

(defrule CHOOSE-TOPIC::advice1-4new
  (declare (salience 50))
  (answer Q1-15 1)
=>
  (assert (advice 1-4))
  (focus ADVICE)
  (printout t crlf crlf))

;;If the answer to question 1-14 is 2 then give advice 1-10

(defrule CHOOSE-TOPIC::advice1-10
  (declare (salience 50))
  (answer Q1-14 2)
=>
  (assert (advice 1-10))
  (focus ADVICE)
  (printout t crlf crlf))

;;If the answer to question 1-2 is 3 then give advice 1-11

(defrule CHOOSE-TOPIC::advice1-11
  (declare (salience 50))
  (answer Q1-2 3)
=>
  (assert (advice 1-11))
  (focus ADVICE)
  (printout t crlf crlf))
 
;;If the answer to question 1-2 is 2 and the answer to question 1-8 is 1
;;then give advice 1-12

(defrule CHOOSE-TOPIC::advice1-12
  (declare (salience 50))
  (and (answer Q1-2 2)
       (answer Q1-8 1))
=>
  (assert (advice 1-12))
  (focus ADVICE)
  (printout t crlf crlf))
 
;;If the answer to question 1-8 is 2 then ask question 1-9

(defrule CHOOSE-TOPIC::question1-9
  (declare (salience 50))
         (answer Q1-8 2)
=>
  (printout t crlf crlf)
  (printout t 
"  Have you looked for a supervisor?" crlf crlf
"                 1.) yes and I can't find one" crlf
"                 2.) no, not yet" crlf
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
  (bind ?response1-9 (read))
  (printout t crlf crlf)
  (assert (answer Q1-9 ?response1-9))
  (assert (answer-check ?response1-9 Q1-9 3))
  (focus ERROR))

;;If the answer to question 1-9 is 2 then give advice 1-13

(defrule CHOOSE-TOPIC::advice1-13
  (declare (salience 50))
  (answer Q1-9 2)
=>
  (assert (advice 1-13))
  (focus ADVICE)
  (printout t crlf crlf))

;;If the answer to question 1-9 is 1 then give advice 1-14

(defrule CHOOSE-TOPIC::advice1-14
  (declare (salience 50))
  (answer Q1-9 1)
=>
  (assert (advice 1-14))
  (focus ADVICE)
  (printout t crlf crlf))

;;If the answer to question 1-5 is 2 then give advice 1-15

(defrule CHOOSE-TOPIC::advice1-15
  (declare (salience 50))
  (answer Q1-5 2)
=>
  (assert (advice 1-15))
  (focus ADVICE)
  (printout t crlf crlf))

;;If the answer to question 1-5 is 1 and the answer to 
;;question 1-4 is 2 give advice 1-16

(defrule CHOOSE-TOPIC::advice1-16
  (declare (salience 50))
  (and (answer Q1-5 1)
       (answer Q1-4 2))  
=>
  (assert (advice 1-16))
  (focus ADVICE)
  (printout t crlf crlf))

;;If the answer to question 1-6 is 2 and the answer to 
;;question 1-1 is 1 give advice 1-17

(defrule CHOOSE-TOPIC::advice1-17
  (declare (salience 50))
  (and (answer Q1-6 2)
       (answer Q1-1 1))  
=>
  (assert (advice 1-17))
  (focus ADVICE)
  (printout t crlf crlf))

;;If the answer to question 1-13 is 2 give advice 1-18

(defrule CHOOSE-TOPIC::advice1-18
  (declare (salience 50))
  (answer Q1-13 2)
=>
  (assert (advice 1-18))
  (focus ADVICE)
  (printout t crlf crlf))

;;If question 1-13 is completed then ask the user the general question
;;asked in the end.

(defrule CHOOSE-TOPIC::general-question
  (completed Q1-13)
=>
  (assert (question general))
  (focus ADVICE)
  (printout t crlf crlf))

;;***************************************
;;      START-PROJECT MODULE            *
;;***************************************

(defrule START-PROJECT::question2-1
  (answer topics 2)
=>
  (printout t crlf crlf)
  (printout t 
" One common reason why students have problems starting their project " crlf
" is that they cannot create new ideas. They just sit in their room with " crlf
" a blank piece of paper and a pencil, expecting good ideas to come from above." crlf 
" Do you find yourself in the situation described above?" crlf crlf	
"                 1.) yes" crlf
"                 2.) No, i don't have a problem creating ideas " crlf
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response2-1 (read))
(printout t crlf crlf)
(assert (answer Q2-1 ?response2-1))
(assert (answer-check ?response2-1 Q2-1 3))
(focus ERROR))

;;If the answer to question 2-1 is 1 ask question 2-2

(defrule START-PROJECT::question2-2
  (answer Q2-1 1)
=>
  (printout t crlf crlf)
  (printout t 
" Have you read some of the literature relevant to your topic?" crlf crlf	
"                 1.) yes" crlf
"                 2.) no" crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response2-2 (read))
(printout t crlf crlf)
(assert (answer Q2-2 ?response2-2))
(assert (answer-check ?response2-2 Q2-2 3))
(assert (completed Q2-2))
(focus ERROR))

;;If the answer to question 2-2 is 2 give advice 2-1

(defrule START-PROJECT::advice2-1
  (declare (salience 50))
  (answer Q2-2 2)
=>
  (assert (advice 2-1))
  (focus ADVICE))

;;If the answer to question 2-2 is 1 ask question2-3

(defrule START-PROJECT::question2-3
  (declare (salience 50))
  (answer Q2-2 1)
=>
  (printout t crlf crlf)
  (printout t 
" Have you tried reading the literature with a question in mind," crlf
" e.g  What is wrong with this?  How can i use this?" crlf crlf	
"                 1.) yes, I do that" crlf
"                 2.) no, not really" crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response2-3 (read))
(printout t crlf crlf)
(assert (answer Q2-3 ?response2-3))
(assert (answer-check ?response2-3 Q2-3 3))
(assert (completed Q2-3))
(focus ERROR))

;;If the answer to question 2-3 is 2 then give advice 2-2

(defrule START-PROJECT::advice2-2
  (declare (salience 50))
  (answer Q2-3 2)
=>
  (assert (advice 2-2))
  (focus ADVICE))

;;If question 2-2 is completed ask question 2-4

(defrule START-PROJECT::question2-4
   (completed Q2-2)
=>
  (printout t crlf crlf)
  (printout t 
" Have you talked to people about your topic of research? " crlf crlf	
"                 1.) yes" crlf
"                 2.) no, not really" crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response2-4 (read))
(printout t crlf crlf)
(assert (answer Q2-4 ?response2-4))
(assert (answer-check ?response2-4 Q2-4 3))
(assert (completed Q2-4))
(focus ERROR))

;;If the answer to question 2-4 is 2 then give advice 2-3

(defrule START-PROJECT::advice2-3
  (declare (salience 50))
  (answer Q2-4 2)
=>
  (assert (advice 2-3))
  (focus ADVICE))

;;If question 2-4 is completed ask question 2-5

(defrule START-PROJECT::question2-5
   (completed Q2-4)
=>
  (printout t crlf crlf)
  (printout t 
" Are you trying to tackle the problem as it is from the beginning, " crlf
" i.e in all its complexity? " crlf crlf	
"                 1.) Yes, i'm working on the problem as it is" crlf
"                 2.) No, i'm tackling a simplified version of the problem" crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response2-5 (read))
(printout t crlf crlf)
(assert (answer Q2-5 ?response2-5))
(assert (answer-check ?response2-5 Q2-5 3))
(assert (completed Q2-5))
(focus ERROR))

;;If the answer to question 2-5 is 1 then give advice 2-4

(defrule START-PROJECT::advice2-4
  (declare (salience 50))
  (answer Q2-5 1)
=>
  (assert (advice 2-4))
  (focus ADVICE))

;;If question 2-5 is completed ask question 2-6

(defrule START-PROJECT::question2-6
   (completed Q2-5)
=>
  (printout t crlf crlf)
  (printout t 
" Have you tried writing your ideas down in a working paper? " crlf crlf	
"                 1.) yes" crlf
"                 2.) not really " crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response2-6 (read))
(printout t crlf crlf)
(assert (answer Q2-6 ?response2-6))
(assert (answer-check ?response2-6 Q2-6 3))
(assert (completed Q2-6))
(focus ERROR))

;;If the answer to question 2-6 is 1 ask question2-7

(defrule START-PROJECT::question2-7
  (declare (salience 50))
  (answer Q2-6 1)
=>
  (printout t crlf crlf)
  (printout t 
" So you write down your ideas but have you actually imagined yourself " crlf
" trying to explain these ideas of yours to someone? " crlf crlf	
"                 1.) yes, I do that" crlf
"                 2.) no, not really" crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response2-7 (read))
(printout t crlf crlf)
(assert (answer Q2-7 ?response2-7))
(assert (answer-check ?response2-7 Q2-7 3))
(focus ERROR))

;;If the answer to question 2-7 is 2 then give advice 2-5

(defrule START-PROJECT::advice2-5
   (declare (salience 50))
   (answer Q2-7 2)
=>
  (assert (advice 2-5))
  (focus ADVICE))


;;If the answer to question 2-6 is 2 then give advice 2-6

(defrule START-PROJECT::advice2-6
   (declare (salience 50))
   (answer Q2-6 2)
=>
  (assert (advice 2-6))
  (focus ADVICE))


;;If the answer to question 2-1 is 2 ask question2-8

(defrule START-PROJECT::question2-8
  (answer Q2-1 2)
=>
  (printout t crlf crlf)
  (printout t 
" Do you find that nobody else seems to understand your ideas?" crlf crlf	
"                 1.) Yes, no one seems to understand my ideas" crlf
"                 2.) I haven't talked to anyone about them" crlf 
"                 3.) No, people understand my ideas " crlf 
"                 4.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response2-8 (read))
(printout t crlf crlf)
(assert (answer Q2-8 ?response2-8))
(assert (answer-check ?response2-8 Q2-8 4))
(assert (completed Q2-8))
(focus ERROR))

;;If the answer to question 2-8 is 1 then give advice 2-7

(defrule START-PROJECT::advice2-7
   (declare (salience 50))
   (answer Q2-8 1)
=>
  (assert (advice 2-7))
  (focus ADVICE))


;;If the answer to question 2-8 is 2 then give advice 2-8

(defrule START-PROJECT::advice2-8
   (declare (salience 50))
   (answer Q2-8 2)
=>
  (assert (advice 2-8))
  (focus ADVICE))


;;If question 2-8 is completed ask question 2-9

(defrule START-PROJECT::question2-9
   (completed Q2-8)
=>
  (printout t crlf crlf)
  (printout t 
" Do you have a great idea  and it only remains to test it by proving " crlf
" a theorem, writing a program, explaining it to a friend etc but something " crlf
" is holding you back? " crlf crlf	
"                 1.) Yes, that is true " crlf
"                 2.) No, that's not true " crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response2-9 (read))
(printout t crlf crlf)
(assert (answer Q2-9 ?response2-9))
(assert (answer-check ?response2-9 Q2-9 3))
(assert (completed Q2-9))
(focus ERROR))

;;If the answer to question 2-9 is 1 then give advice 2-9

(defrule START-PROJECT::advice2-9
   (declare (salience 50))
   (answer Q2-9 1)
=>
  (assert (advice 2-9))
  (focus ADVICE))


;;If either question 2-6 is completed or question 2-9 is completed 
;;ask question 2-10

(defrule START-PROJECT::question2-10
   (or (completed Q2-6)
       (completed Q2-9)) 
=>
  (printout t crlf crlf)
  (printout t 
" Your work in AI should apparently be aimed at understanding some " crlf
" property of intelligence. How do you approach the problem? " crlf crlf	
"                 1.) I'm trying to work on a concrete situation first" crlf
"                 2.) I'm trying to tackle the problem the way it is" crlf
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response2-10 (read))
(printout t crlf crlf)
(assert (answer Q2-10 ?response2-10))
(assert (answer-check ?response2-10 Q2-10 3))
(assert (completed Q2-10))
(focus ERROR))

;;If the question 2-10 is completed ask question 2-11

(defrule START-PROJECT::question2-11
   (completed Q2-10) 
=>
  (printout t crlf crlf)
  (printout t 
" Do you have high standards for your finished product? " crlf crlf	
"                 1.) yes, I have high standards " crlf
"                 2.) not so high" crlf
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response2-11 (read))
(printout t crlf crlf)
(assert (answer Q2-11 ?response2-11))
(assert (answer-check ?response2-11 Q2-11 3))
(assert (completed Q2-11))
(focus ERROR))

;;If the question 2-11 is completed ask question 2-13

(defrule START-PROJECT::question2-13
   (completed Q2-11) 
=>
  (printout t crlf crlf)
  (printout t 
" Do you believe in what you do? " crlf crlf	
"                 1.) yes, I am very confident about it " crlf
"                 2.) I'm not so sure" crlf
"                 3.) Not so much, I'm afraid that it will may not be worth it " crlf
"                     in the end" crlf 	
"                 4.) return to the main menu " crlf crlf                                                                                                                                                                      
"      Choice: " )
(bind ?response2-13 (read))
(printout t crlf crlf)
(assert (answer Q2-13 ?response2-13))
(assert (answer-check ?response2-13 Q2-13 4))
(assert (completed Q2-13))
(focus ERROR))

;;If the question 2-13 is completed ask question 2-14

(defrule START-PROJECT::question2-14
   (completed Q2-13) 
=>
  (printout t crlf crlf)
  (printout t 
" How do you feel about your research skills?" crlf crlf	
"                 1.) I'm confident " crlf
"                 2.) I'm not sure if I have the skills" crlf
"                 3.) I'm very much afraid that I'm not capable of doing " crlf 
"                     research" crlf 	
"                 4.) return to the main menu " crlf crlf                                                                                                                                                                      
"      Choice: " )
(bind ?response2-14 (read))
(printout t crlf crlf)
(assert (answer Q2-14 ?response2-14))
(assert (answer-check ?response2-14 Q2-14 4))
(assert (completed Q2-14))
(focus ERROR))

;;If the question 2-14 is completed ask question 2-15

(defrule START-PROJECT::question2-15
   (completed Q2-14) 
=>
  (printout t crlf crlf)
  (printout t 
" Have you defined your research goal(s), " crlf
" i.e. do you know what you are trying to prove?" crlf crlf	
"                 1.) yes, I have " crlf
"                 2.) No, it's not completely clear to me" crlf
"                 3.) return to the main menu " crlf crlf                                                                                                                                                                      
"      Choice: " )
(bind ?response2-15 (read))
(printout t crlf crlf)
(assert (answer Q2-15 ?response2-15))
(assert (answer-check ?response2-15 Q2-15 3))
(focus ERROR))

;;If the answer to question 2-10 is 2 then give advice 2-10

(defrule START-PROJECT::advice2-10
   (declare (salience 50))
   (answer Q2-10 2)
=>
  (assert (advice 2-10))
  (focus ADVICE))


;;If the answer to question 2-11 is 1 ask question Q2-12

(defrule START-PROJECT::question2-12
  (declare (salience 50))
  (answer Q2-11 1)
=>
  (printout t crlf crlf)
  (printout t 
" Do you try to apply your high standards to your initial attempts?" crlf crlf	
"                 1.) yes" crlf
"                 2.) no" crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response2-12 (read))
(printout t crlf crlf)
(assert (answer Q2-12 ?response2-12))
(assert (answer-check ?response2-12 Q2-12 3))
(focus ERROR))

;;If the answer to question 2-12 is 1 give advice 2-11

(defrule START-PROJECT::advice2-11
   (declare (salience 50))
   (answer Q2-12 1)
=>
  (assert (advice 2-11))
  (focus ADVICE))


;;If the answer to question 2-13 is 2 or 3 give advice 2-12

(defrule START-PROJECT::advice2-12
   (declare (salience 50))
   (or (answer Q2-13 2)
       (answer Q2-13 3))
=>
  (assert (advice 2-12))
  (focus ADVICE)
  (printout t crlf crlf))

;;If the answer to question 2-14 is 2 or 3 give advice 2-13

(defrule START-PROJECT::advice2-13
   (declare (salience 50))
   (or (answer Q2-14 2)
       (answer Q2-14 3))
=>
  (assert (advice 2-13))
  (focus ADVICE)
  (printout t crlf crlf))

;;If the answer to question 2-15 is 2 give advice 2-14

(defrule START-PROJECT::advice2-14
   (answer Q2-15 2)
=>
  (assert (advice 2-14))
  (focus ADVICE))


;;If the answer to question 2-15 is 1 ask question 2-16

(defrule START-PROJECT::question2-16
  (answer Q2-15 1)
=>
  (printout t crlf crlf)
  (printout t 
" Do you have a single one goal?"crlf crlf	
"                 1.) yes" crlf
"                 2.) no, I have more than one research goals on my way " crlf
"                     to the final goal" crlf
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response2-16 (read))
(printout t crlf crlf)
(assert (completed Q2-16))
(assert (answer Q2-16 ?response2-16))
(assert (answer-check ?response2-16 Q2-16 3))
(focus ERROR))

;;If the question 2-16 is completed ask question 2-18

(defrule START-PROJECT::question2-18
   (completed Q2-16) 
=>
  (printout t crlf crlf)
  (printout t 
" Have you chosen a methodology to follow?" crlf crlf	
"                 1.) yes, I have " crlf
"                 2.) yes, but I haven't determined all the stages " crlf
"                     of my research" crlf
"                 3.) no, I haven't done that " crlf
"                 4.) return to the main menu " crlf crlf                                                                                                                                                                      
"      Choice: " )
(bind ?response2-18 (read))
(printout t crlf crlf)
(assert (answer Q2-18 ?response2-18))
(assert (answer-check ?response2-18 Q2-18 4))
(assert (completed Q2-18))
(focus ERROR))

;;If the answer to question 2-16 is 1 give advice 2-15

(defrule START-PROJECT::advice2-15
   (declare (salience 50))
   (answer Q2-16 1)
=>
  (assert (advice 2-15))
  (focus ADVICE))


;;If the answer to question 2-16 is 2 ask question 2-17

(defrule START-PROJECT::question2-17
   (declare (salience 50))
   (answer Q2-16 2)
=>
  (printout t crlf crlf)
    (printout t 
" Have you put your research goals in a hierarchy of prioritized goals? " crlf crlf	
"                 1.) yes, I have " crlf
"                 2.) no " crlf
"                 3.) return to the main menu " crlf crlf                                                                                                                                                                      
"      Choice: " )
(bind ?response2-17 (read))
(printout t crlf crlf)
(assert (answer Q2-17 ?response2-17))
(assert (answer-check ?response2-17 Q2-17 3))
(focus ERROR))

;;If the answer to question 2-17 is 2 give advice 2-16

(defrule START-PROJECT::advice2-16
   (declare (salience 50))
   (answer Q2-17 2)
=>
  (assert (advice 2-16))
  (focus ADVICE))


;;If the answer to question 2-18 is 2 or 3 give advice 2-17

(defrule START-PROJECT::advice2-17
  (declare (salience 50))
  (or (answer Q2-18 2)
      (answer Q2-18 3))
=>
  (assert (advice 2-17))
  (focus ADVICE))


;;If all the answer lead to the conclusion that although the user has problems 
;;creating new ideas there is nothing that he does wrong, give advice 2-18

(defrule START-PROJECT::advice2-18
(declare (salience 50))
  (and (answer Q2-1 1)
       (answer Q2-3 1)
       (answer Q2-4 1)
       (answer Q2-5 2)
       (answer Q2-7 1)
       (answer Q2-10 1)
       (or (answer Q2-11 2)
           (and (answer Q2-11 1)
                (answer Q2-12 2)))
       (answer Q2-13 1)
       (answer Q2-14 1)
       (answer Q2-15 1)
       (answer Q2-16 2)
       (answer Q2-17 1)
       (answer Q2-18 1))
=>
  (assert (advice 2-18))
  (focus ADVICE))


;;If all the answer lead to the conclusion that there seems to be no 
;;problem, give advice 2-19

(defrule START-PROJECT::advice2-19
(declare (salience 50))
  (and (answer Q2-1 2)
       (answer Q2-8 3)
       (answer Q2-9 2)
       (or (answer Q2-11 2)
           (and (answer Q2-11 1)
                (answer Q2-12 2)))
       (answer Q2-13 1)
       (answer Q2-14 1)
       (answer Q2-15 1)
       (answer Q2-16 2)
       (answer Q2-17 1)
       (answer Q2-18 1))
=>
  (assert (advice 2-19))
  (focus ADVICE))


;;If Q2-18 is completed ask the user to either return to the main menu or exit
 
(defrule START-PROJECT::general-question
  (completed Q2-18)
=>
  (assert (question general))
  (focus ADVICE))



;;***************************************
;;         PROGRAMMING MODULE           *
;;***************************************

(defrule PROGRAMMING::question3-1
  (answer topics 3)
=>
  (printout t crlf crlf)
  (printout t 
" When you want to write a program, do you immediately ..." crlf crlf	
"                 1.) start writing code ?" crlf
"                 2.) try to come up with a specification for the program?" crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response3-1 (read))
(printout t crlf crlf)
(assert (completed Q3-1))
(assert (answer Q3-1 ?response3-1))
(assert (answer-check ?response3-1 Q3-1 3))
(focus ERROR))

;;If question 3-1 is completed ask question 3-2

(defrule PROGRAMMING::question3-2
  (completed Q3-1)
=>
  (printout t crlf crlf)
  (printout t 
" You have found a bug somewhere in your program. Do you... " crlf crlf	
"                 1.) keep making changes until the bug goes away ?" crlf
"                 2.) get a listing and then go and try to study the problem?" crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response3-2 (read))
(printout t crlf crlf)
(assert (completed Q3-2))
(assert (answer Q3-2 ?response3-2))
(assert (answer-check ?response3-2 Q3-2 3))
(focus ERROR))

;;If the answer to question 3-1 is 1 give advice 3-1

(defrule PROGRAMMING::advice3-1
  (declare (salience 50))
  (answer Q3-1 1)
=>
  (assert (advice 3-1))
  (focus ADVICE))


;;If the answer to question 3-2 is 1 ask question 3-3

(defrule PROGRAMMING::question3-3
  (declare (salience 50))
  (answer Q3-2 1)
=>
  (printout t crlf crlf)
  (printout t 
" When the bug is exposed and a nice output generated, what do you do? " crlf crlf	
"                 1.) Nothing, the bug is gone so it's ok " crlf
"                 2.) I try to explain why it now works and find the theory " crlf
"                     behind it" crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response3-3 (read))
(printout t crlf crlf)
(assert (answer Q3-3 ?response3-3))
(assert (answer-check ?response3-3 Q3-3 3))
(focus ERROR))


;;If the answer to question 3-3 is 1 give advice 3-2

(defrule PROGRAMMING::advice3-2
  (declare (salience 50))
  (answer Q3-3 1)
=>
  (assert (advice 3-2))
  (focus ADVICE))

;;If the answer to question 3-2 is 2 or the answer to question 3-3 is 2
;;ask question 3-4

(defrule PROGRAMMING::question3-4
  (declare (salience 50))
  (or (answer Q3-2 2)
      (answer Q3-3 2))
=>
  (printout t crlf crlf)
  (printout t 
" You find it hard to work out the theory. What do you do?" crlf crlf	
"                 1.) I leave the theory. If I cannot work it out I'm not" crlf
"                     going to bother with it" crlf
"                 2.) I try to describe how it works to a friend or in a seminar" crlf 
"                 3.) I don't know what to do " crlf
"                 4.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response3-4 (read))
(printout t crlf crlf)
(assert (answer Q3-4 ?response3-4))
(assert (answer-check ?response3-4 Q3-4 4))
(focus ERROR))

;;If the answer to question 3-4 is 2 ask question 3-5

(defrule PROGRAMMING::question3-5
  (declare (salience 50))
  (answer Q3-4 2)
=>
  (printout t crlf crlf)
  (printout t 
" Do people understand what you are trying to explain?" crlf crlf	
"                 1.) Yes, they seem to understand" crlf
"                 2.) No, they don't" crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response3-5 (read))
(printout t crlf crlf)
(assert (answer Q3-5 ?response3-5))
(assert (answer-check ?response3-5 Q3-5 3))
(focus ERROR))

;;If the answer to question 3-4 is 1 give advice 3-3

(defrule PROGRAMMING::advice3-3
  (declare (salience 50))
  (answer Q3-4 1)
=>
  (assert (advice 3-3))
  (focus ADVICE))

;;If the answer to question 3-4 is 3 give advice 3-4

(defrule PROGRAMMING::advice3-4
  (declare (salience 50))
  (answer Q3-4 3)
=>
  (assert (advice 3-4))
  (focus ADVICE))


;;If the answer to question 3-5 is 2 give advice 3-5

(defrule PROGRAMMING::advice3-5
  (declare (salience 50))
  (answer Q3-5 2)
=>
  (assert (advice 3-5))
  (focus ADVICE))


;;If question 3-2 is completed ask question 3-6

(defrule PROGRAMMING::question3-6
  (completed Q3-2)
=>
  (printout t crlf crlf)
  (printout t 
" How do you feel as an AI programmer? " crlf crlf	
"                 1.) I feel quite good" crlf
"                 2.) I feel that my skills need a lot of improvement" crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response3-6 (read))
(printout t crlf crlf)
(assert (answer Q3-6 ?response3-6))
(assert (completed Q3-6))
(assert (answer-check ?response3-6 Q3-6 3))
(focus ERROR))


;;If question 3-6 is completed ask question 3-9

(defrule PROGRAMMING::question3-9
  (completed Q3-6)
=>
  (printout t crlf crlf)
  (printout t 
"  Do you try to add features to your program to make it more interesting?" crlf crlf	
"                 1.) yes, if my code is too simple it is not going to be " crlf
"                     interesting " crlf
"                 2.) no, I try to make programs as simple as possible " crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response3-9 (read))
(printout t crlf crlf)
(assert (answer Q3-9 ?response3-9))
(assert (completed Q3-9))
(assert (answer-check ?response3-9 Q3-9 3))
(focus ERROR))

;;If the answer to question 3-6 is 2 ask question 3-7

(defrule PROGRAMMING::question3-7
  (declare (salience 50))
  (answer Q3-6 2)
=>
  (printout t crlf crlf)
  (printout t 
" Have you tried talking to experienced AI programmers in order to improve " crlf
" your skills?" crlf crlf	
"                 1.) yes " crlf
"                 2.) no " crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response3-7 (read))
(printout t crlf crlf)
(assert (completed Q3-7))
(assert (answer Q3-7 ?response3-7))
(assert (answer-check ?response3-7 Q3-7 3))
(focus ERROR))

;;If question 3-7 is completed ask question 3-8

(defrule PROGRAMMING::question3-8
  (declare (salience 50))
  (completed Q3-7)
=>
  (printout t crlf crlf)
  (printout t
"  Have you tried reading other people's code? " crlf crlf
"                 1.) yes " crlf
"                 2.) no " crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response3-8 (read))
(printout t crlf crlf)
(assert (answer Q3-8 ?response3-8))
(assert (answer-check ?response3-8 Q3-8 3))
(focus ERROR))

;;If the answer to question 3-7 is 2 give advice 3-6

(defrule PROGRAMMING::advice3-6
  (declare (salience 100))
  (answer Q3-7 2)
=>
  (assert (advice 3-6))
  (focus ADVICE))

;;If the answer to question 3-8 is 2 give advice 3-7

(defrule PROGRAMMING::advice3-7
  (declare (salience 100))
  (answer Q3-8 2)
=>
  (assert (advice 3-7))
  (focus ADVICE))


;;If question 3-9 is completed ask question 3-10

(defrule PROGRAMMING::question3-10
  (completed Q3-9)
=>
  (printout t crlf crlf)
  (printout t 
"  Do you put a lot of comments in your code? " crlf crlf	
"                 1.) yes, I do " crlf
"                 2.) yes, but not too many, only to help me remember what " crlf
"                     I've done " crlf
"                 3.) not from the beginning, I will do it when my program works" crlf
"                 4.) no, I always remember what I've done " crlf 
"                 5.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response3-10 (read))
(printout t crlf crlf)
(assert (completed Q3-10))
(assert (answer Q3-10 ?response3-10))
(assert (answer-check ?response3-10 Q3-10 5))
(focus ERROR))


;;If question 3-10 is completed ask question 3-11

(defrule PROGRAMMING::question3-11
  (completed Q3-10)
=>
  (printout t crlf crlf)
  (printout t 
"  Do you find that the existing languages do not offer exactly what you need " crlf
"  for your project? " crlf crlf	
"                 1.) yes " crlf
"                 2.) no " crlf 
"                 3.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response3-11 (read))
(printout t crlf crlf)
(assert (completed Q3-11))
(assert (answer Q3-11 ?response3-11))
(assert (answer-check ?response3-11 Q3-11 3))
(focus ERROR))

;;If the answer to question 3-9 is 1 give advice 3-8

(defrule PROGRAMMING::advice3-8
  (declare (salience 50))
  (answer Q3-9 1)
=>
  (assert (advice 3-8))
  (focus ADVICE))

;;If the answer to question 3-10 is 2 give advice 3-9

(defrule PROGRAMMING::advice3-9
  (declare (salience 50))
  (answer Q3-10 2)
=>
  (assert (advice 3-9))
  (focus ADVICE))

;;If the answer to question 3-10 is 3 give advice 3-10

(defrule PROGRAMMING::advice3-10
  (declare (salience 50))
  (answer Q3-10 3)
=>
  (assert (advice 3-10))
  (focus ADVICE))

;;If the answer to question 3-10 is 4 give advice 3-11

(defrule PROGRAMMING::advice3-11
  (declare (salience 50))
  (answer Q3-10 4)
=>
  (assert (advice 3-11))
  (focus ADVICE))

;;If the answer to question 3-11 is 1 ask question 3-12

(defrule PROGRAMMING::question3-12
  (declare (salience 50))
  (answer Q3-11 1)
=>
  (printout t crlf crlf)
  (printout t 
" What do you do? Do you..."crlf crlf	
"                 1.) leave out what you cannot do and explain in your project " crlf
"                     why" crlf
"                 2.) find one language and add to it what you need" crlf 
"                 3.) write a useful, new language " crlf
"                 4.) return to the main menu " crlf crlf 
"      Choice: " )
(bind ?response3-12 (read))
(printout t crlf crlf)
(assert (answer-check ?response3-12 Q3-12 4))
(assert (answer Q3-12 ?response3-12))
(focus ERROR))


;;If the answer to question 3-12 is 3 give advice 3-12

(defrule PROGRAMMING::advice3-12
  (declare (salience 50))
  (answer Q3-12 3)
=>
  (assert (advice 3-12))
  (focus ADVICE))

;;If the answer to question 3-12 is 1 give advice 3-13

(defrule PROGRAMMING::advice3-13
  (declare (salience 50))
  (answer Q3-12 1)
=>
  (assert (advice 3-13))
  (focus ADVICE))


;;If the answers to all questions show that there is no problem 
;;give advice 3-14

(defrule PROGRAMMING::advice3-14
  (declare (salience 30))
  (and (answer Q3-1 2)
       (or (answer Q3-2 2)
           (answer Q3-3 2))
       (answer Q3-5 1)
       (or (answer Q3-6 1)
           (and (answer Q3-6 2)
                (answer Q3-7 1)
                (answer Q3-8 1)))
       (answer Q3-9 2)
       (answer Q3-10 1)
       (or (answer Q3-11 2)
           (and (answer Q3-11 1)
                (answer Q3-12 2))
           (and (answer Q3-11 1)
                (answer Q3-12 1))))
=>
  (assert (advice 3-14))
  (focus ADVICE))

;;If the question 3-11 is completed ask the user if he wants to return to the 
;;main menu or exit the program

(defrule PROGRAMMING::general-question
  (completed Q3-11)
=>
  (assert (question general))
  (focus ADVICE))

;;***********************************
;;***********************************
;;        EVALUATION MODULE        **
;;***********************************
;;***********************************

(defrule EVALUATION::question4-1
  (answer topics 4)
=>
  (printout t crlf crlf
  " What is the area/field of your research? ")
  (bind ?field (readline))
  (assert (field ?field))
  (assert (completed Q4-1)))

(defrule EVALUATION::question4-2
  (completed Q4-1)
=>
  (printout t crlf crlf
" Does your research work... " crlf
" (If you think that more than one of the following applies to your research " crlf
" put all your choices separated by a space and then press return) " crlf crlf
"                 1.) describe a new technique? " crlf
"                 2.) extend or improve an existing technique? " crlf
"                 3.) establish properties of a technique or relations between "crlf
"                     two or more techniques either empirically or theoretically? " crlf
"                 4.) describe a new application of a technique? " crlf
"                 5.) test the psychological validity of a technique? " crlf
"                 6.) combine several techniques into a system? " crlf
"                 7.) identify and motivate a new problem? " crlf
"                 8.) serve a tutorial role, e.g. a survey? " crlf
"                 9.) do none of the above? " crlf 
"                10.) return to the main menu. " crlf crlf
"          Choice(s): " )
(assert (completed Q4-2))
(bind ?response4-2 (explode$ (readline)))
(while (= (length$ ?response4-2) 0)
 do (printout t "You have to make a choice: ")
     (bind ?response4-2 (explode$ (readline))))
(while (<> (length$ ?response4-2) 0)
 do
(assert (answer Q4-2 (first$ ?response4-2)))
(assert (multi-answer-check (first$ ?response4-2) Q4-2 10))
(bind ?temp ?response4-2)
(bind ?response4-2 (rest$ ?temp))
(focus ERROR)))

(defrule EVALUATION::system-involved
   (answer Q4-2 6)
=>
   (assert (answer Q4-5 1)))
 
(defrule EVALUATION::question4-3a
   (answer Q4-2 9)
   (not (answer Q4-2 1|2|3|4|5|6|7|8))
 =>
(printout t crlf crlf
 " State what is the content of your research and press return: " )
(bind ?content (readline))
(printout t crlf crlf
" Sorry, but I cannot help you with \"" ?content "\", I don't have that " crlf
" functionality yet. Make sure that \""?content"\" doesn't fall into one " crlf
" of the above categories. Sometimes, when you examine it more carefully " crlf
" you find out that it does. " crlf crlf)
(assert (question end))
(focus ADVICE))

(defrule EVALUATION::question4-3b
   (declare (salience 50))
   ?ans-9 <-(answer Q4-2 9)
   (exists (answer Q4-2 1|2|3|4|5|6|7|8))
=>
(printout t crlf crlf
 " The 9 option will be ignored as you've already made some choices. " crlf)
 (retract ?ans-9))

(defrule EVALUATION::question4-3c
   (answer Q4-2 5|7|8)
   (not (answer Q4-2 1|2|3|4|6))
=>
(printout t crlf crlf
 " Sorry but the system cannot help you. It is beyond the system's " crlf
 " functionality to advise you on evaluation methods, with this content " crlf
 " of research. " crlf crlf)
(assert (question end))
(focus ADVICE))

(defrule EVALUATION::question4-3d
   (exists (answer Q4-2 5|7|8)
           (answer Q4-2 1|2|3|4|6))
 =>
  (printout t crlf crlf
    " Sorry but the system cannot help you with: "crlf)
  (assert (completed Q4-3d)))


(defrule EVALUATION::question4-3e
   (declare (salience 50))
   (completed Q4-3d)
   (answer Q4-2 ?answer&5|7|8)
  =>
 (if (= ?answer 5)
  then
 (printout t  
 "- Testing the psychological validity of your technique" crlf))
 (if (= ?answer 7)
  then
 (printout t
 "- Identify and motivate a new problem " crlf))
 (if (= ?answer 8)
 then
 (printout t
 "- Serving a tutorial role, e.g. a survey" crlf))
 (assert (completed Q4-3e)))

 (defrule EVALUATION::question4-3f
  (completed Q4-3e) 
 =>
 (printout t crlf 
  " It is beyond the system's functionality. " crlf
  " You will though be given advice for the rest. " crlf crlf))

(defrule EVALUATION::question4-4
  (completed Q4-2)
=>
  (printout t crlf crlf
  " Does the technique mentioned above involve some sort of knowledge " crlf
  " representation? " crlf crlf 
  "               1.)Yes. " crlf
  "               2.)No. " crlf
  "               3.)Return to the main menu. " crlf crlf
  "      Choice: " )
  (bind ?response4-4 (read))
  (printout t crlf crlf)
  (assert (answer-check ?response4-4 Q4-4 3))
  (assert (answer Q4-4 ?response4-4))
  (assert (completed Q4-4))
  (focus ERROR))

 (defrule EVALUATION::question4-5
   (completed Q4-4)
   (not (answer Q4-2 6))
  =>
  (printout t crlf crlf
   " The technique ... " crlf crlf
   "              1.)is applied in a \"system\"? " crlf
   "              2.)exists only in theory? " crlf
   "              3.)Return to the main menu. " crlf crlf
   "     Choice: " )
   (bind ?response4-5 (read))
   (printout t crlf crlf)
   (assert (answer-check ?response4-5 Q4-5 3))
   (assert (answer Q4-5 ?response4-5))
   (focus ERROR))

   
  (defrule EVALUATION::question4-6a
   (answer Q4-5 2)
  =>
  (printout t crlf crlf
   " Does the theory involve ... " crlf crlf
   "              1.)an algorithm or combination of several algorithms? " crlf
   "              2.)a formal model of the technique using mathematical tools " crlf
   "                 or other? " crlf
   "              3.)all of the above? " crlf
   "              4.)nothing of the above? " crlf
   "              5.)Return to the main menu. " crlf crlf
   "     Choice: " )
   (bind ?response4-6 (read))
   (printout t crlf crlf)
   (assert (answer-check ?response4-6 Q4-6 5))
   (assert (answer Q4-6 ?response4-6))
   (assert (completed Q4-6))
   (focus ERROR))

   (defrule EVALUATION::question4-6b
     (declare (salience 50))
     (answer Q4-6 3)
   =>
     (assert (answer Q4-6 2))
     (assert (answer Q4-6 1)))

 (defrule EVALUATION::question4-6c
     (declare (salience 50))
     (answer Q4-6 4)
   =>
     (assert (advice 4-36))
     (assert (question end))
     (focus ADVICE))

  (defrule EVALUATION::question4-7
      (completed Q4-6)
      (not (answer Q4-2 2))
   =>
       (printout t crlf crlf
" In your research hypothesis you claim that ... " crlf
" (If you think that more than one of the following applies to your research " crlf
" put all your choices separated by a space and then press return) " crlf crlf
"                 1.) this technique is good for doing task X? " crlf
"                 2.) this technique is better than a rival technique for " crlf
"                     doing task X? " crlf
"                 3.) some behavioural property or the efficiency of the " crlf
"                     technique depends on the range of inputs? " crlf
"                 4.) return to the main menu. " crlf crlf
"          Choice(s): " )
(assert (completed Q4-7))
(bind ?response4-7 (explode$ (readline)))
(while (= (length$ ?response4-7) 0)
 do (printout t "You have to make a choice: ")
     (bind ?response4-7 (explode$ (readline))))
(while (<> (length$ ?response4-7) 0)
 do
(assert (answer Q4-7 (first$ ?response4-7)))
(assert (multi-answer-check (first$ ?response4-7) Q4-7 4))
(bind ?temp ?response4-7)
(bind ?response4-7 (rest$ ?temp))
(focus ERROR)))


(defrule EVALUATION::question4-8
  (completed Q4-4)
  (answer Q4-5 1)
  =>
  (printout t crlf crlf
   " By the word \"system\" you mean ..." crlf crlf
   "              1.)a system that interacts with a human?" crlf
   "              2.)an architecture or a system that performs a task " crlf
   "                 but does not interact with a human? " crlf
   "              3.)Return to the main menu. " crlf crlf
   "     Choice: " )
   (assert (completed Q4-8))
   (bind ?response4-8 (read))
   (printout t crlf crlf)
   (assert (answer-check ?response4-8 Q4-8 3))
   (assert (answer Q4-8 ?response4-8))
   (assert (completed Q4-8))
   (focus ERROR))


(defrule EVALUATION::question4-9
  (completed Q4-8)
 =>
  (printout t crlf crlf
   " Does the \"system\" involve some sort of programming? " crlf crlf
   "              1.)yes " crlf
   "              2.)no " crlf
   "              3.)Return to the main menu. " crlf crlf
   "     Choice: " )
   (bind ?response4-9 (read))
   (printout t crlf crlf)
   (assert (answer-check ?response4-9 Q4-9 3))
   (assert (answer Q4-9 ?response4-9))
   (assert (completed Q4-9))
   (focus ERROR))


(defrule EVALUATION::question4-10
  (completed Q4-9)
  (not (answer Q4-2 2))
 =>
  (printout t crlf 
   " In your research hypothesis do you claim that  ... " crlf 
   " (If you think that more than one of the following applies to your research " crlf
   " put all your choices separated by a space and then press return) " crlf crlf
   "              1.)the \"system\" is good at doing task Z? " crlf
   "              2.)the \"system\" is better than rival \"system\" Y at doing task Z?" crlf
   "              3.)some behavioural property or the efficiency of the \"system\" " crlf
   "                 depends on the range of inputs? " crlf
   "              4.)Return to the main menu. " crlf crlf
   "     Choice(s): " )
   (assert (completed Q4-10))
   (bind ?response4-10 (explode$ (readline)))
   (while (= (length$ ?response4-10) 0)
   do (printout t "You have to make a choice: ")
      (bind ?response4-10 (explode$ (readline))))
   (while (<> (length$ ?response4-10) 0)
    do
   (assert (answer Q4-10 (first$ ?response4-10)))
   (assert (multi-answer-check (first$ ?response4-10) Q4-10 4))
   (bind ?temp ?response4-10)
   (bind ?response4-10 (rest$ ?temp))
   (focus ERROR)))

(defrule EVALUATION::question4-11
  (or (completed Q4-6)
      (completed Q4-9))
  (answer Q4-2 2)
 =>
  (printout t crlf crlf
   " The improvement/extending of the technique involves ... " crlf crlf
   "              1.)changing only one parameter? " crlf
   "              2.)changing a lot of parameters/aspects of the technique? " crlf
   "              3.)Return to the main menu. " crlf crlf
   "     Choice: " )
   (assert (completed Q4-11))
   (bind ?response4-11 (read))
   (printout t crlf crlf)
   (assert (answer-check ?response4-11 Q4-11 3))
   (assert (answer Q4-11 ?response4-11))
   (focus ERROR))

(defrule EVALUATION::question4-12
   (completed Q4-11)
 =>
  (printout t crlf crlf
   " When you claim that you've improved/extended an existing technique " crlf
   " you have to prove that the technique has really been improved/extended, " crlf
   " i.e. the technique is now better than it used to be before at doing " crlf
   " task Z/the technique now has a wider range of applications than before." crlf 
   " Do you want to get advice about this? " crlf crlf
   "              1.)yes " crlf
   "              2.)no " crlf
   "              3.)Return to the main menu. " crlf crlf
   "     Choice: " )
   (bind ?response4-12 (read))
   (printout t crlf crlf)
   (assert (answer-check ?response4-12 Q4-12 3))
   (assert (answer Q4-12 ?response4-12))
   (focus ERROR))


(defrule EVALUATION::question4-13
   (answer Q4-12 1)
 =>
  (printout t crlf crlf
   " Have you... " crlf crlf
   "              1.)generally improved the existing technique (maybe " crlf
   "                 by extending it as well)? " crlf
   "              2.)just extended the existing technique to cover a " crlf
   "                 wider range of applications? " crlf
   "              3.)Return to the main menu. " crlf crlf
   "     Choice: " )
   (bind ?response4-13 (read))
   (printout t crlf crlf)
   (assert (answer-check ?response4-13 Q4-13 3))
   (assert (answer Q4-13 ?response4-13))
   (focus ERROR))

(defrule EVALUATION::question4-14
   (answer Q4-13 1)
 =>
  (printout t crlf crlf
   " In which dimension(s) do you claim that you have improved the technique? " crlf
   " (If you think that more than one of the following applies to your research " crlf
   " put all your choices separated by a space and then press return) " crlf crlf
   "              1.) Behaviour (e.g. quality, correctness) " crlf
   "              2.) Efficiency (e.g. of time, space, etc) " crlf
   "              3.) Coverage (e.g. range of application) " crlf
   "              4.) Return to the main menu. " crlf crlf
   "     Choice(s): " )
   (assert (completed Q4-14))
   (bind ?response4-14 (explode$ (readline)))
   (while (= (length$ ?response4-14) 0)
   do (printout t "You have to make a choice: ")
      (bind ?response4-14 (explode$ (readline))))
   (while (<> (length$ ?response4-14) 0)
    do
   (assert (answer Q4-14 (first$ ?response4-14)))
   (assert (multi-answer-check (first$ ?response4-14) Q4-14 4))
   (bind ?temp ?response4-14)
   (bind ?response4-14 (rest$ ?temp))
   (focus ERROR)))

(defrule EVALUATION::question4-15a
   (answer Q4-13 2)
 =>
  (printout t crlf crlf
   " When you claim that you have just extended an existing technique "crlf
   " you have to prove that the extended technique covers a wider range of " crlf
   " applications than the existing technique. " crlf
   " Do you want to get advice about this? " crlf crlf
   "              1.)yes " crlf
   "              2.)no " crlf
   "              3.)Return to the main menu. " crlf crlf
   "     Choice: " )
   (assert (completed Q4-15))
   (bind ?response4-15 (read))
   (printout t crlf crlf)
   (assert (answer-check ?response4-15 Q4-15 3))
   (assert (answer Q4-15 ?response4-15))
   (focus ERROR))

(defrule EVALUATION::question4-15b
  (answer Q4-15 1)
=>
  (assert (answer Q4-14 3)))

 
(defrule EVALUATION::question4-16
   (or (answer Q4-12 2)
       (completed Q4-15)
       (completed Q4-14))
     (answer Q4-5 2)
   =>
  (printout t crlf crlf
   " Do you also want to do some other test? " crlf 
   " (If you think that more than one of the following applies to your research " crlf
   " put all your choices separated by a space and then press return) " crlf crlf
   "              1.)yes, to establish some properties of the new version of " crlf
   "                 the technique independently of the existing technique. " crlf
   "              2.)yes, to compare the technique with some rival. " crlf
   "              3.)yes, to test how some behavioural property or the " crlf
   "                 efficiency of the technique depends on the range of inputs. " crlf
   "              4.)no, nothing else. " crlf
   "              5.)Return to the main menu. " crlf crlf
   "     Choice(s): " )
   (assert (completed Q4-16))
  (bind ?response4-16 (explode$ (readline)))
   (while (= (length$ ?response4-16) 0)
   do (printout t "You have to make a choice: ")
      (bind ?response4-16 (explode$ (readline))))
   (while (<> (length$ ?response4-16) 0)
    do
   (assert (answer Q4-16 (first$ ?response4-16)))
   (assert (multi-answer-check (first$ ?response4-16) Q4-16 5))
   (bind ?temp ?response4-16)
   (bind ?response4-16 (rest$ ?temp))
   (focus ERROR)))

(defrule EVALUATION::question4-17a
   (or (answer Q4-12 2)
       (completed Q4-15)
       (completed Q4-14))
   (answer Q4-5 1)
 =>
  (printout t crlf crlf
   " Do you also want to do some other test? " crlf 
   " (If you think that more than one of the following applies to your research " crlf
   " put all your choices separated by a space and then press return) " crlf crlf
   "              1.)yes, to establish some properties of the \"system\"." crlf
   "              2.)yes, to compare the \"system\" with some rival. " crlf
   "              3.)yes, to test how some behavioural property or the " crlf
   "                 efficiency of the \"system\" depends on the range of inputs. " crlf
   "              4.)no, nothing else. " crlf
   "              5.)Return to the main menu. " crlf crlf
   "     Choice(s): " )
   (assert (completed Q4-17))
   (bind ?response4-17 (explode$ (readline)))
   (while (= (length$ ?response4-17) 0)
   do (printout t "You have to make a choice: ")
      (bind ?response4-17 (explode$ (readline))))
   (while (<> (length$ ?response4-17) 0)
    do
   (assert (answer Q4-17 (first$ ?response4-17)))
   (assert (multi-answer-check (first$ ?response4-17) Q4-17 5))
   (bind ?temp ?response4-17)
   (bind ?response4-17 (rest$ ?temp))
   (focus ERROR)))

(defrule EVALUATION::question4-17b
  (declare (salience 100))
  (answer Q4-16|Q4-17 4)
  (not (exists (answer Q4-16|Q4-17 1|2|3)))
=>
(assert (question end)))

(defrule EVALUATION::question4-17c
  (declare (salience 100))
  (answer Q4-16|Q4-17 4)
  (exists (answer Q4-16|Q4-17 1|2|3))
=>
  (printout t crlf crlf
  " Answer 4 is ignored as you have made some other choices " crlf crlf))

(defrule EVALUATION::question4-18
   (or (answer Q4-16 1)
       (answer Q4-7 1))   
 =>
  (printout t crlf crlf
   " For establishing properties of your technique ... " crlf
   " What properties of your technique do you want to establish? " crlf
   " (If you think that more than one of the following applies to your research " crlf
   " put all your choices separated by a space and then press return) " crlf crlf
   "              1.)Behaviour (e.g. quality, correctness) " crlf
   "              2.)Efficiency (e.g. of time, space, etc) " crlf
   "              3.)Coverage (e.g. range of application) " crlf
   "              4.)Return to the main menu. " crlf crlf
   "     Choice(s): " )
   (assert (completed Q4-18))
   (bind ?response4-18 (explode$ (readline)))
   (while (= (length$ ?response4-18) 0)
   do (printout t "You have to make a choice: ")
      (bind ?response4-18 (explode$ (readline))))
   (while (<> (length$ ?response4-18) 0)
    do
   (assert (answer Q4-18 (first$ ?response4-18)))
   (assert (multi-answer-check (first$ ?response4-18) Q4-18 4))
   (bind ?temp ?response4-18)
   (bind ?response4-18 (rest$ ?temp))
   (focus ERROR)))

(defrule EVALUATION::question4-19
   (or (answer Q4-16 2)
       (answer Q4-7 2))   
 =>
  (printout t crlf crlf
   " For comparing your technique with rivals ... " crlf
   " In what sort of dimension are you comparing the techniques?" crlf
   " (If you think that more than one of the following applies to your research " crlf
   " put all your choices separated by a space and then press return) " crlf crlf
   "              1.)Behaviour (e.g. quality, correctness) " crlf
   "              2.)Efficiency (e.g. of time, space, etc) " crlf
   "              3.)Coverage (e.g. range of application) " crlf
   "              4.)Return to the main menu. " crlf crlf
   "     Choice(s): " )
   (assert (completed Q4-19))
   (bind ?response4-19 (explode$ (readline)))
   (while (= (length$ ?response4-19) 0)
   do (printout t "You have to make a choice: ")
      (bind ?response4-19 (explode$ (readline))))
   (while (<> (length$ ?response4-19) 0)
    do
   (assert (answer Q4-19 (first$ ?response4-19)))
   (assert (multi-answer-check (first$ ?response4-19) Q4-19 4))
   (bind ?temp ?response4-19)
   (bind ?response4-19 (rest$ ?temp))
   (focus ERROR)))

(defrule EVALUATION::question4-20
   (or (answer Q4-17 1)
       (answer Q4-10 1))   
 =>
  (printout t crlf crlf
   " For establishing properties of the system ... " crlf
   " What properties of your \"system\" do you want to establish? " crlf
   " (If you think that more than one of the following applies to your research " crlf
   " put all your choices separated by a space and then press return) " crlf crlf
   "              1.)Behaviour (e.g. quality, correctness) " crlf
   "              2.)Efficiency (e.g. of time, space, etc) " crlf
   "              3.)Coverage (e.g. range of application) " crlf
   "              4.)Return to the main menu. " crlf crlf
   "     Choice(s): " )
   (assert (completed Q4-20))
   (bind ?response4-20 (explode$ (readline)))
   (while (= (length$ ?response4-20) 0)
   do (printout t "You have to make a choice: ")
      (bind ?response4-20 (explode$ (readline))))
   (while (<> (length$ ?response4-20) 0)
    do
   (assert (answer Q4-20 (first$ ?response4-20)))
   (assert (multi-answer-check (first$ ?response4-20) Q4-20 4))
   (bind ?temp ?response4-20)
   (bind ?response4-20 (rest$ ?temp))
   (focus ERROR)))

(defrule EVALUATION::question4-21
   (or (answer Q4-17 2)
       (answer Q4-10 2))   
 =>
  (printout t crlf crlf
   " For comparing your system with rivals ... " crlf
   " In what sort of dimension are you comparing the \"systems\"?" crlf
   " (If you think that more than one of the following applies to your research " crlf
   " put all your choices separated by a space and then press return) " crlf crlf
   "              1.)Behaviour (e.g. quality, correctness) " crlf
   "              2.)Efficiency (e.g. of time, space, etc) " crlf
   "              3.)Coverage (e.g. range of application) " crlf
   "              4.)Return to the main menu. " crlf crlf
   "     Choice(s): " )
   (assert (completed Q4-21))
   (bind ?response4-21 (explode$ (readline)))
   (while (= (length$ ?response4-21) 0)
   do (printout t "You have to make a choice: ")
      (bind ?response4-21 (explode$ (readline))))
   (while (<> (length$ ?response4-21) 0)
    do
   (assert (answer Q4-21 (first$ ?response4-21)))
   (assert (multi-answer-check (first$ ?response4-21) Q4-21 4))
   (bind ?temp ?response4-21)
   (bind ?response4-21 (rest$ ?temp))
   (focus ERROR)))

(defrule EVALUATION::question4-22
   (declare (salience 100)) 
   (or (completed ?which-question&Q4-14|Q4-18|Q4-19|Q4-20|Q4-21|state-range)
       (completed Q4-25 ?which-question)
       (answer ?which-question&Q4-15 1))
  =>
  (printout t crlf crlf
   " Do you want to do ... " crlf crlf
   "              1.)a theoretical evaluation? " crlf
   "              2.)an experimental evaluation?" crlf
   "              3.)Return to the main menu. " crlf crlf
   "     Choice: " )
   (assert (completed Q4-22))
   (bind ?response4-22 (read))
   (printout t crlf crlf)
   (assert (answer-check ?response4-22 Q4-22 3))
   (assert (answer Q4-22 ?response4-22 ?which-question))
   (focus ERROR))

(defrule EVALUATION::advice4-37
   (declare (salience 150))
   (answer Q4-5 2) 
   (answer Q4-22 2 Q4-18)
   =>
   (assert (advice 4-37))
   (assert (question Q4-32 Q4-18))
   (focus ADVICE))

(defrule EVALUATION::advice4-44a
   (declare (salience 150))
   (answer Q4-5 2) 
   (answer Q4-22 2 Q4-19)
=>
   (assert (advice 4-44))
   (assert (question Q4-32 Q4-19))
   (focus ADVICE))

(defrule EVALUATION::advice4-44b
   (declare (salience 150))
   (answer Q4-5 2)
   (completed advice4-51|advice4-52) 
   (answer Q4-22 2 Q4-14|Q4-15)
=>
   (assert (advice 4-44))
   (assert (question Q4-32 Q4-14))
   (focus ADVICE))


(defrule EVALUATION::question4-32a
   (declare (salience 150)) 
   ?q <- (question Q4-32 ?which-question&Q4-18|Q4-19|Q4-14|Q4-15|state-range)
   =>
  (printout t crlf crlf
   " Do you want me to give you advice about theoretical evaluation? " crlf crlf
   "              1.)Yes" crlf
   "              2.)No" crlf
   "              3.)Return to the main menu. " crlf crlf
   "     Choice: " )
   (retract ?q)
   (bind ?response4-32 (read))
   (printout t crlf crlf)
   (assert (answer-check ?response4-32 Q4-32 3))
   (assert (answer Q4-32 ?response4-32 ?which-question))
   (focus ERROR))

(defrule EVALUATION::sources
   (declare (salience 100))
   (completed Q4-22)
=>
   (assert (advice sources))
   (focus ADVICE))

(defrule EVALUATION::question4-23
   (answer Q4-4 1)
   (completed Q4-18|Q4-20)
=>
   (printout t crlf crlf
  "As there is also some sort of knowledge represented in your technique " crlf
  "you need to evaluate the knowledge as well. " crlf
  "Do you want to get advice on that? " crlf crlf
  "              1.)yes " crlf
  "              2.)no " crlf
  "              3.)Return to the main menu. " crlf crlf
  "     Choice: " )
  (bind ?response4-23 (read))
  (assert (answer-check ?response4-23 Q4-23 3))
  (assert (answer Q4-23 ?response4-23))
  (focus ERROR))

(defrule EVALUATION::question4-24
   (answer Q4-4 1)
   (completed Q4-19|Q4-21)
=>
   (printout t crlf crlf
  "As there is also some sort of knowledge represented in your technique " crlf
  "you need to compare the knowledge represented as well in order to compare " crlf
  "it with its rivals. " crlf
  "Do you want to get advice on that? " crlf crlf
  "              1.)yes " crlf
  "              2.)no " crlf
  "              3.)Return to the main menu. " crlf crlf
  "     Choice: " )
  (bind ?response4-24 (read))
  (assert (answer-check ?response4-24 Q4-24 3))
  (assert (answer Q4-24 ?response4-24))
  (focus ERROR))

  (defrule EVALUATION::question4-25a
   (answer ?knowledge&Q4-24|Q4-23 1)
 =>
   (printout t crlf crlf
  "Do you want to evaluate the knowledge in terms of ... " crlf crlf
  "              1.)Behaviour (e.g. if the knowledge is correct, consistent etc)? "crlf
  "              2.)Coverage (e.g. range of application)? " crlf
  "              3.)Both? " crlf
  "              4.)Return to the main menu. " crlf crlf
  "     Choice: " )
  (assert (completed Q4-25 ?knowledge))
  (bind ?response4-25 (read))
  (assert (answer-check ?response4-25 Q4-25 4))
  (assert (answer Q4-25 ?response4-25 ?knowledge))
  (focus ERROR))

(defrule EVALUATION::question4-25b
  (declare (salience 100))
  (answer Q4-25 3 ?knowledge)
=>
  (assert (answer Q4-25 1 ?knowledge))
  (assert (answer Q4-25 2 ?knowledge)))

(defrule EVALUATION::advice4-1
 (declare (salience 50)) 
   (answer Q4-8 1)
   (answer Q4-20 1)
 =>
  (assert (advice 4-1))
  (assert (completed advice4-1))
  (focus ADVICE))

(defrule EVALUATION::question4-26
 (declare (salience 100))
   (answer Q4-22 2 Q4-20)  
   (completed advice4-1)
  =>
  (printout t crlf crlf
  "Do you want to do a user-centered evaluation? " crlf crlf
  "              1.)yes " crlf
  "              2.)no " crlf
  "              3.)Return to the main menu. " crlf crlf
  "     Choice: " )
  (bind ?response4-26 (read))
  (assert (answer-check ?response4-26 Q4-26 3))
  (assert (answer Q4-26 ?response4-26))
  (focus ERROR))

 (defrule EVALUATION::advice4-2
 (declare (salience 100)) 
   (answer Q4-26 2)
 =>
  (assert (advice 4-2))
  (focus ADVICE))

  (defrule EVALUATION::question4-27
  (declare (salience 100)) 
   (answer Q4-26 1)
  =>
  (printout t crlf crlf
  "Is the evaluation early on the design stage when there is no " crlf
  "runnable system? " crlf crlf
  "              1.)yes " crlf
  "              2.)no " crlf
  "              3.)Return to the main menu. " crlf crlf
  "     Choice: " )
  (bind ?response4-27 (read))
  (assert (answer-check ?response4-27 Q4-27 3))
  (assert (answer Q4-27 ?response4-27))
  (focus ERROR))

  (defrule EVALUATION::advice4-3
  (declare (salience 100)) 
   (answer Q4-27 1)
 =>
  (assert (advice 4-3))
  (focus ADVICE))

   (defrule EVALUATION::question4-28
  (declare (salience 100)) 
   (answer Q4-27 2)
  =>
  (printout t crlf crlf
  "Which approach of user-centered evaluation do you want to take? " crlf crlf
  "              1.) Observational methods, which gather information about " crlf
  "                  the system by observing the user interacting with it? " crlf
  "              2.) Query techniques, which get information by asking the " crlf
  "                  user directly? " crlf
  "              3.) Conduct an experiment? " crlf
  "              4.) Return to the main menu. " crlf crlf
  "     Choice: " )
  (bind ?response4-28 (read))
  (assert (answer-check ?response4-28 Q4-28 4))
  (assert (answer Q4-28 ?response4-28))
  (focus ERROR))


(defrule EVALUATION::advice4-4
 (declare (salience 100)) 
   (answer Q4-28 3)
 =>
  (assert (advice 4-4))
  (focus ADVICE))

(defrule EVALUATION::advice4-5
 (declare (salience 100)) 
   (answer Q4-28 1)
 =>
  (assert (advice 4-5))
  (focus ADVICE))

(defrule EVALUATION::advice4-6
 (declare (salience 100))    
 (answer Q4-28 2)
 =>
  (assert (advice 4-6))
  (focus ADVICE))

(defrule EVALUATION::advice4-7
 (declare (salience 100))
   (answer Q4-22 1 Q4-20)  
   (completed advice4-1)
  =>
   (assert (question Q4-29))
   (assert (question Q4-30))
   (assert (question Q4-31))
   (assert (advice 4-7))
  (focus ADVICE))


(defrule EVALUATION::note
 (declare (salience -50))
  (exists (question Q4-29|Q4-30|Q4-31))
=>
(printout t crlf crlf
 " Things to note throughout the evaluation: " crlf crlf)
 (assert (completed note)))
 
(defrule EVALUATION::question4-29
 (declare (salience -50))
 (question Q4-29)
 (completed note)
=>
 (printout t crlf crlf 
 " Theoretical analysis nearly always requires some simplifying " crlf
 " assumptions. If these assumptions are false then the theoretical " crlf
 " results may be invalid. " crlf
 " Do you want me to give you ways of validating any model you've used " crlf
 " for theoretical evaluation, i.e check if the assumptions you made " crlf
 " are correct? " crlf crlf
 "              1.)yes " crlf
 "              2.)no " crlf
 "              3.)Return to the main menu. " crlf crlf
 "     Choice: " )
  (bind ?response4-29 (read))
  (assert (answer-check ?response4-29 Q4-29 3))
  (assert (answer Q4-29 ?response4-29))
  (focus ERROR))

(defrule EVALUATION::question4-30
  (declare (salience -50))
  (question Q4-30)
  (completed note)
=>
 (printout t crlf crlf 
 " It is important to test that any model you use, implements the assumptions " crlf
 " you've made correctly, i.e. does what it is intended to do. " crlf
 " Do you want me to give you ways of verifying your model ,i.e." crlf
 " check if the model does what is intended to do? " crlf crlf
 "              1.)yes " crlf
 "              2.)no " crlf
 "              3.)Return to the main menu. " crlf crlf
 "     Choice: " )
  (bind ?response4-30 (read))
  (assert (answer-check ?response4-30 Q4-30 3))
  (assert (answer Q4-30 ?response4-30))
  (focus ERROR))

(defrule EVALUATION::question4-31
 (declare (salience -50))
 (question Q4-31)
 (completed note)
=>
 (printout t crlf crlf 
 " It is important that any test examples you use are representative. " crlf
 " Do you want to get advice about ways of obtaining such representative " crlf
 " sets of examples? " crlf crlf
 "              1.)yes " crlf
 "              2.)no " crlf
 "              3.)Return to the main menu. " crlf crlf
 "     Choice: " )
  (bind ?response4-31 (read))
  (assert (answer-check ?response4-31 Q4-31 3))
  (assert (answer Q4-31 ?response4-31))
  (focus ERROR))

(defrule EVALUATION::advice4-8
  (answer Q4-29 1)
 =>
  (assert (advice 4-8))
  (focus ADVICE))

(defrule EVALUATION::advice4-9
 (answer Q4-30 1)
 =>
  (assert (advice 4-9))
  (focus ADVICE))

(defrule EVALUATION::advice4-10
  (answer Q4-31 1)
 =>
  (assert (advice 4-10))
  (focus ADVICE))

(defrule EVALUATION::advice4-11
 (declare (salience 50)) 
  (answer Q4-20 2)
  (answer Q4-22 2 Q4-20)
 =>
  (assert (advice 4-11))
  (assert (question Q4-31))
  (focus ADVICE))

(defrule EVALUATION::advice4-12
 (declare (salience 50)) 
  (answer Q4-20 2)
  (answer Q4-22 1 Q4-20)
  (answer Q4-9 1)
 =>
  (assert (advice 4-12))
  (focus ADVICE))

(defrule EVALUATION::advice4-13
 (declare (salience 50)) 
  (answer Q4-20 2)
  (answer Q4-22 1 Q4-20)
  (answer Q4-9 2)
 =>
  (assert (advice 4-13))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (focus ADVICE))

(defrule EVALUATION::advice4-14
 (declare (salience 50)) 
  (answer Q4-20 3)
 =>
  (assert (advice 4-14))
  (assert (completed advice4-14))
 (focus ADVICE))

(defrule EVALUATION::advice4-15
 (declare (salience 100))
   (answer Q4-22 2 Q4-20)  
   (completed advice4-14)
  =>
   (assert (question Q4-31))
   (assert (advice 4-15))
  (focus ADVICE))

(defrule EVALUATION::advice4-16
 (declare (salience 100))
   (answer Q4-22 1 Q4-20)  
   (completed advice4-14)
  =>
   (assert (question Q4-29))
   (assert (question Q4-30))
   (assert (question Q4-31))
   (assert (advice 4-16))
  (focus ADVICE))

(defrule EVALUATION::advice4-17
 (declare (salience 50))
   (or (and (answer Q4-21 1)
            (answer Q4-22 2 Q4-21))
       (and (answer Q4-14 1)
            (completed advice4-51)
            (answer Q4-5 1)
            (answer Q4-22 2 Q4-14)))
   (answer Q4-8 1)  
 =>
   (assert (question Q4-31))
   (assert (advice 4-17))
  (focus ADVICE))

(defrule EVALUATION::advice4-18
 (declare (salience 50))
   (or (and (answer Q4-21 1)
            (answer Q4-22 1 Q4-21))
       (and (answer Q4-14 1)
            (completed advice4-51)
            (answer Q4-5 1)
            (answer Q4-22 1 Q4-14)))
   (answer Q4-8 1)  
 =>
   (assert (question Q4-29))
   (assert (question Q4-30))
   (assert (question Q4-31))
   (assert (advice 4-18))
  (focus ADVICE))

(defrule EVALUATION::advice4-19
 (declare (salience 50))
   (or (and (answer Q4-21 2)
            (answer Q4-22 2 Q4-21))
       (and (answer Q4-14 2)
            (completed advice4-51)
            (answer Q4-5 1)
            (answer Q4-22 2 Q4-14)))
  =>
   (assert (advice 4-19))
  (focus ADVICE))

(defrule EVALUATION::advice4-20a
 (declare (salience 50))
   (or (and (answer Q4-21 2)
            (answer Q4-22 1 Q4-21))
       (and (answer Q4-14 2)
            (completed advice4-51)
            (answer Q4-5 1)
            (answer Q4-22 1 Q4-14)))
   (answer Q4-9 1)
  =>
   (assert (advice 4-20a))
   (assert (completed advice4-20a))
  (focus ADVICE))

(defrule EVALUATION::advice4-20b
 (declare (salience 100))
   ?ca4-20-21 <- (completed advice4-20a|advice4-21)
   (or (and (answer Q4-21 2)
            (answer Q4-22 1 Q4-21))
       (and (answer Q4-14 2)
            (completed advice4-51)
            (answer Q4-5 1)
            (answer Q4-22 1 Q4-14)))
   (answer Q4-8 1)
  =>
   (retract ?ca4-20-21)
   (assert (advice 4-20b))
   (assert (completed advice4-20b))
  (focus ADVICE))

(defrule EVALUATION::advice4-20c
 (declare (salience 50))
   (or (and (answer Q4-21 2)
            (answer Q4-22 1 Q4-21))
       (and (answer Q4-14 2)
            (completed advice4-51)
            (answer Q4-5 1)
            (answer Q4-22 1 Q4-14)))
   (or (completed advice4-20b)
       (and (answer Q4-8 2)
            (completed advice4-20a|advice4-21)))
  =>
   (assert (question Q4-29))
   (assert (question Q4-30))
   (assert (advice 4-20c))
   (focus ADVICE))

(defrule EVALUATION::advice4-21
 (declare (salience 50))
   (or (and (answer Q4-21 2)
            (answer Q4-22 1 Q4-21))
       (and (answer Q4-14 2)
            (completed advice4-51)
            (answer Q4-5 1)
            (answer Q4-22 1 Q4-14)))
   (answer Q4-9 2)
 =>
   (assert (advice 4-21))
   (assert (completed advice4-21))
   (focus ADVICE))

(defrule EVALUATION::advice4-22
 (declare (salience 50))
   (or (and (answer Q4-21 3)
            (answer Q4-22 2 Q4-21))
       (and (answer Q4-14 3)
            (completed advice4-51|advice4-52)
            (answer Q4-5 1)
            (answer Q4-22 2 Q4-14|Q4-15)))
=>
   (assert (advice 4-22))
   (assert (question Q4-31))
   (focus ADVICE))

(defrule EVALUATION::advice4-23
 (declare (salience 50))
   (or (and (answer Q4-21 3)
            (answer Q4-22 1 Q4-21))       
       (and (answer Q4-14 3)
            (completed advice4-51|advice4-52)
            (answer Q4-5 1)
            (answer Q4-22 1 Q4-14|Q4-15)))
  =>
   (assert (advice 4-23))
   (focus ADVICE))

(defrule EVALUATION::advice4-24
 (declare (salience 50))
   (answer Q4-20 1)
   (answer Q4-8 2)
   (answer Q4-22 2 Q4-20)
 =>
   (assert (advice 4-24))
   (focus ADVICE))

(defrule EVALUATION::advice4-25
 (declare (salience 50))
   (answer Q4-20 1)
   (answer Q4-8 2)
   (answer Q4-22 1 Q4-20)
 =>
   (assert (advice 4-25))
   (assert (question Q4-29))
   (assert (question Q4-30))
   (assert (question Q4-31))
   (focus ADVICE))

(defrule EVALUATION::advice4-26
 (declare (salience 50))
   (or (and (answer Q4-21 1)
            (answer Q4-22 2 Q4-21))
       (and (answer Q4-14 1)
            (completed advice4-51)
            (answer Q4-22 2 Q4-14)))
   (answer Q4-8 2)
 =>
   (assert (advice 4-26))
   (focus ADVICE))


(defrule EVALUATION::advice4-27
 (declare (salience 50))
   (or (and (answer Q4-21 1)
            (answer Q4-22 1 Q4-21))
       (and (answer Q4-14 1)
            (completed advice4-51)
            (answer Q4-22 1 Q4-14)))
   (answer Q4-8 2)
   
 =>
   (assert (advice 4-27))
   (focus ADVICE))

(defrule EVALUATION::advice4-28
 (declare (salience 50))
   (answer Q4-25 1 Q4-23)
   (answer Q4-22 2 Q4-23)
 =>
   (assert (advice 4-28))
   (focus ADVICE))


(defrule EVALUATION::advice4-29
 (declare (salience 50))
   (answer Q4-25 1 Q4-23)
   (answer Q4-22 1 Q4-23)
 =>
   (assert (advice 4-29))
   (focus ADVICE))

(defrule EVALUATION::advice4-30
 (declare (salience 50))
   (answer Q4-25 2 Q4-23)
   (answer Q4-22 2 Q4-23)
 =>
   (assert (advice 4-30))
   (focus ADVICE))

(defrule EVALUATION::advice4-31
 (declare (salience 50))
   (answer Q4-25 2 Q4-23)
   (answer Q4-22 1 Q4-23)
 =>
   (assert (advice 4-31))
   (focus ADVICE))

(defrule EVALUATION::advice4-32
 (declare (salience 50))
   (answer Q4-25 1 Q4-24)
   (answer Q4-22 2 Q4-24)
 =>
   (assert (advice 4-32))
   (focus ADVICE))

(defrule EVALUATION::advice4-33
 (declare (salience 50))
   (answer Q4-25 1 Q4-24)
   (answer Q4-22 1 Q4-24)
 =>
   (assert (advice 4-33))
   (focus ADVICE))

(defrule EVALUATION::advice4-34
 (declare (salience 50))
   (answer Q4-25 2 Q4-24)
   (answer Q4-22 2 Q4-24)
 =>
   (assert (advice 4-34))
   (focus ADVICE))

(defrule EVALUATION::advice4-35
 (declare (salience 50))
   (answer Q4-25 2 Q4-24)
   (answer Q4-22 1 Q4-24)
 =>
   (assert (advice 4-35))
   (focus ADVICE))

(defrule EVALUATION::advice4-38-39
 (declare (salience 50))
 (answer Q4-22|Q4-32 1 Q4-18)
 (answer Q4-18 1)
 (exists (answer Q4-6 1|2))
=>
(printout t crlf crlf
"TEST BEHAVIOUR ")
(assert (completed advice4-38-39)))
 
 
(defrule EVALUATION::advice4-38
 (declare (salience 50))
   (completed advice4-38-39)
   (answer Q4-22|Q4-32 1 Q4-18)
   (answer Q4-18 1)
   (answer Q4-6 1)
  =>
   (assert (advice 4-38))
   (assert (question Q4-31))
   (focus ADVICE))

(defrule EVALUATION::advice4-39
 (declare (salience 50))
   (completed advice4-38-39)
   (answer Q4-22|Q4-32 1 Q4-18)
   (answer Q4-18 1)
   (answer Q4-6 2)
  =>
   (assert (advice 4-39))
   (assert (question Q4-31))
   (assert (question Q4-29))
   (assert (question Q4-30))
   (focus ADVICE))


(defrule EVALUATION::advice4-40-41
(declare (salience 50))
   (answer Q4-22|Q4-32 1 Q4-18)
   (answer Q4-18 2)
   (exists (answer Q4-6 1|2))
=>
(printout t crlf crlf
"TEST EFFICIENCY ")
(assert (completed advice4-40-41))) 

(defrule EVALUATION::advice4-40
 (declare (salience 50))
   (completed advice4-40-41)
   (answer Q4-22|Q4-32 1 Q4-18)
   (answer Q4-18 2)
   (answer Q4-6 1)
  =>
   (assert (advice 4-40))
   (assert (question Q4-31))
   (focus ADVICE))

(defrule EVALUATION::advice4-41
 (declare (salience 50))
   (answer Q4-22|Q4-32 1 Q4-18)
   (answer Q4-18 2)
   (answer Q4-6 2)
   (completed advice4-40-41)
  =>
  (assert (advice 4-41))
  (assert (question Q4-31))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (focus ADVICE))

(defrule EVALUATION::advice4-42-43
(declare (salience 50))
   (answer Q4-22|Q4-32 1 Q4-18)
   (answer Q4-18 2)
   (exists (answer Q4-6 1|2))
=>
(printout t crlf crlf
"TEST COVERAGE ")
(assert (completed advice4-42-43))) 

(defrule EVALUATION::advice4-42
 (declare (salience 50))
   (answer Q4-22|Q4-32 1 Q4-18)
   (answer Q4-18 3)
   (answer Q4-6 1)
   (completed advice4-42-43)
  =>
   (assert (advice 4-42))
   (assert (question Q4-31))
   (focus ADVICE))

(defrule EVALUATION::advice4-43
 (declare (salience 50))
   (answer Q4-22|Q4-32 1 Q4-18)
   (answer Q4-18 3)
   (answer Q4-6 2)
   (completed advice4-42-43)
  =>
  (assert (advice 4-43))
  (assert (question Q4-31))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (focus ADVICE))
  
(defrule EVALUATION::advice4-45-46
 (declare (salience 100))
  (or (and (answer Q4-22|Q4-32 1 Q4-19)
            (answer Q4-19 1))
      (and (answer Q4-22|Q4-32 1 Q4-14)
            (answer Q4-14 1)
            (completed advice4-51)
            (answer Q4-5 2)))
  (exists (answer Q4-6 1|2))
  =>
  (printout t crlf crlf
  "COMPARE RIVALS IN TERMS OF BEHAVIOUR")
  (assert (completed advice4-45-46))) 
  

(defrule EVALUATION::advice4-45
 (declare (salience 100))
   (completed advice4-45-46)
   (or (and (answer Q4-22|Q4-32 1 Q4-19)
            (answer Q4-19 1))
       (and (answer Q4-22|Q4-32 1 Q4-14)
            (answer Q4-14 1)
            (completed advice4-51)
            (answer Q4-5 2)))
   (answer Q4-6 1)
  =>
    (assert (advice 4-45))
   (assert (question Q4-31))
   (focus ADVICE))

(defrule EVALUATION::advice4-46
 (declare (salience 100))
   (completed advice4-45-46)
   (or (and (answer Q4-22|Q4-32 1 Q4-19)
            (answer Q4-19 1))
       (and (answer Q4-22|Q4-32 1 Q4-14)
            (answer Q4-14 1)
            (completed advice4-51)
            (answer Q4-5 2)))
   (answer Q4-6 2)
  =>
   (assert (advice 4-46))
   (assert (question Q4-31))
   (assert (question Q4-29))
   (assert (question Q4-30))
   (focus ADVICE))

(defrule EVALUATION::advice4-45-6
   (declare (salience 100))
   (completed advice4-45-46)
   (or (and (answer Q4-22|Q4-32 1 Q4-19)
            (answer Q4-19 1))
       (and (answer Q4-22|Q4-32 1 Q4-14)
            (answer Q4-14 1)
            (completed advice4-51)
            (answer Q4-5 2)))
   (exists (answer Q4-6 1|2))
  =>
   (assert (advice 4-45-6))
   (focus ADVICE))

(defrule EVALUATION::advice4-47-48
 (declare (salience 50))
   (or (and (answer Q4-22|Q4-32 1 Q4-19)
            (answer Q4-19 2))
       (and (answer Q4-22|Q4-32 1 Q4-14)
            (answer Q4-14 2)
            (completed advice4-51)
            (answer Q4-5 2)))
   (exists (answer Q4-6 1|2))
  =>
  (printout t crlf crlf
  "COMPARE RIVALS IN TERMS OF EFFICIENCY")
  (assert (completed advice4-47-48))) 

(defrule EVALUATION::advice4-47
 (declare (salience 50))
   (completed advice4-47-48)
   (or (and (answer Q4-22|Q4-32 1 Q4-19)
            (answer Q4-19 2))
       (and (answer Q4-22|Q4-32 1 Q4-14)
            (answer Q4-14 2)
            (completed advice4-51)
            (answer Q4-5 2)))
   (answer Q4-6 1)
  =>
   (assert (advice 4-47))
   (assert (question Q4-31))
   (focus ADVICE))

(defrule EVALUATION::advice4-48
 (declare (salience 50))
   (completed advice4-47-48)
   (or (and (answer Q4-22|Q4-32 1 Q4-19)
            (answer Q4-19 2))
       (and (answer Q4-22|Q4-32 1 Q4-14)
            (answer Q4-14 2)
            (completed advice4-51)
            (answer Q4-5 2)))
   (answer Q4-6 2)
  =>
  (assert (advice 4-48))
  (assert (question Q4-31))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (focus ADVICE))

(defrule EVALUATION::advice4-49-50
 (declare (salience 50))
   (or (and (answer Q4-22|Q4-32 1 Q4-19)
            (answer Q4-19 3))
       (and (answer Q4-22|Q4-32 1 Q4-14|Q4-15)
            (answer Q4-14 3)
            (completed advice4-51|advice4-52)
            (answer Q4-5 2)))
   (exists (answer Q4-6 1|2))
=>
  (printout t crlf crlf
  "COMPARE RIVALS IN TERMS OF COVERAGE")
  (assert (completed advice4-49-50))) 


(defrule EVALUATION::advice4-49
 (declare (salience 50))
   (completed advice4-49-50)
   (or (and (answer Q4-22|Q4-32 1 Q4-19)
            (answer Q4-19 3))
       (and (answer Q4-22|Q4-32 1 Q4-14|Q4-15)
            (answer Q4-14 3)
            (completed advice4-51|advice4-52)
            (answer Q4-5 2)))
   (answer Q4-6 1)
  =>
   (assert (advice 4-49))
   (assert (question Q4-31))
   (focus ADVICE))

(defrule EVALUATION::advice4-50
 (declare (salience 50))
   (completed advice4-49-50)
   (or (and (answer Q4-22|Q4-32 1 Q4-19)
            (answer Q4-19 3))
       (and (answer Q4-22|Q4-32 1 Q4-14|Q4-15)
            (answer Q4-14 3)
            (completed advice4-51|advice4-52)
            (answer Q4-5 2)))
   (answer Q4-6 2)
  =>
  (assert (advice 4-50))
  (assert (question Q4-31))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (focus ADVICE))

(defrule EVALUATION::advice4-51
 (declare (salience 100))
   (answer Q4-11 2)
   (answer Q4-13 1)
   (completed Q4-22)
=>
  (assert (advice 4-51))
  (assert (completed advice4-51))
  (focus ADVICE))

(defrule EVALUATION::advice4-52
 (declare (salience 100))
   (answer Q4-11 2)
   (answer Q4-13 2)
   (completed Q4-22)
=>
  (assert (advice 4-52))
  (assert (completed advice4-52))
  (focus ADVICE))

(defrule EVALUATION::advice4-53
 (declare (salience 100))
   (or (completed advice4-51)
       (completed advice4-52))
   (or (answer Q4-5 1)
       (answer Q4-2 6))
 =>
  (assert (advice 4-53))
  (assert (completed advice4-53))
  (focus ADVICE))

;;to let the user know that he will see repeated advice
(defrule EVALUATION::advice
 (declare (salience 100))
   (or (completed advice4-51)
       (completed advice4-52))
   (or (and (answer Q4-22 ?any&1|2 Q4-19|Q4-21)
            (answer Q4-22 ?any Q4-14|Q4-15)
            (not (advice 4-44)))
       (and (answer Q4-32 1 Q4-19)
            (answer Q4-22|Q4-32 1 Q4-14|Q4-15)))
   (answer Q4-16|Q4-17 2)
  =>
  (printout t crlf
  "Some of the advice may be the same as the one given for testing that the " crlf
  "technique has indeed been improved/extended, because, as mentioned earlier, " crlf
  "testing that a technique has been improved/extended when you've changed " crlf
  "more than one parameters in it, is like comparing two rival techniques, " crlf
  "with the old version and the new improved/extended version being the rivals." crlf crlf))  

(defrule EVALUATION::advice4-54
 (declare (salience 50))
   (answer Q4-11 1)
   (answer Q4-22 ?any Q4-14)
=>
  (assert (advice 4-54))
  (assert (completed advice4-54))
  (focus ADVICE))

(defrule EVALUATION::advice4-55
   (declare (salience 50))
   (answer Q4-11 1)
   (answer Q4-22 ?any Q4-15)
=>
  (assert (advice 4-55))
  (assert (completed advice4-55))
  (focus ADVICE))

(defrule EVALUATION::advice4-56a
   (declare (salience 150))
   (completed advice4-54|advice4-55)
   (answer Q4-22 2 Q4-14)
   (answer Q4-5 1)
=>
  (assert (advice 4-56a))
  (focus ADVICE))

(defrule EVALUATION::advice4-56b
   (declare (salience 150))
   (completed advice4-54|advice4-55)
   (answer Q4-22 1 Q4-14)
   (answer Q4-5 1)
=>
  (assert (advice 4-56b))
  (focus ADVICE))

(defrule EVALUATION::advice4-57
   (declare (salience 100))
   (completed advice4-54)
   (answer Q4-22 2 Q4-14)
   (answer Q4-14 1)  
   (answer Q4-8 1)
 =>
  (assert (advice 4-57))
  (assert (question Q4-31))
  (focus ADVICE))

(defrule EVALUATION::advice4-58
   (declare (salience 100))
   (completed advice4-54)
   (answer Q4-22 1 Q4-14)
   (answer Q4-14 1)  
   (answer Q4-8 1)
 =>
  (assert (advice 4-58))
  (assert (question Q4-31))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (focus ADVICE))

(defrule EVALUATION::advice4-59
   (declare (salience 100))
   (completed advice4-54)
   (answer Q4-22 2 Q4-14)
   (answer Q4-14 2)  
   (answer Q4-5 1)
 =>
  (assert (advice 4-59))
  (focus ADVICE))

(defrule EVALUATION::advice4-60a
   (declare (salience 100))
   (completed advice4-54)
   (answer Q4-22 1 Q4-14)
   (answer Q4-14 2)
   (answer Q4-9 1)  
   (answer Q4-5 1)
 =>
  (assert (advice 4-60a))
  (assert (completed advice4-60a))
  (focus ADVICE))

(defrule EVALUATION::advice4-60b
   (declare (salience 100))
   (completed advice4-54)
   (completed advice4-60a)
   (answer Q4-22 1 Q4-14)
   (answer Q4-14 2)
   (answer Q4-9 1)  
   (answer Q4-8 1)
 =>
  (assert (advice 4-60b))
  (focus ADVICE))

(defrule EVALUATION::advice4-61a
   (declare (salience 100))
   (completed advice4-54)
   (answer Q4-22 1 Q4-14)
   (answer Q4-14 2)
   (answer Q4-9 2)  
   (answer Q4-5 1)
 =>
  (assert (advice 4-61a))
  (assert (completed advice4-61a))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (focus ADVICE))

(defrule EVALUATION::advice4-61b
   (declare (salience 100))
   (completed advice4-54)
   (completed advice4-61a)
   (answer Q4-22 1 Q4-14)
   (answer Q4-14 2)
   (answer Q4-9 2)  
   (answer Q4-8 1)
 =>
  (assert (advice 4-61b))
  (focus ADVICE))

(defrule EVALUATION::advice4-62
   (declare (salience 100))
   (completed advice4-54|advice4-55)
   (answer Q4-22 2 Q4-14|Q4-15)
   (or (answer Q4-14 3)
       (answer Q4-15 1))
   (answer Q4-5 1)
 =>
  (assert (advice 4-62))
  (assert (question Q4-31))
  (focus ADVICE))

(defrule EVALUATION::advice4-63
   (declare (salience 100))
   (completed advice4-54|advice4-55)
   (answer Q4-22 1 Q4-14|Q4-15)
   (or (answer Q4-14 3)
       (answer Q4-15 1))
   (answer Q4-5 1)
 =>
  (assert (advice 4-63))
  (focus ADVICE))

(defrule EVALUATION::advice4-64
   (declare (salience 100))
   (completed advice4-54)
   (answer Q4-22 2 Q4-14)
   (answer Q4-14 1)  
   (answer Q4-8 2)
 =>
  (assert (advice 4-64))
  (focus ADVICE))

(defrule EVALUATION::advice4-65
   (declare (salience 100))
   (completed advice4-54)
   (answer Q4-22 1 Q4-14)
   (answer Q4-14 1)  
   (answer Q4-8 2)
 =>
  (assert (advice 4-65))
  (focus ADVICE))

(defrule EVALUATION::advice4-66
   (declare (salience 150))
   (completed advice4-54)
   (answer Q4-22 2 Q4-14)
   (answer Q4-5 2)
=>
  (assert (advice 4-66))
  (assert (question Q4-32 Q4-14))
  (focus ADVICE))

(defrule EVALUATION::advice4-67
   (declare (salience 150))
   (completed advice4-55)
   (answer Q4-22 2 Q4-15)
   (answer Q4-5 2)
=>
  (assert (advice 4-67))
  (assert (question Q4-32 Q4-15))
  (focus ADVICE))

(defrule EVALUATION::advice4-68-69
   (declare (salience 100))
   (completed advice4-54)
   (answer Q4-22|Q4-32 1 Q4-14)
   (answer Q4-5 2)
   (answer Q4-14 1)
   (answer Q4-6 2)
=>
   (printout t crlf crlf
   " TEST BEHAVIOUR")
   (assert (completed advice4-68-69)))


(defrule EVALUATION::advice4-68
   (declare (salience 100))
   (completed advice4-68-69)
   (completed advice4-54)
   (answer Q4-22|Q4-32 1 Q4-14)
   (answer Q4-5 2)
   (answer Q4-14 1)
   (answer Q4-6 2)
=>
  (assert (advice 4-68))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (assert (question Q4-31))
  (focus ADVICE))

(defrule EVALUATION::advice4-69
   (declare (salience 100))
   (completed advice4-68-69)
   (completed advice4-54)
   (answer Q4-22|Q4-32 1 Q4-14)
   (answer Q4-5 2)
   (answer Q4-14 1)
   (answer Q4-6 1)
=>
  (assert (advice 4-69))
  (focus ADVICE))

(defrule EVALUATION::advice4-68-9
   (declare (salience 100))
   (completed advice4-68-69)
   (completed advice4-54)
   (answer Q4-22|Q4-32 1 Q4-14)
   (answer Q4-5 2)
   (answer Q4-14 1)
   (exists (answer Q4-6 1|2))
=>
  (assert (advice 4-68-9))
  (focus ADVICE))

(defrule EVALUATION::advice4-70-71
   (declare (salience 100))
   (completed advice4-54)
   (answer Q4-22|Q4-32 1 Q4-14)
   (answer Q4-5 2)
   (answer Q4-14 2)
   (exists (answer Q4-6 1|2))
=>
   (assert (completed advice4-70-71))
   (printout t crlf crlf
   " TEST EFFICIENCY")) 

(defrule EVALUATION::advice4-70
   (declare (salience 100))
   (completed advice4-70-71)
   (completed advice4-54)
   (answer Q4-22|Q4-32 1 Q4-14)
   (answer Q4-5 2)
   (answer Q4-14 2)
   (answer Q4-6 1)
=>
  (assert (advice 4-70))
  (assert (question Q4-31))
  (focus ADVICE))

(defrule EVALUATION::advice4-71
   (declare (salience 100))
   (completed advice4-54)
   (completed advice4-70-71)
   (answer Q4-22|Q4-32 1 Q4-14)
   (answer Q4-5 2)
   (answer Q4-14 2)
   (answer Q4-6 2)
=>
  (assert (advice 4-71))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (assert (question Q4-31))
  (focus ADVICE))

(defrule EVALUATION::advice4-72-73
   (declare (salience 100))
   (completed advice4-54|advice4-55)
   (answer Q4-22|Q4-32 1 Q4-14|Q4-15)
   (answer Q4-5 2)
   (or (answer Q4-14 3)
       (answer Q4-15 1))
   (exists (answer Q4-6 1|2))
=>
  (assert (completed advice4-72-73))
   (printout t crlf crlf
   " TEST COVERAGE")) 

(defrule EVALUATION::advice4-72
   (declare (salience 100))
   (completed advice4-72-73)
   (completed advice4-54|advice4-55)
   (answer Q4-22|Q4-32 1 Q4-14|Q4-15)
   (answer Q4-5 2)
   (or (answer Q4-14 3)
       (answer Q4-15 1))
   (answer Q4-6 2)
=>
  (assert (advice 4-72))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (assert (question Q4-31))
  (focus ADVICE))

(defrule EVALUATION::advice4-73
   (declare (salience 100))
   (completed advice4-72-73)
   (completed advice4-54|advice4-55)
   (answer Q4-22|Q4-32 1 Q4-14|Q4-15)
   (answer Q4-5 2)
   (or (answer Q4-14 3)
       (answer Q4-15 1))
   (answer Q4-6 1)
=>
  (assert (advice 4-73))
  (assert (question Q4-31))
  (focus ADVICE))

(defrule EVALUATION::advice4-74
   (declare (salience 50))
   (completed advice4-55)
=>
  (assert (advice 4-74))
  (focus ADVICE))

(defrule EVALUATION::advice4-75
   (declare (salience 50))
   (completed advice4-54)
=>
  (assert (advice 4-75))
  (focus ADVICE))

(defrule EVALUATION::state-range
   (declare (salience 50))
   (answer Q4-7|Q4-16|Q4-10|Q4-17 3)
=>
   (printout t crlf
   "For testing that behaviour and efficiency depend on the range of inputs..." crlf)
   (assert (completed state-range))) 

(defrule EVALUATION::advice4-76
   (declare (salience 50))
   (answer Q4-22 2 state-range)
   (answer Q4-8 1)
=>
  (assert (advice 4-76))
  (focus ADVICE))

(defrule EVALUATION::advice4-77
   (declare (salience 50))
   (answer Q4-22 1 state-range)
   (answer Q4-8 1)
=>
  (assert (advice 4-77))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (assert (question Q4-31))
  (focus ADVICE))


(defrule EVALUATION::advice4-78
   (declare (salience 50))
   (answer Q4-22 2 state-range)
   (answer Q4-5 1)
=>
  (assert (advice 4-78))
  (assert (question Q4-31))
  (focus ADVICE))

(defrule EVALUATION::advice4-79
   (declare (salience 50))
   (answer Q4-22 1 state-range)
   (answer Q4-9 1)
   (answer Q4-5 1)
=>
  (assert (advice 4-79))
  (focus ADVICE))

(defrule EVALUATION::advice4-80
   (declare (salience 50))
   (answer Q4-22 1 state-range)
   (answer Q4-9 2)
   (answer Q4-5 1)
=>
  (assert (advice 4-80))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (focus ADVICE))

(defrule EVALUATION::advice4-81
   (declare (salience 50))
   (answer Q4-22 2 state-range)
   (answer Q4-8 2)
=>
  (assert (advice 4-81))
  (assert (question Q4-31))
  (focus ADVICE))

(defrule EVALUATION::advice4-82
   (declare (salience 50))
   (answer Q4-22 1 state-range)
   (answer Q4-8 2)
=>
  (assert (advice 4-82))
  (assert (question Q4-31))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (focus ADVICE))

(defrule EVALUATION::advice4-83
   (declare (salience 100))
   (answer Q4-22 2 state-range)
   (answer Q4-5 2)
=>
  (assert (advice 4-83))
  (assert (question Q4-32 state-range))
  (focus ADVICE))

(defrule EVALUATION::advice4-84-85
   (declare (salience 70))
   (answer Q4-22|Q4-32 1 state-range)
   (exists (answer Q4-6 1|2))
=>
   (printout t crlf crlf
" TEST THAT BEHAVIOUR DEPENDS ON THE RANGE OF INPUTS ")
   (assert (completed advice4-84-85)))


(defrule EVALUATION::advice4-84
   (declare (salience 70))
   (completed advice4-84-85)
   (answer Q4-22|Q4-32 1 state-range)
   (answer Q4-6 2)
=>
  (assert (advice 4-84))
  (assert (completed advice4-85))
  (assert (question Q4-31))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (focus ADVICE))

(defrule EVALUATION::advice4-85
   (declare (salience 70))
   (completed advice4-84-85)
   (answer Q4-22|Q4-32 1 state-range)
   (answer Q4-6 1)
=>
  (assert (advice 4-85))
  (assert (completed advice4-85))
  (assert (question Q4-31))
  (focus ADVICE))

(defrule EVALUATION::advice4-86-87
   (declare (salience 50))
   (answer Q4-22|Q4-32 1 state-range)
   (exists (answer Q4-6 1|2))
=>
  (printout t crlf crlf
" TEST THAT EFFICIENCY DEPENDS ON THE RANGE OF INPUTS ")
   (assert (completed advice4-86-87)))

   
(defrule EVALUATION::advice4-86
   (declare (salience 50))
   (completed advice4-86-87)
   (answer Q4-22|Q4-32 1 state-range)
   (answer Q4-6 1)
=>
  (assert (advice 4-86))
  (assert (question Q4-31))
  (focus ADVICE))

(defrule EVALUATION::advice4-87
   (declare (salience 50))
   (completed advice4-86-87)
   (answer Q4-22|Q4-32 1 state-range)
   (answer Q4-6 2)
=>
  (assert (advice 4-87))
  (assert (question Q4-31))
  (assert (question Q4-29))
  (assert (question Q4-30))
  (focus ADVICE))

(defrule EVALUATION::end
 (declare (salience -200))
=>
   (assert (question end))
   (focus ADVICE))



;;***********************************
;;***********************************
;;        ERROR MODULE             **
;;***********************************
;;***********************************

;;**********************************
;;Error checking for valid answers *
;;**********************************

(defrule ERROR::error
  ?ans <- (answer ?question ?response)
  ?err-check <- (answer-check ?response ?question ?no-choices)
  (test (or (not (integerp ?response))  ;;test if it is an integer
            (< ?response 1)             ;;test if it is smaller that 1
            (> ?response ?no-choices))) ;;test if it is bigger than the number of 
                                        ;;choices 
=>
  (printout t "You have to make a choice between 1 and " ?no-choices crlf 
              "Please make another choice: " )
  (retract ?err-check)
  (retract ?ans)
  (bind ?response (read))
  (assert (answer-check ?response ?question ?no-choices))
  (assert (answer ?question ?response))) ;;no return because we have to stay in the 
                                         ;;error module until a valid answer is given 


(defrule ERROR::multi-error1
  (declare (salience 70))
  ?multi-ans <- (answer ?question ?response)
  (multi-answer-check ?response ?question ?no-choices)
  (test (or (not (integerp ?response))  ;;test if it is an integer
            (< ?response 1)             ;;test if it is smaller that 1
            (> ?response ?no-choices))) ;;test if it is bigger than the number of 
                                        ;;choices 
=>
  (retract ?multi-ans)
  (assert (error-found ?question ?no-choices)))


(defrule ERROR::multi-error2
  (declare (salience 50))
  ?multi-err-check2 <- (multi-answer-check ?response ?question ?no-choices)
=>
  (retract ?multi-err-check2)
  (assert (completed multi-error-check2)))



(defrule ERROR::multi-error3
?completed <- (completed multi-error-check2)
?error <- (error-found ?question ?no-choices)
=>
 (retract ?error)
 (retract ?completed)
 (printout t "All your choices have to be between 1 and " ?no-choices crlf 
              "Please choose again (you do not have to repeat the valid choices " crlf
              "you've already made) :  ")
  (bind ?response (explode$ (readline)))
  (while (= (length$ ?response) 0)
  do (printout t "You have to make a choice: ")
     (bind ?response (explode$ (readline))))
  (bind ?first (first$ ?response))
  (while (<> (length$ ?response) 0)
   do
  (assert (multi-answer-check ?first ?question ?no-choices))
  (assert (answer ?question ?first))
  (bind ?temp ?response)
  (bind ?response (rest$ ?temp))
  (bind ?first (first$ ?response))))

  

;;***************************************
;; If the user enters the last choice   *
;;    from a question,                  *
;; Then return to the main menu.        *
;;***************************************


(defrule ERROR::user-returns
  (answer-check ?response ?question ?no-choices)
  (test (and (integerp ?response)
             (= ?response ?no-choices)))
=>
  (reset)
  (run))

(defrule ERROR::multi-user-returns
  (declare (salience 100))
  (multi-answer-check ?response ?question ?no-choices)
  (test (and (integerp ?response)
             (= ?response ?no-choices)))
=>
  (printout t crlf crlf
  "One of your options is " ?no-choices ", so you will return " crlf
  "to the main menu. "crlf) 
  (reset)
  (run))


;;*****************************************************
;;If the user enters a response of 5 to the main menu *
;;      Then exit the program                         *
;;*****************************************************

(defrule ERROR::user-quits
  (declare (salience 100))  ;;this rule has priority over the others
  (answer-check 5 topics 5)
=>
 (exit))

;;*************************************  
;;*************************************
;;         ADVICE MODULE             **
;;*************************************
;;*************************************

(defrule ADVICE::intro-1
  (introduction 1)
=>
  (printout t crlf crlf
" Choosing an appropriate topic of research is one of the most important " crlf
" decisions that you will have to make and it deserves careful consideration." crlf
" If a poor choice is made it may cause problems in your research, plus that " crlf
" you are not going to be happy all the way through your project. " crlf
" The following questions will help guide your choice." crlf 
" Press enter to continue:                                            ")
  (focus CHOOSE-TOPIC)
  (readline))

(defrule ADVICE::intro-2
  (introduction 2)
=>
  (printout t crlf crlf
" You have chosen your project topic, so what do you do next? " crlf
" Most students have problems at the beginning of their project on what " crlf
" to do , how to approach their problem, how to organise their thoughts." crlf
" Getting this initial \"push\" that will make you start being productive" crlf
" is one of the most important things when doing a project." crlf 
" The following questions will help deal with problems at the beginning " crlf
" of your project. " crlf
" Press enter to continue:                                            ")
  (focus START-PROJECT)
(readline))

(defrule ADVICE::intro-3
  (introduction 3)
=>
  (printout t crlf crlf
" Not every AI thesis involves code, and there are people in AI who have" crlf
" never written a significant program, but to a first approximation you " crlf
" have to be able to program to do AI. Not only does most AI work involve" crlf
" writing programs, but learning to program gives you crucial intuitions "  crlf
" into what is and isn't computationally feasible, which is the major source" crlf
" of constraint AI contributes to cognitive science." crlf 
" The following questions will help you deal with problems when programming" crlf
" is involved." crlf 
" Press enter to continue:                                            ")
 (focus PROGRAMMING)
(readline))

 (defrule ADVICE::intro-4
  (introduction 4)
=>
  (printout t crlf crlf
" One of the most difficult problems when doing research is to know what" crlf
" needs to be tested in order for the research hypothesis to be justified " crlf
" and also to find out an appropriate way to do these tests in order to support " crlf
" or refute the research hypothesis."  crlf
" The following questions will guide you on what needs to be done and some" crlf
" suggestions on how, will be given in the end. " crlf
" Press enter to continue:                                            ")
 (focus EVALUATION)
(readline))

(defrule ADVICE::1-1
  (advice 1-1)
=>
  (printout t crlf crlf
 " So, you are enthusiastic about doing research in AI but you cannot " crlf
 " find an appropriate area of research. Try talking to potential " crlf
 " supervisors and ask them what topics they recommend. " crlf
 " Read the literature to find out what interests you more and talk " crlf
 " to people in spacific fields. This can trigger your imagination " crlf
 " to think of something and propose it to potential supervisors. " crlf
 " Remember your research project must fulfil the following criteria: " crlf crlf
 "  You should be enthusiastic about it. " crlf crlf
 "  It must contribute to knowledge. " crlf crlf
 "  It must be do-able in the time given. " crlf crlf
 "  There must be someone in the Division willing to supervise it. " crlf
" Press enter to continue:                                            ")
  (assert (question general))  ;;To ask the user either return to the main menu or exit
 (readline))

(defrule ADVICE::1-2
  (advice 1-2)
=>
  (printout t crlf crlf
" As i can see your problem seems to be that you are not enthusiastic about " crlf
" any field in AI. Talking to people and reading the literature critically " crlf
" will be very important in making you see if there is some area you like in AI. " crlf
" If after that you still think that AI research does not apply to you," crlf
" you should reconsider doing research in AI. You shouldn't put yourself " crlf
" into something you don't like. " crlf crlf
" On the other hand if for some reason you have to do it you should " crlf
" try to find something that you like more. Read the literature to " crlf
" find out what interests you more and talk to people in specific " crlf
" fields. This can trigger your imagination to think of something " crlf
" and propose it to potential supervisors. But even if you cannot " crlf
" think of something specific there are definitely people you can ask " crlf
" for suggestions and help. And try not choose something too difficult " crlf
" because you will not be able to finish and it will make you" crlf
" feel worse about AI. " crlf 
" Press enter to continue:                                            ")
 (assert (question general))  ;;To ask the user either return to the main menu or exit
(readline))

(defrule ADVICE::1-3
  (advice 1-3)
=>
  (printout t crlf crlf

" As i can see your problem seems to be that you are not enthusiastic about " crlf
" any field in AI. You've talked to people and read the literature and AI " crlf
" still doesn't seem to appeal to you. You shouldn't put yourself into " crlf
" something you don't like. You will not be able to go through the demanding " crlf
" times that will come. " crlf crlf
" On the other hand if for some reason you have to do it you should try to find " crlf
" something that you like more. Read the literature to find out what interests " crlf
" you more and talk to people in specific fields. This can trigger your imagination " crlf
" to think of something and propose it to potential supervisors. But even if you " crlf
" cannot think of something specific there are definitely people you can ask for " crlf
" suggestions and help. And try not choose something too difficult because you " crlf
" will not be able to finish and it will make you feel worse about AI." crlf 
" Press enter to continue:                                            "
)
(readline)
 (assert (question general)))  ;;To ask the user either return to the main menu or exit


(defrule ADVICE::1-4
  (advice 1-4)
=>
  (printout t crlf crlf
" Beware of choosing an area new to you because of the superficial appeal. " crlf
" The gloss will soon wear off when you are faced with the hard grind " crlf
" necessary to get a basic grounding in it. It will help if you choose " crlf
" an area which utilizes your already proven abilities, e.g. mathematical " crlf
" reasoning for mathematicians; natural language for linguistics. " crlf crlf
" Press enter to continue:                                            "
)
  (assert (completed advice1-4))
   (readline))

(defrule ADVICE::1-5
  (advice 1-5)
=>
  (printout t crlf crlf
" Having chosen the general area or problem you want to work on, you " crlf
" must try to define a specific project. This is where your supervisor" crlf
" comes in. Find a member of academic or research staff whose interests" crlf
" lie in this area and who is prepared to advise you. S/he may have " crlf
" some projects to suggest and will also be able to pass an opinion " crlf
" on the worthiness and doability of anything you suggest. " crlf
" On the whole, beginning students tend to underestimate the worthiness" crlf
" and overestimate the doability of projects - quite modest sounding " crlf
" projects prove harder than they look. So do listen to your supervisor's " crlf
" advice and don't try to \"Solve the world\". The project must be " crlf
" do-able in the period of time given. " crlf crlf
" Get your supervisor to suggest some reading material. You will find " crlf
" suitable projects in the future work papers and theses. It is good" crlf
" research methodology to continue working on a problem from where" crlf
" someone else left off. You may find some work you consider badly" crlf
" done - consider redoing it properly. You may be able to simplify" crlf
" the program, relate it to other work or build a more powerful" crlf
" program. " crlf crlf
" Projects to avoid, because they lead to bad research, are programs " crlf
" which do a task without addressing any important issues or which are " crlf
" not based on previous work. " crlf crlf
" Also remember that the topic chosen must contribute to knowledge and" crlf
" that it must be something you believe in and are enthusiastic about. " crlf 
" Press enter to continue:                                            ")
(readline)  
(assert (question general)))


(defrule ADVICE::1-6
  (advice 1-6)
=>
  (printout t crlf crlf
" Having chosen the general area or problem you want to work on, you must " crlf
" try to define a specific project. Your supervisor will advise you. " crlf
" S/he may have some projects to suggest and will also be able to pass "
" an opinion on the worthiness and doability of anything you suggest. " crlf
" On the whole, beginning students tend to underestimate the worthiness " crlf
" and overestimate the doability of projects - quite modest sounding " crlf
" projects prove harder that they look. So do listen to your supervisor's " crlf
" advice and don't try to "Solve the world".The project must be do-able in " crlf
" the period of time given. " crlf crlf
" Get your supervisor to suggest some reading material. You will find " crlf
" suitable projects in the future work papers and theses. It is good research " crlf
" methodology to continue working on a problem from where someone else " crlf
" left off. You may find some work you consider badly done - consider " crlf
" redoing it properly. You may be able to simplify the program," crlf
" relate it to other work or build a more powerfull program. " crlf crlf
" Projects to avoid, because they lead to bad research, are programs" crlf
" which do a task without addressing any important issues or which" crlf
" are not based on previous work. " crlf 
" Press enter to continue:                                            ")
  (readline)
(assert (question general)))

(defrule ADVICE::1-7
  (advice 1-7)
=>
  (printout t crlf crlf
" I see that the topic you've chosen is something you like and you feel " crlf
" confident with, so I don't see any problem here. Just check again with " crlf
" your supervisor if satisfies the criteria for successful research. " crlf
" Press enter to continue:                                            ")
(readline)
  (assert (question general)))

(defrule ADVICE::1-8
  (advice 1-8)
=>
  (printout t crlf crlf
" It is very important that you enjoy your research. Do not " crlf
" underestimate the importance of being enthusiastic about your project." crlf
" You are going to need all the enthusiasm you can raise to give you" crlf
" perseverence and motivation to see you through what will be hard," crlf
" lonely and unstructured period. " crlf 
" Press enter to continue:                                            ")
(readline))

(defrule ADVICE::1-10
  (advice 1-10)
=>
  (printout t crlf crlf
" It is not good that you do not feel confidence in what you do. Part " crlf
" of a researcher's skill includes an appropriate mental attitude to his/her " crlf
" work. This can be learnt, if you know what you are aiming for and " crlf
" are determined enough. One of the main ingredients of this mental" crlf
" attitude is a belief in what you are doing. Do not be afraid of a" crlf
" little egotism! You must believe that the problem you are tackling" crlf
" is important and that your contribution to the solution is significant." crlf
" Otherwise, how are you to generate the energy to see yourself through" crlf
" the long hours of hard work required? Thus it is very important in" crlf
" obtaining this self-assurance to pick a research topic you can " crlf
" believe in. Of course, you should be able to listen to critisism. " crlf
" You must be prepared to modify your ideas if they are wrong. " crlf 
" Press enter to continue:                                            ")
(readline)) 

(defrule ADVICE::1-11
  (advice 1-11)
=>
  (printout t crlf crlf
" If you are sure that your project is not a contribution to knowledge you " crlf
" shouldn't do it. Talk to people and read the literature to be sure that indeed " crlf
" the project you've chosen will not contribute to knowledge. " 
" Press enter to continue:                                            ")
(readline)
(assert (question general)))


(defrule ADVICE::1-12
  (advice 1-12)
=>
  (printout t crlf crlf
" The fact that you've found a supervisor for your project shows that it is " crlf
" probably worthy, i.e it is a contribution to knowledge. You may want to talk " crlf
" to your supervisor about it, if you have doubts and take his/her advice. " crlf
" Press enter to continue:                                            ")
(readline))

(defrule ADVICE::1-13
  (advice 1-13)
=>
  (printout t crlf crlf
" Having a topic in mind you should try to look for a potential supervisor. " crlf
" Find a member of academic or research staff whose interests lie in this " crlf
" area and who is prepared to advise you. S/he will also be able to pass an " crlf
" opinion on whether the project is a contribution to knowledge and also " crlf
" give advice about the doability of anything you suggest. On the whole, " crlf
" beginning students tend to underestimate the worthiness and overestimate " crlf
" the doability of projects - quite modest sounding projects prove harder " crlf
" that they look. So do listen to your supervisor advice and don't try to " crlf
" \"Solve the world \". The project must be do-able in the period of time given. "
crlf crlf
" Get your supervisor to suggest some reading material and talk to fellow " crlf
" workers. That will make you have a clearer idea of what you want. " crlf
" Projects to avoid, because they lead to bad research, are: programs " crlf
" which do a task without addressing any important issues and programs which " crlf
" are not based on previous work." crlf
" Press enter to continue:                                            ")
(readline)
(assert (question general)))

(defrule ADVICE::1-14
  (advice 1-14)
=>
 (printout t crlf crlf
" If you cannot find anyone willing to supervise your project" crlf
" you must reconsider if it is a good idea after all. Ask some academic " crlf
" staff why they don't want to supervise your project and take their " crlf
" advice. They know better if your project will result to good research. " crlf
" Ask for advice and suggestions." crlf
" Press enter to continue:                                            ")
(readline)
 (assert (question general)))

(defrule ADVICE::1-15
  (advice 1-15)
=>
 (printout t crlf crlf
" Most students pick research goals that are far too ambitious. " crlf
" This is particularly easy in AI. So many tasks which humans find" crlf
" easy turn out to be really hard to emulate. Listen to your supervisor," crlf
" s/he has more experience. If s/he thinks that your goals are too " crlf
" ambitious reconsider your topic. Read the literature critically " crlf
" and talk to fellow workers. One good source of ideas is the further " crlf
" work sections of papers.Another good source is redoing bad work " crlf
" better. " crlf 
" Press enter to continue:                                            ")
(readline)
 (assert (question general)))

(defrule ADVICE::1-16
  (advice 1-16)
=>
 (printout t crlf crlf
" You should have more confidence in yourself. If you think your " crlf
" topic is too difficult while your supervisor doesn't it means " crlf
" that you should raise your self-esteem a little. Doubts about " crlf
" your own ability can put you in a frame of mind where the " crlf
" dedication and enthusiasm necessary to produce results evaporates." crlf
" The way out of this vicious circle is to realise that research" crlf
" ability does not depend on some magic essence. It is a skill which" crlf
" can be learnt, like any other." crlf 
" Press enter to continue:                                            ")
(readline))

(defrule ADVICE::1-17
  (advice 1-17)
=>
 (printout t crlf crlf
" I see that you haven't read some relative literature. You should " crlf
" ask some academic staff for suggestions on some. It is very important " crlf
" to read the literature, especially if you are doing a PhD or MPhil" crlf
" in order to make sure that you know what is going on in related " crlf
" research, to make sure nobody is currently working on what you want" crlf
" to do, and generally to see if what you are going to do really " crlf
" interests you. " crlf crlf
" Sometimes reading can be difficult. You should know what to read and how. " crlf
" It may be helpful to think of the work of others as arranged in concentric " crlf
" circles around your own, where the relevance of the work decreases as you " crlf
" get further from the centre. Different study techniques are appropriate " crlf
" for different circles. For the outer circle, you can achieve an appropriate " crlf
" level of familiarity with the work in this circle by skimming papers or " crlf
" reading the abstracts. For the middle circle, you need to spend more time; " crlf
" you will need to read some papers right through. Finally, for the inner " crlf
" circle you should read papers more than once for deep understanding. You  " crlf
" may want to work through the examples, set yourself some exercises, get " crlf
" in touch with the author about it. " crlf
" Press enter to continue:                                            ")
(readline))

(defrule ADVICE::1-18
  (advice 1-18)
=>
 (printout t crlf crlf
" Talking to people can be very useful. You can learn many " crlf
" things from their experience and knowledge, that is difficult to " crlf
" find in the literature. Talk with people whose research is related to your " crlf
" topic, but also to your fellow students.Don't miss the opportunity " crlf
" to communicate with people and take advice." crlf 
" Press enter to continue:                                            ")
(readline))

 (defrule ADVICE::2-1
  (advice 2-1)
=>
  (printout t crlf crlf
" Read some of the literature. Ask your supervisor for suggestions. "crlf 
" Read the literature critically, with a question in mind. It can be " crlf
" proven very useful for your creativity and imagination." crlf 
" Press enter to continue:                                            ")
(readline))
 
 (defrule ADVICE::2-2
  (advice 2-2)
=>
  (printout t crlf crlf
" Read the literature critically, with a question in mind. It can be " crlf
" proven very useful for your creativity and imagination. " crlf 
" Press enter to continue:                                            ")
(readline))

 
(defrule ADVICE::2-3
  (advice 2-3)
=>
  (printout t crlf crlf
" Try talking to people. Do not go away and hide. Do not be ashamed of " crlf
" your ideas. Other people's are sillier. " crlf 
" Press enter to continue:                                            ")
(readline))
 

(defrule ADVICE::2-4
  (advice 2-4)
=>
  (printout t crlf crlf
" Try tackling a simplified version of the problem first. You should ask " crlf
" your supervisor for exercises, mini-projects, etc. It will give you " crlf
" confidence and a better understanding of your original problem. Also " crlf
" try doing the easiest thing first, then stand on shoulders of that work " crlf
" and do the next easiest thing etc. " crlf 
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::2-5
  (advice 2-5)
=>
  (printout t crlf crlf
" It will help to try to talk through your ideas and explain them to someone or " crlf
" to a small group of people. You cannot imagine how half-baked ideas " crlf
" take shape and bugs are exposed. It is not necessary to have a willing " crlf
" audience, even if one doesn't exist you can try to explain your " crlf
" ideas to an imaginary one. " crlf 
" Press enter to continue:                                            ")
(readline))

(defrule ADVICE::2-6
  (advice 2-6)
=>
  (printout t crlf crlf
" It will help if you tried to write your ideas down. It will help organise " crlf
" your thoughts and identify bugs." crlf
" You should also talk through your ideas and explain them to someone or to a small " crlf
" group of people. You cannot imagine how half-baked ideas take shape and bugs" crlf
" are exposed. It isn't necessary to have a willing audience, even" crlf 
" if one doesn't exist you can try to explain your ideas to an imaginary one. " crlf
" Press enter to continue:                                            ")
(readline))

(defrule ADVICE::2-7
  (advice 2-7)
=>
  (printout t crlf crlf
" You may believe that the reason why no one understands you is because you" crlf
" are a genius and the others are all looneys and charlatans. When people " crlf
" do not understand it is your fault - you should try harder. You should consider" crlf
" alternative causes for misunderstanding like: " crlf crlf
" Love of jargon: " crlf 
" AI is full of jargon. Try to rephrase your ideas using " crlf
" ordinary English\; try using someone else's jargon." crlf crlf 
" Love of complexity: " crlf
" It is not a virtue to make a complicated program - it is just a nuisance " crlf
" to other people. Do it the simplest way you can." crlf
" Press enter to continue:                                            ")
(readline))

(defrule ADVICE::2-8
  (advice 2-8)
=>
  (printout t crlf crlf
" You should try talking to people. Do not go away and hide. Do not be ashamed " crlf
" of  your ideas. Other people's are sillier. " crlf 
" Press enter to continue:                                            ")
(readline))

(defrule ADVICE::2-9
  (advice 2-9)
=>
  (printout t crlf crlf
" It could be that you are secretly afraid that your idea is not so great " crlf
" after all. Hard experience has taught you that ideas that appear to be " crlf
" solutions to all your problems in the middle of the night, evaporate in the" crlf
" cold dawn. Courage! Research is always like this. Ten steps forward and nine " crlf
" steps back. The sooner you subject your idea to the acid test, the sooner you"  crlf
" will discover its limitations and be ready for the next problem." crlf 
" Press enter to continue:                                            ")
(readline))

(defrule ADVICE::2-10
  (advice 2-10)
=>
  (printout t crlf crlf
" One cause of your problem may be that you are lost in abstraction. In order " crlf
" to achieve anything you must tackle the abstract property in a concrete " crlf
" situation, that is you must build a program to do some task that requires" crlf
" search to be controlled, knowledge to be represented, knowledge to be learnt." crlf
" Trying to tackle the problem in the abstract will only lead to paralysis and " crlf
" frustration." crlf 
" Press enter to continue:                                            ")
(readline))

(defrule ADVICE::2-11
  (advice 2-11)
=>
  (printout t crlf crlf
" It is good to have high standards for your finished product but do not " crlf
" try to apply the same standards to your initial attempts, or you may never " crlf
" get started. Do something simple, then apply your standards to refine it " crlf
" to something more worthy. " crlf 
" Press enter to continue:                                            ")
(readline))

(defrule ADVICE::2-12
  (advice 2-12)
=>
  (printout t crlf crlf
" I see you do not believe a lot in what you do. Part of a researcher's skill " crlf
" includes an appropriate mental attitude to his/her work. This is something " crlf
" that can be learnt, if you know what you're aiming for and are determined enough." crlf
" You must believe that the problem you are tackling is important and that your" crlf
" contribution to the solution is significant. Otherwise you are not going to " crlf
" find the energy to see yourself through the long hours of work required. The " crlf
" first step obtaining this self-assurance is to pick a research topic you can " crlf
" believe in. (You can go to the \"Choosing a research topic - checking an already " crlf
" chosen one\" option in the initial menu to get advice on this.)"crlf crlf 
" Of course you should not refuse listening to criticism. You should be prepared" crlf
" to modify your ideas if they are wrong." crlf " 
Press enter to continue: ")
(readline))

(defrule ADVICE::2-13
  (advice 2-13)
=>
  (printout t crlf crlf
" I see that you have doubts about your research abilities. This can cause " crlf
" problems to your research. Doubts about your own ability can put you in a " crlf
" frame of mind where the dedication and enthusiasm necessary to produce results" crlf
" evaporates. The way out of this vicious circle is to realise that research" crlf
" ability does not depend on some magic essence. It is a skill which can be " crlf
" learnt, like any other. You too can do original research, by following the " crlf
" advice of this program! " crlf 
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::2-14
  (advice 2-14)
=>
  (printout t crlf crlf
" Your problem apparently is that it's not clear to you what you are trying to " crlf
" do, and what should be your goal. So before you start you should define your " crlf
" research goal(s). That will direct your work. Form a hypothesis and work " crlf
" towards proving or disproving this hypothesis." crlf crlf

" Have a range of ideas on the boil. Try to construct a hierarchy of research" crlf
" goals. This imposes a structure on the work and acts as a safety net when one" crlf
" finds that one has attempted more than is possible in the project." crlf crlf

" Start from the easiest part first and build on to that to continue with the " crlf
" next easiest thing. " crlf 
" Press enter to continue:                                            ")
(readline) 
(assert (question general)))



(defrule ADVICE::2-15
  (advice 2-15)
=>
  (printout t crlf crlf
" Do not try to reach your final goal from the beginning. Try with something" crlf
" easier first. It is good to have a hierarchy of research goals that lead to" crlf
" your final goal. This imposes a structure on the work and acts as a safety " crlf
" net when one finds that one has attempted more than is possible in the project." crlf 
" Press enter to continue:                                            ")
(readline))

(defrule ADVICE::2-16
  (advice 2-16)
=>
  (printout t crlf crlf
" It is good to have a hierarchy of research goals. You can start easier when" crlf
" you have set your priorities. This imposes a structure on the work and acts" crlf
" as a safety net when one finds that one has attempted more than is possible" crlf
" in the project. Start with something easier first and build on to that to " crlf
" continue with the next easiest thing. " crlf 
" Press enter to continue:                                            ")
(readline))

(defrule ADVICE::2-17
  (advice 2-17)
=>
  (printout t crlf crlf
" If you do not have a structured methodology it is very difficult to start" crlf
" working in the right way. You should first decide exactly which methodology " crlf
" you are going to use and then start working. You will see how better it is " crlf
" when you have a structured procedure to follow in your mind." crlf 
" Press enter to continue:                                            ")
(readline)
 (assert (question general)))

(defrule ADVICE::2-18
  (advice 2-18)
=>
  (printout t crlf crlf
" It seems that you've done everything one can do in order to start working in" crlf
" a project, but you still cannot create new ideas. I think you should go " crlf
" through the questions again and reconsider the truth of your answers. If you" crlf
" think you've done everything you could, just try harder. I'm sure ideas will" crlf
" eventually come." crlf 
" Press enter to continue:                                            ")
(readline)
 (assert (question general)))


(defrule ADVICE::2-19
  (advice 2-19)
=>
  (printout t crlf crlf
" Im afraid I cannot understand what your problem is. You seem pretty happy " crlf
" with your work, you don't have a problem creating new ideas, you've chosen " crlf
" your research goals and your methodology. Don't expect everything to be " crlf
" perfect from the beginning. Be patient!" crlf
" Press enter to continue:                                            ")
(readline)
(assert (question general)))

(defrule ADVICE::3-1
  (advice 3-1)
=>
  (printout t crlf crlf
" It is not good to start writing code immediately. You should first try to " crlf
" think of how your program should implement the task (probably by writing down " crlf
" some scenarios) and why. Try to write a specification for your program, sketch " crlf
" the algorithm\; it will help you a lot." crlf
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::3-2
  (advice 3-2)
=>
  (printout t crlf crlf
" You may get a satisfying sense of achievement when a bug is exposed or a " crlf
" nice output generated but this is illusory. Your program must be explainable" crlf
" at a higher level than code, for it to make a real contribution to knowledge." crlf
" Try to plan your program theoretically before going to the terminal. If you " crlf
" must work some of it out at the terminal then rush away soon and work out the" crlf
" theory. If you find it hard, try to describe how it works to a friend, in a " crlf
" paper or at a seminar." crlf
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::3-3
  (advice 3-3)
=>
  (printout t crlf crlf
" It's not good not to explain the theory behind your code. If you don't explain " crlf
" your program at a higher level than code it will not make a real contribution " crlf
" to knowledge. It will help to try to describe how it works to a friend, in a " crlf
" paper or at a seminar. You will be amazed to see how half-baked ideas take shape " crlf
" and bugs get exposed that way. " crlf
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::3-4
  (advice 3-4)
=>
  (printout t crlf crlf
" Try to describe how it works to a friend, in a paper or at a seminar. You " crlf
" will be amazed to see how half-baked ideas take shape and bugs get exposed" crlf
" that way." crlf
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::3-5
  (advice 3-5)
=>
  (printout t crlf crlf
" If people don't understand it's your fault. You should try harder! " crlf crlf
" One cause of misunderstanding may be that you use a lot of jargon. " crlf
" Try to rephrase your ideas using ordinary English or in someone else's jargon." crlf crlf
" Another cause may be your love of complexity. It is not a virtue to make a " crlf
" complicated - it is just a nuisance to other people. Do it the simplest way " crlf
" you can. " crlf
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::3-6
  (advice 3-6)
=>
  (printout t crlf crlf
" Take advantage of opportunities to work directly with more experienced " crlf
" programmers. Or see if you can get one to critique your code. Try to gain" crlf
" as much as you can from their experience. " crlf
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::3-7
  (advice 3-7)
=>
  (printout t crlf crlf
" It is true that it is time-consuming to take other people's code and try to " crlf
" understand it but the benefits can outweigh the costs. Read the code carefully. " crlf
" It can take you as long to read and fully understand someone else's code as " crlf
" it would take to write it yourself, but you will learn a whole lot of nifty " crlf
" tricks you wouldn't have thought of and that are not in any textbook. You'll " crlf
" also learn how not to write code when you read pages of incomprehensible " crlf
" uncommented gibberish." crlf
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::3-8
  (advice 3-8)
=>
  (printout t crlf crlf
" Be careful of making overcomplicated programs. Making a complicated program " crlf
" is not a virtue - it is just a nuisance to other people. Do it the simplest " crlf
" way you can. " crlf crlf
" Like papers, programs can be over-polished. Rewriting code till it's perfect," crlf
" making everything maximally abstract, writing macros and libraries, and " crlf
" playing with operating system internals has sucked many people out of their" crlf
" theses and out of the field. " crlf
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::3-9
  (advice 3-9)
=>
  (printout t crlf crlf
" It is necessary to put a lot of comments in your code. You should keep in " crlf
" mind that your program should be comprehensible by other people apart from you." crlf
" After time people may want to read what you've done, use your program, " crlf
" improve your code, base their study and research on yours\; you should keep " crlf
" that in mind and comment your code as much as you can. You will also be " crlf
" surprised with how many things you can forget why and how you did them " crlf
" even in one day! " crlf
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::3-10
  (advice 3-10)
=>
  (printout t crlf crlf
" When the program works you will have forgotten most of the things in your " crlf
" program, i.e. the why you did it the way you did it. So write comments as " crlf
" you go along, and write as much as you can. " crlf
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::3-11
  (advice 3-11)
=>
  (printout t crlf crlf
" It's a big mistake not to put comments into your program. Primarily, " crlf
" you will not remember what you've done and how. You will be surprised with " crlf
" how many things you can forget why and how you did them even in one day! " crlf
" Secondarily, you should keep in mind that your program should be comprehensible "crlf
" by other people apart from you. People may want to read what " crlf
" you've done, use your program, improve your code, base their study " crlf
" and research on yours. Thus, you should keep that in mind and comment your code" crlf
" as much as you can." crlf
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::3-12
  (advice 3-12)
=>
  (printout t crlf crlf
" The fact that the existing languages do not offer exactly what you need " crlf
" for your project is no excuse for writing another one. You can usually find " crlf
" a reasonable candidate and add to it what you need. Writing a useful, new " crlf
" language requires an encyclopaedic knowledge of AI needs and experience of" crlf
" systems programming. No one will use the language you write, not even you!" crlf
" You will have to spend all your time on the language and none on the project" crlf
" you started with. If you really believe that existing languages are inadequate " crlf
" then write a paper on it, carefully describing the deficiencies. If you do" crlf
" a good enough demolition job, enhanced languages will spring up overnight. " crlf
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::3-13
  (advice 3-13)
=>
  (printout t crlf crlf
" There is always a chance to find a reasonable candidate language and add to" crlf
" it what you need. Maybe this can help you do what you want for your project." crlf crlf
" Apart from only explaining in your project why you didn't do all things " crlf
" planned you can also write a paper about the inadequacies of the existing " crlf
" languages. If you do a good enough demolition job, enhanced languages will " crlf
" spring up overnight." crlf
" Press enter to continue:                                            ")
(readline))


(defrule ADVICE::3-14
  (advice 3-14)
=>
  (printout t crlf crlf
" I see that you do everything the right way. I cannot see what your problem is." crlf
" You seem to be doing just fine! " crlf))

;;******************************************************************************

(defrule ADVICE::sources
  (advice sources)
=>
  (printout t crlf crlf
" In the evaluation methods suggested, there might be cases were you are " crlf
" advised to look at a specific source for more details. These sources are " crlf
" referred to with their codes. The sources used and their codes are the " crlf
" following: " crlf
" (You may want to note them down now, so that you do not have to remember " crlf
"  all of them.) " crlf crlf
"- Human-Computer Interaction, 2nd edition, A.Dix and J.Finlay and G.Abowd " crlf
"  and R. Beale (code: HCI) " crlf 
"- Genetic algorithms and genetic programming, Peter Ross, Department of AI " crlf
"  University of Edinburgh, January 1998 (code: GAGP) " crlf
"- ALGORITHMICS - The spirit of Computing, David Harel, second edition " crlf
"  (code: ALG) " crlf
"- Modelling and Simulation (CS module), Jane Hillston, University of " crlf
"  Edinburgh (code: MS)" crlf
"- AI Research Methodologies (AI module), Alan Bundy, University of " crlf
"  Edinburgh (code: AIRM) " crlf crlf
"Press enter to continue: ")
(readline))

(defrule ADVICE::4-1
  (advice 4-1)
=>
  (printout t crlf crlf
" TEST BEHAVIOUR " crlf
" The evaluation of an interactive system (in terms of behaviour) has " crlf
" three main goals: " crlf crlf
"- To assess the extent of the system's functionality and make sure not " crlf
"  only that this functionality meets the requirements of the user (in " crlf
"  terms of behaviour, efficiency and coverage) but also that this " crlf
"  functionality can be easily reached by the user, in terms of the actions " crlf
"  s/he needs to take to perform the task. " crlf crlf
"- To assess the effect of the interface on the user i.e. how easy the " crlf
"  system is to learn, it's usability and the user's attitude to it. In " crlf
"  addition, it is important to identify areas of design which overload " crlf
"  the user in some way, e.g. by requiring an excessive amount of " crlf
"  information to be remembered. You can find common usability measures, " crlf
"  such as learnability, flexibility, robustness, in books about interface " crlf
"  evaluation (e.g. HCI-chapter 4). " crlf crlf
"- Identify specific problems with the design. " crlf crlf
" You should also make sure that this system is better overall than its " crlf
" rivals. It may not be better in all criteria (behaviour, coverage, " crlf
" efficiency). However, the system should certainly not be worse than some " crlf
" rival on all counts and should certainly be better on the criteria which " crlf
" the user rates most highly. If there are \"better\" systems that do the " crlf
" same task your system will probably not be preferred. " crlf crlf
"Press enter to continue: ")
(readline))

(defrule ADVICE::4-2
  (advice 4-2)
=>
  (printout t crlf crlf
" There are three methods of doing empirical evaluation of the system " crlf
" without users (see also HCI-chapter 11): " crlf crlf
"- Cognitive walkthrough, in which evaluators step through a sequence " crlf
"  of actions, that the interface of the system will require a user to " crlf
"  perform in order to accomplish some task, and check for potential " crlf
"  usability or any other problems. " crlf crlf
"- Expert heuristic evaluation, where some experts are requested to " crlf
"  evaluate the design. Also an expert in the task X may be called to " crlf
"  evaluate the quality of the task performed. " crlf crlf
"- Heuristic evaluation, where some evaluators are requested to evaluate " crlf
"  the system, who are not experts but are provided with some heuristics " crlf
"  that will help them generate ideas while critiquing the system. (These " crlf
"  heuristics can be found in the HCI book-pages 413-414)" crlf crlf
"Press enter to continue: ")
(readline))

(defrule ADVICE::4-3
  (advice 4-3)
=>
  (printout t crlf crlf
" The best way to evaluate the system early at the design stage using " crlf
" users is to have a prototype working. However, you can still use users " crlf
" to evaluate the design using a paper and pencil walkthrough method. Users " crlf
" are asked to \"walk through\" typical tasks using paper mock-ups of the " crlf
" system design. This is intended to show up discrepancies between the " crlf
" user's requirements and the actual design as proposed. Such exercises " crlf
" provide a simple and cheap technique for early assessment of models." crlf crlf
"Press enter to continue: ")
(readline))

(defrule ADVICE::4-4
  (advice 4-4)
=>
  (printout t crlf crlf
" In order to use empirical evaluation to support your claims about the " crlf
" behaviour of the system you need to conduct a controlled experiment for " crlf
" each claim you make. Any experiment has the same basic form. First you " crlf
" choose a hypothesis to test, which can be determined by measuring some " crlf
" attribute of subject behaviour. A number of experimental conditions should " crlf
" be considered which differ only in the values of certain control variables. " crlf
" The control variables here will be different example session that the " crlf
" subject should go through. Every other condition should stay the same!" crlf
" (If that is not possible you should find ways to counteract the changes " crlf
" in other conditions or make sure that their influence on the results has " crlf
" been taken into account)" crlf crlf
"Press enter to continue: ")
(readline)
(printout t crlf crlf
" Then the properties of behaviour you want to test will be measured " crlf
" across the range of different examples. There are two experimental " crlf
" methods (according to HCI-chapter 11): " crlf crlf
"- Between-groups (or randomized) design, where each subject is assigned " crlf
"  to a different condition. It has the advantage that any learning effect" crlf
"  resulting from the user performing in one condition and then the other is" crlf
"  controlled and the disadvantage that a greater number of subjects are " crlf
"  required and that individual differences between users can bias the " crlf
"  results." crlf crlf
"- Within-groups, where each user performs under each different condition. " crlf
"  This design can suffer from transfer of learning effects, but this can be" crlf
"  lessened if the order in which conditions are tackled is varied between " crlf
"  users. Within-groups is less costly than between-groups, since fewer users" crlf
"  are required and there is also less chance of effects from variation " crlf
"  between subjects. " crlf crlf
"Press enter to continue: ")
(readline)
(printout t crlf crlf
" Within the basic form there are a number of factors that are important " crlf
" to the overall reliability of the experimental design. Thus, you should " crlf
" note that: " crlf crlf
"- Subjects should be chosen to match the expected user population as closely " crlf
"  as possible. Ideally this would involve experimental testing with the " crlf
"  actual users but this is not always possible. Also the sample size should " crlf
"  be large enough to be representative of the population. " crlf crlf
"- Differences in experimental evidence may only be due to chance. Statistical " crlf
"  analysis may be required to show that these differences are significant " crlf
"  (e.g. t-test described in GAGP-appendix A). " crlf crlf
"- Experimental conditions should only differ in the control variable and be" crlf
"  otherwise identical." crlf crlf
"- The property of interest must be measurable in some way." crlf crlf
"Press enter to continue: ")
(readline))


(defrule ADVICE::4-5
  (advice 4-5)
=>
  (printout t crlf crlf
" Three observational techniques are suggested: " crlf crlf
"- Think aloud: While the users are observed interacting with the system " crlf
"  they are asked to elaborate their actions by \"thinking aloud\", i.e. describing" crlf
"  what they believe is happening, why they take an action, what they are " crlf
"  trying to do." crlf crlf
"- Protocol analysis: The user actions are recorded. There are a number of" crlf
"  ways for recording users actions:" crlf
"  1)Paper and pencil " crlf
"  2)Audio recording" crlf
"  3)Video recording" crlf
"  4)Computer logging" crlf
"  5)User notebooks" crlf crlf
"  As the task of analysing protocols is time-consuming and tedious by hand " crlf
"  you should try to use automatic analysis tools to support the task. " crlf crlf
"- Post-task walkthroughs: After the interaction with the system the user" crlf 
"  is asked to reflect back to his/her actions. " crlf crlf
"For more details see also HCI-chapter 11." crlf crlf
"Press enter to continue: ")
(readline))

(defrule ADVICE::4-6
(advice 4-6)
=>
(printout t crlf crlf
" Two types of query techniques are suggested:" crlf crlf
"- Interviews: The users are interviewed about their experience from " crlf
"  interacting with the system. In the interview you should try to address " crlf
"  all the aspects of behaviour you want to test. " crlf crlf
"- Questionnaires: The users are asked to fill in questionnaires after they " crlf
"  interact with the system. The questionnaires should be well designed and" crlf
"  address all the aspects of the behaviour you want to test. The questions" crlf
"  should be well phrased so that the users understand and answer correctly." crlf crlf
" For more details see also HCI-chapter 11." crlf crlf
"Press enter to continue: ")
(readline))

(defrule ADVICE::4-7
  (advice 4-7)
=>
  (printout t crlf crlf
" There are two ways of theoretical evaluation of an interactive system " crlf
" in terms of the behaviour (according to HCI-chapter 11):" crlf crlf
"- Review-based evaluation: Experimental psychology and human-computer" crlf
"  interaction between them possess a wealth of experimental results and" crlf
"  empirical evidence. Some of this is specific to a particular domain " crlf
"  and other deals with more generic issues and applies in a variety of" crlf
"  situations. This approach to evaluating the design exploits this " crlf
"  inheritance and scours the literature for evidence to support (or refute)" crlf
"  aspects of the design. After all, it is wasteful to repeat experiments " crlf
"  continually, for the sake of it, and, although a literature review is " crlf
"  time consuming to perform, it can probably be completed in the time that" crlf
"  it would take to repeat just one experiment. However, it should be noted" crlf
"  that experimental results cannot be expected to hold arbitrarily across" crlf
"  contexts. You should therefore select evidence carefully, noting the " crlf
"  experimental design chosen, the population of subjects used, the analyses" crlf
"  performed and the assumptions made." crlf crlf
"- Model-based evaluation: another approach to evaluating the system" crlf
"  theoretically is the use of models. Certain cognitive and design models" crlf
"  provide a means of combining design specification and evaluation into the " crlf
"  same framework. One example is the GOMS model (described in HCI-chapter 6)." crlf crlf
"Press enter to continue: ")
(readline)
(printout t crlf crlf
"  Some aspects of the behaviour of the system are not covered by cognitive " crlf
"  models. For these aspects the only way to test them theoretically is to " crlf
"  find a way to model your system or technique in a formal way using " crlf
"  mathematical tools or other. You may then be able to prove some properties " crlf
"  you want to test. You can also exploit the theory about modelling that " crlf
"  already exists. There are a lot of ways to model a system and derive " crlf
"  performance measures, e.g. the operational laws are simple equations which " crlf
"  may be used as an abstract representation or model of the average behaviour " crlf
"  of almost any system. " crlf crlf
"Press enter to continue: ")
(readline))

(defrule ADVICE::4-8
  (advice 4-8)
=>
  (printout t crlf crlf
"  Validation is the task of demonstrating that the model is a reasonable " crlf
"  representation of the actual system/technique: that it reproduces system/technique " crlf
"  behaviour with enough fidelity to satisfy analysis objectives (MS-section 15). " crlf
"  Validation is usually specific to the model, and system/technique in question. "crlf
"  There are three approaches to model validation (taken from MS-section 15)"crlf crlf
"- Expert intuition." crlf 
"  According to this approach an \"expert\", with respect to the system/technique"
" (who should be other than the modeller, who is an \"expert\" with respect to " crlf
"  the model) will be asked to examine the model. The model should be fully " crlf
"  instrumented, i.e. every possible performance measure should be able to be "crlf 
"  extracted from the model for validation purposes regardless of the objectives " crlf
"  of the performance study. " crlf crlf
"- Real system measurements. "crlf
"  Comparison with a real system is a reliable way to validate a model. In " crlf
"  practice, however, this is often infeasible either because the real system " crlf
"  doesn't exist or because the measurements would be too expensive to carry out." crlf
"  Assumptions, input values, output values, workloads, configurations and " crlf
"  system behaviour should all be compared with those observed in the real " crlf
"  world. In the case of simulation models, when full measurement data is " crlf
"  available it may be possible to use trace-driven simulation to observe the " crlf
"  model under exactly the same conditions as the real system. " crlf crlf
"Press enter to continue: ")
(readline)
(printout t crlf crlf
"- Theoretical results/analysis. " crlf
"  Sometimes it is possible to use a more abstract representation of the " crlf
"  system/technique than the one you've used to model the system/technique, to" crlf
"  provide a crude validation of the model. E.g. if you haven't used operational " crlf
"  laws to model your system (MS-section 2) you can test if an operational analysis" crlf
"  will give results that coincide with your model's output and if it does then " crlf
"  it can be taken as evidence that the model behaves correctly. " crlf
"  Validation of models against the results or behaviour of other models is a " crlf
"  technique which should be used with care as both may be invalid in the " crlf
"  sense that they both may not represent the behaviour of the real system/technique " crlf
"  accurately. " crlf crlf
"  Another analytic approach is to determine invariants which must hold in " crlf
"  every state of the system (or every step of the technique). Showing that the " crlf
"  model always satisfies such an invariant is one way of increasing confidence " crlf
"  in the model, and providing support for its validity. The disadvantage of such " crlf
"  an approach is that it can be computationally expensive to carry out the " crlf
"  necessary checks regularly within a model. " crlf crlf
"Press enter to continue: ")
(readline))

(defrule ADVICE::4-9
  (advice 4-9)
=>
  (printout t crlf crlf
" Some techniques for model verification (taken from MS-section 15) are:" crlf crlf
"- Antibugging. " crlf 
"  Antibugging consists of including additional checks and outputs in a model " crlf
"  that may be used to capture bugs if they exist. These are features of the " crlf
"  model which do not have a role in representing the system, or even " crlf
"  necessarily in calculating performance measures. Their only role is to " crlf
"  check the behaviour of the model." crlf crlf
"- Structured walk-through/one-step analysis. " crlf 
"  Explaining the model to another person, or group of people, can make you " crlf
"  focus on different aspects of the model and therefore discover problems " crlf
"  with its current implementation. Even if the listeners do not understand " crlf
"  the details of the model, or the system/technique, you may become aware of " crlf
"  bugs simply by studying the model carefully and trying to explain how it " crlf
"  works. Preparing documentation for a model can have a similar effect by " crlf
"  making you look at the model from a different perspective. In the absence " crlf
"  of a willing audience you should try to carry out the same sort of step-by-step" crlf
"  analysis of the model to convince yourself that it behaves correctly. Some " crlf
"  modelling paradigms provide support for doing this. " crlf crlf
"Press enter to continue: ")
(readline)
  (printout t crlf crlf
"- Simplified models. " crlf
"  It is sometimes possible to reduce the model to its minimal possible " crlf 
"  behaviour. Since one-step analysis, described earlier, can be extremely " crlf
"  time-consuming it is often applied to a simplified model. Of course, a model " crlf
"  that works for simple cases is not guaranteed to work for more complex " crlf
"  cases. On the other hand, a model which does not work for simple cases " crlf
"  will certainly not work for more complex ones." crlf crlf
"- Continuity testing. " crlf
"  At an abstract level all systems, techniques and models can be thought of as " crlf
"  generating a function from input values to output values, and in most cases, " crlf
"  we expect that function to be continious. In other words, in most cases, we " crlf
"  do not anticipate that a slight change in input will result in very large " crlf
"  changes in the corresponding output value. Continuity testing consists of solving " crlf
"  the model, or running a simulation model, several times for slightly " crlf
"  different values of input parameters. For any one parameter, a slight change " crlf
"  in input should generally produce only a slight change in the output. Any " crlf
"  sudden changes in the output are taken to be an indication of a possible " crlf
"  error which should be investigated unless this is known behaviour of the " crlf
"  system/technique. " crlf crlf
"Press enter to continue: ")
(readline)
  (printout t crlf crlf
"- Degeneracy testing. " crlf
"  Degenerate cases for a model are those values of input parameters which are " crlf
"  at the extremes of the model's intended range of representation. Degeneracy " crlf
"  testing consists of checking that the model works for the extreme values of " crlf
"  system/technique and workload (input) parameters. Although extreme cases may not " crlf
"  represent typical cases, degeneracy testing can help you find bugs that " crlf
"  would not otherwise have been discovered. " crlf crlf
"- Consistency testing. " crlf
"  For most models and systems it is reasonable to assume that similarly" crlf
"  loaded systems will exhibit similar characteristics, even if the arrangements " crlf
"  of the workload varies. Consistency tests are used to check that a model " crlf
"  produces similar results for input parameter values that have similar effects. " crlf
"  If the model output shows a significant difference, either it should be " crlf
"  possible to explain the difference from more detailed knowledge of the " crlf
"  system, or the possibility of a modelling error should be investigated." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-10
  (advice 4-10)
  (field ?field)
=>
  (printout t crlf crlf
" Here are some standard solutions for obtaining representative sets of " crlf
" examples (taken from AIRM - Criteria for assessing AI research, section 6.1):" crlf crlf
"- Distinguish training from test examples: The generality of a technique can " crlf
"  be demonstrated by selecting a set of test examples and not looking at them " crlf
"  or running the technique on them until it has been fully debugged on a " crlf
"  separate training set. This guards against the accusation of special tuning. " crlf
"  Of course, to be fair to the technique, the training set should itself be " crlf
"  representative of the range and complexity of problems that will be encounter " crlf
"  in the test set. " crlf crlf
"- Use a lot of dissimilar examples: Showing that a technique is successful " crlf
"  and/or efficient on a lot of dissimilar examples in itself provides evidence " crlf
"  that the technique is widely applicable. In addition it becomes harder to " crlf
"  bias the example set in favour of the technique if the set is large and " crlf
"  heterogeneous. " crlf crlf
"Press enter to continue: ")
(readline)
  (printout t crlf crlf
"- Collect examples from an independent source: If some third party has provided " crlf
"  the examples and they have not been subsequently filtered then we have " crlf
"  evidence that they are not biased in favour of any technique. However, care " crlf
"  must be taken to show that the source is truly independent,i.e. is not, due " crlf
"  to origin, inherently biased. For instance, if the technique is aimed at " crlf
"  some final application then it is good if the examples have arisen from " crlf
"  that application. This might even be taken as a definition of representative. " crlf crlf
"- Use the shared examples of the field, for you: "crlf 
"  \" " ?field " \". " crlf 
"  If the field has developed a standard test set then it aids comparison of " crlf
"  rival techniques if they are tested on this shared set. This does not make " crlf
"  the test set representative. Indeed there is a real danger that the test set " crlf
"  is provided by the founder of the field and is biased in favour of the " crlf
"  techniques that they developed. One of the aims of the field  should be to " crlf
"  modify the standard test set to make it more representative. " crlf crlf
"- Use challenging examples: It is easy to get a high success rate on easy " crlf
"  problems, but this does not aid behavioural or coverage comparison with " crlf
"  other techniques. The technique needs to be tested on examples that have " crlf
"  caused problems for other techniques. " crlf crlf
"- Use pathological examples: Most fields pass around a selection of pathological " crlf
"  examples. These are of most use during the development phase for exposing " crlf
"  over-simplifications, missing cases, false assumptions, etc. in the technique. " crlf
"  Success on them also demonstrates the generality of the technique, although " crlf
"  pathological examples are in some sense the opposite of representative. " crlf crlf
"Press enter to continue: ")
(readline))

(defrule ADVICE::4-11
  (advice 4-11)
=>
  (printout t crlf crlf
" TEST EFFICIENCY " crlf
" In order to empirically test the efficiency of the system you just have to " crlf
" measure how much memory it uses (if space is considered) or how much CPU time " crlf
" it uses (if time is considered) in some sets of test examples. Each example " crlf
" will represent a point P(x,y) where x is the size of input and y is the " crlf
" space or time. Then you can plot the graph of the space or time at y-axis " crlf
" and the size of input at x-axis by connecting these points. You can also " crlf
" transform dots before plotting e.g. by using logarithmic graph paper. Then, " crlf
" by doing curve fitting you can declare if the complexity is exponential, "crlf
" polynomial etc. " crlf crlf
"Press enter to continue: ")
 (readline))


(defrule ADVICE::4-12
  (advice 4-12)
=>
  (printout t crlf crlf
" TEST EFFICIENCY " crlf
" In order to evaluate the system in terms of efficiency theoretically you " crlf
" need to use complexity analysis. You can do complexity analysis to measure " crlf
" efficiency in terms of space or time. You can calculate the efficiency of " crlf
" all the program, which is very hard to do theoretically, or of a specific "crlf
" procedure. Complexity analysis measures efficiency in terms of an order of " crlf
" magnitude estimate. e.g. that the cpu time is a linear, quadratic or " crlf
" exponential function of the size of input to a procedure. Any polynomial " crlf
" function is usually considered acceptable, but exponential or worse is " crlf
" considered unacceptably inefficient. Most estimates are for worse case " crlf
" performance, but you can also do an average case analysis. " crlf
" More details on complexity analysis can be found in ALG-chapter 6. " crlf crlf
"Press enter to continue: ")
(readline))

(defrule ADVICE::4-13
  (advice 4-13)
=>
  (printout t crlf crlf
" TEST EFFICIENCY " crlf
" If you want to measure efficiency theoretically when a program is not " crlf
" involved, or you don't have access to it, then the only way is to find " crlf
" a way to model your system or technique in a formal way using mathematical " crlf
" tools or other. You may then be able to work out the time or space as a " crlf
" function of the size of input. You can also exploit the theory about " crlf
" modelling that already exists. There are a lot of ways to model a system and " crlf
" derive performance measures like response time, service rate or waiting time, " crlf
" to measure efficiency. One example is the operational laws (for details see " crlf
" MS-section 2), which are simple equations that may be used as an abstract " crlf
" representation or model of the average behaviour of almost any system."
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-14
  (advice 4-14)
=>
  (printout t crlf crlf
" TEST COVERAGE " crlf
" Testing coverage of a technique means testing the range of its application. " crlf
" Total coverage is called completeness,i.e. a complete technique behaves " crlf
" correctly in all situations. This is rarely the case and you must be prepared " crlf
" for incorrect or no behaviour in some situations. A technique may also vary " crlf
" in the quality of its behaviour and in its efficiency in different parts of " crlf
" its range of application. So investigations of coverage often consist of " crlf
" identifying patterns in changes of behaviour and efficiency in different " crlf
" situations. It is often informative to use graphical and statistical methods " crlf
" to explore, hypothesise and test such patterns. There are words that describe " crlf
" certain kinds of such patterns. For instance, robustness refers to quality of " crlf
" a uniformly high standard of behaviour across a wide (and maybe predictable) " crlf
" range of inputs. Brittleness refers to the frequent occurrence of low quality " crlf
" behaviour within the apparent range of a technique. " crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-15
  (advice 4-15)
=>
  (printout t crlf crlf
" In order to test the coverage of a system/technique experimentally you need " crlf
" to conduct experiments where you feed input into your system/technique and " crlf
" see how well your system/technique behaves, if at all. Each input represents " crlf
" a different situation that the system/technique may be requested to deal with. " crlf
" First you have to identify the range of applications where the system/technique " crlf
" behaves correctly, i.e. does the task and produces correct results. That will " crlf
" define the coverage of your system. But usually that is not enough. You have " crlf
" to identify patterns in changes of behaviour or efficiency in different " crlf
" situations. E.g. you may find out that your system behaves very well for a " crlf
" range of applications and not so well, i.e. reduced quality or efficiency, " crlf
" in some other range. " crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-16
  (advice 4-16)
=>
  (printout t crlf crlf
" In order to evaluate your system theoretically in terms of coverage you need " crlf
" to be able to represent your system in a formal way using mathematical tools " crlf
" or other. You can also use existing modelling techniques. Then you will be " crlf
" able to predict the behaviour of the system in each of some test examples and " crlf
" identify in what range of examples is the system applicable, in what range " crlf
" it behaves correctly and also identify patterns in changes of behaviour or " crlf
" efficiency in different situations. E.g. you may find out that your system " crlf
" behaves very well for a range of applications and not so well, i.e. reduced " crlf
" quality or efficiency, in some other range. You may have to prove a theorem " crlf
" in order to justify your claims." crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-17
  ?a4-17 <- (advice 4-17)
=>
  (printout t crlf crlf
" COMPARE RIVALS IN TERMS OF BEHAVIOUR " crlf
" In order to use empirical evaluation to support your claims about the " crlf
" behaviour of the system compared to some rival one you need to conduct " crlf
" a controlled experiment for each claim you make. Any experiment has the " crlf
" same basic form. First you choose a hypothesis to test, which can be " crlf
" determined by measuring some attribute of subject behaviour. A number of " crlf
" experimental conditions should be considered which differ only in the values " crlf
" of certain control variables, which here will be sets of test examples. " crlf
" Every other aspect should stay the same! (If that is not possible you should find " crlf
" ways to counteract the changes in other conditions or make sure that their " crlf
" influence on the results has been taken into account). The subject can have a " crlf
" session with both rival systems with the same sets of examples and the property " crlf
" of behaviour tested should be measured." crlf crlf
"Press enter to continue: ")
  (readline)
  (printout t crlf crlf
" There are two experimental methods: " crlf crlf
"- Between groups (or randomized) design, where each subject is assigned to a " crlf
"  different condition. It has the advantage that any  learning effect " crlf
"  resulting from the user performing in one condition and then the other is " crlf
"  controlled and the disadvantage that a greater number of subjects is required " crlf
"  and that individual differences between users can bias the results. " crlf crlf
"- Within groups, where each user performs under each different condition. " crlf
"  This design can suffer from transfer of learning effects, but this can be " crlf
"  lessened if the order in which conditions are tackled is varied between users." crlf
"  Within-groups is less costly than between groups, since fewer users are " crlf
"  required, and there is also less chance of effects from variation between " crlf
"  subjects. " crlf crlf
"Press enter to continue: ")
  (readline)
  (printout t crlf crlf
" Within the basic form there are a number of factors that are important to " crlf
" the overall reliability of the experimental design. Thus, you should note that: " crlf crlf
"- Subjects should be chosen to match the expected user population as closely " crlf
"  as possible. Ideally this would involve experimental testing with the actual " crlf
"  users but this is not always possible. Also the sample size should be large " crlf
"  enough to be representative of the population. " crlf crlf
"- Differences in experimental evidence may only be due to chance. Statistical " crlf
"  analysis may be required to show that these differences are significant " crlf
"  (e.g. t-test, described in GAGP-appendix A)." crlf crlf
"- Experimental conditions should only differ in the control variable and be " crlf
"  otherwise identical. It is usually very hard to tell what causes the " crlf
"  difference, when you find differences in two systems, like e.g. why X is " crlf
"  better than Y, as usually complete systems differ from each other in more " crlf
"  than one feature. If your claim is that e.g. the X system behaves better " crlf
"  because of a specific feature, then you have to make sure that the systems " crlf
"  only differ in this feature. Otherwise micro-analysis of system behaviour " crlf
"  may be necessary to confirm the real cause of behavioural differences. " crlf crlf
"- The property of interest must be measurable in some way. " crlf crlf
"Press enter to continue: ")
  (readline)
  (printout t crlf crlf
" When the results for both systems are taken, a suggested way to compare " crlf
" the results is to do a scatter plot for each property of interest. Each point" crlf
" will represent a test example, with its x value being the property of interest " crlf
" of the X system for this example and its y value being the property of interest " crlf
" of the Y system for this example. Then you can fit a line to these points and " crlf
" measure the angle. If it is 45 degrees then the systems exhibit the same " crlf
" behaviour across the range of these examples, if it is less than 45 degrees " crlf
" then X is better at this property than Y and if it is bigger than 45 then X " crlf
" is worse at this property than Y." crlf crlf
"Press enter to continue: ")
  (readline)
(retract ?a4-17))

(defrule ADVICE::4-18
  ?a4-18 <- (advice 4-18)
=>
  (printout t crlf crlf
" COMPARE RIVALS IN TERMS OF BEHAVIOUR " crlf
" It is very difficult to compare the behaviour of two systems theoretically. " crlf
" The only suggestion is to build a formal model for each system and try to " crlf
" predict the behaviour of each one (using mathematical tools or other modelling " crlf
" techniques) for some sets of test examples. " crlf crlf
" When the results for both techniques are taken, you can either find a " crlf
" theoretical method of comparing the results or you can use empirical methods " crlf
" like statistical or graphical techniques. " crlf
" One suggested way is to do a scatter plot for each property of interest. Each " crlf
" point will represent a test example, with its x value being the property of " crlf
" interest of the X system for this example and its y value being the property " crlf
" of interest of the Y system for this example. Then you can fit a line to these " crlf
" points and measure the angle. If it is 45 degrees then the systems exhibit the " crlf
" same behaviour across the range of these examples, if it is less than 45 degrees " crlf
" then X is better at this property than Y and if it is bigger than 45 then X is " crlf
" worse at this property than Y. Note however that the behaviour is only a " crlf
" prediction. An empirical test with the real systems has to take place. " crlf crlf
"Press enter to continue: ")
  (readline)
  (printout t crlf crlf
" It is very difficult to tell what causes the difference, why e.g. X is better " crlf
" than Y as usually complete systems differ in more than one features from each " crlf
" other. If your claim is that e.g. the X system behaves better because of a " crlf
" specific feature, then you have to model the systems in a way so that they only " crlf
" differ in that feature." crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-18))

(defrule ADVICE::4-19
  ?a4-19 <- (advice 4-19)
=>
  (printout t crlf crlf
" COMPARE RIVALS IN TERMS OF EFFICIENCY" crlf
" In order to do an experimental evaluation to compare two rival systems in terms " crlf
" of efficiency you need to conduct an experiment where you will measure the " crlf
" time or space for each system by feeding a different input size each time. " crlf crlf
" When the results for both systems are taken, a suggested way to compare the " crlf
" results is to do a scatter plot. Each point will represent an input size you " crlf
" fed into each system, with its x value being the time for system X for this " crlf
" example and its y value being the time for system Y for this example." crlf
" Then you can fit a line to these points and measure the angle. If it is " crlf
" 45 degrees then the systems are equally efficient across the range of these " crlf
" examples, if it is less than 45 degrees then X is more efficient than Y and if " crlf
" it is bigger than 45 then X is less efficient than Y. It is very difficult to " crlf
" tell what causes the difference, why e.g. X is more efficient than Y as usually " crlf
" complete systems differ in more than one features from each other. If your " crlf
" claim is that e.g. the X system is more efficient because of a specific feature, " crlf
" then you have to make sure that the systems only differ in this feature. " crlf
" Otherwise micro-analysis of system behaviour may be necessary to confirm the " crlf
" real cause of the difference. " crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-19))

(defrule ADVICE::4-20a
  ?a4-20a <- (advice 4-20a)
=>
  (printout t crlf crlf
" COMPARE RIVALS IN TERMS OF EFFICIENCY " crlf
" In order to compare the two systems theoretically, in terms of efficiency " crlf
" and given that programming is involved you can use complexity analysis. " crlf
" You can do complexity analysis to measure efficiency in terms of space or " crlf
" cpu time. You can calculate the efficiency of all the program or of a " crlf
" specific procedure. Complexity analysis measures efficiency in terms of an " crlf
" order of magnitude estimate, e.g. that the cpu time is a linear, quadratic " crlf
" or exponential function of the size of input to a procedure. Thus, you can " crlf
" calculate the complexity in terms of an order of magnitude estimate for each " crlf
" rival system. Then, you can draw conclusions like e.g. if the complexity is " crlf
" linear for system X and quadratic for system Y then system X is more efficient " crlf
" than system Y. However, you should note that complexity analysis is usually for " crlf
" worst case, i.e. very large inputs. An average case analysis may show different " crlf
" results." crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-20a))

(defrule ADVICE::4-20b
  ?a4-20b <- (advice 4-20b)
=>
  (printout t crlf crlf
" Another way to compare two interactive systems in terms of efficiency is " crlf
" using the GOMS model at the keystroke level. This model is described in " crlf
" HCI-chapter 6, and although it is not reliable for absolute measures of " crlf
" efficiency it can be very useful for relative measures, i.e. for testing " crlf
" if a system is more efficient than another one." crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-20b))

(defrule ADVICE::4-20c
  ?a4-20c <- (advice 4-20c)
=>
  (printout t crlf crlf
" You should note that, if your claim is that e.g. the X system is more efficient " crlf
" than the Y because of a specific feature, then you have to make sure that the " crlf
" systems only differ in this feature. Otherwise, you cannot make such a claim." crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-20c))

(defrule ADVICE::4-21
  ?a4-21 <- (advice 4-21)
=>
  (printout t crlf crlf
" COMPARE RIVALS IN TERMS OF EFFICIENCY " crlf
" One way to theoretically compare two systems in terms of efficiency, when " crlf
" programming is not involved, is to find a formal model for each system and " crlf
" use mathematical tools or other modelling techniques to compare their " crlf
" efficiency. For example you can use operational laws to calculate the " crlf
" throughput for each of the two rival systems, for different test examples." crlf
" Then you can compare their average throughput and draw conclusions about " crlf
" which is more efficient." crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-21))

(defrule ADVICE::4-22
  ?a4-22 <- (advice 4-22)
=>
  (printout t crlf crlf
" COMPARE RIVALS IN TERMS OF COVERAGE " crlf
" In order to compare two rival systems experimentally in terms of the coverage " crlf
" you need to conduct experiments where you feed input into each system and see " crlf
" how well each system behaves, if at all. Each input represents a different " crlf
" situation that the system may be requested to deal with. First you have to " crlf
" identify the range of applications where each system behaves correctly, " crlf
" i.e. does the task and produces correct results. That will define the coverage " crlf
" of each system. But usually that is not enough. You have to identify patterns " crlf
" in changes of behaviour or efficiency in different situations. E.g. you may " crlf
" find out that system X behaves better for a range of applications and worse " crlf
" for another range, i.e. has worse quality or efficiency than system Y in some " crlf
" other range. " crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-22))

(defrule ADVICE::4-23
  ?a4-23 <- (advice 4-23)
=>
  (printout t crlf crlf
" COMPARE RIVALS IN TERMS OF COVERAGE" crlf
" In order to compare the rival systems theoretically in terms of coverage you " crlf
" need to be able to represent the systems in a formal way using mathematical " crlf
" tools or other. Then you will have to prove that the behaviour of the system " crlf
" is correct for some well defined range. Then you can prove that this range, is " crlf
" a superset of the range where the rival system behaves correctly. " crlf crlf
"Press enter to continue: ")
  (readline)
 (retract ?a4-23))

(defrule ADVICE::4-24
  (advice 4-24)
=>
  (printout t crlf crlf
" TEST BEHAVIOUR " crlf
" For a non-interactive system the suggested way to evaluate the system in terms " crlf
" of behaviour empirically is to let an expert in the task, the system does, " crlf
" evaluate the behaviour of the system. In that way you can establish behavioural " crlf
" properties like correctness and quality. Sometimes there are already established " crlf
" properties on what should be expected of the task to be correct or of good " crlf
" quality and you don't need an expert in the task. E.g. If you've built a robot " crlf
" to find its way through a maze then if you test the robot in different test " crlf
" examples and the robot finds its way through the maze, then its behaviour is " crlf
" correct; you don't need an expert to tell you that. Also, there are some times " crlf
" that the behaviour cannot be 100% correct. In these cases you need to measure " crlf
" accuracy, i.e. how far from 100% correct your result in each case is. This can be " crlf
" evaluated in some cases with the help of an expert and in others you can " crlf
" estimate it yourself when you know the absolute correct behaviour. " crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-25
  (advice 4-25)
=>
  (printout t crlf crlf
" TEST BEHAVIOUR " crlf
" It is very difficult to establish some behavioural properties theoretically. " crlf
" There are some properties like correctness that you can prove theoretically, " crlf
" but others like quality are usually hard to test. It depends on whether " crlf
" quality can be formally characterized. " crlf crlf
" If you have built a formal model for your system you may be able to predict " crlf
" properties like these for different situations. You can also do some " crlf
" theoretical research concerning the task your system performs. E.g. If you " crlf
" have built a robot arm to grasp objects you do some theoretical research into " crlf
" the task by using a combination of geometry and mechanics to investigate: " crlf
" optimal places and forces to hold an object, optimal trajectories for a robot" crlf 
" hand/arm to traverse, required accuracies in motors and joints for acceptable " crlf
" error rates etc. This analysis can then be used to evaluate the robot arm and " crlf
" its software. You might predict that the robot's behaviour would be of " crlf
" sufficiently high quality in terms of its success rate and accuracy. Of course " crlf
" you will eventually have to show that these theoretical predictions are borne " crlf
" out in practice by testing the real robot. " crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-26
  ?a4-26 <- (advice 4-26)
=>
  (printout t crlf crlf
" COMPARE RIVALS IN TERMS OF BEHAVIOUR " crlf
" For non-interactive systems or architectures the suggested way to compare them " crlf
" in terms of behaviour empirically, is by letting experts in the task to say " crlf
" which system behaves better. The expert should judge \"blindly\", i.e. the human " crlf
" expert should not know which system s/he judges each time, so that s/he doesn't " crlf
" favour one of them. In that way you can compare rivals in terms of behavioural " crlf
" properties like correctness, quality, simplicity etc. Not always is an expert " crlf
" necessary as a judge. E.g. If you've built a robot to play rugby and you want " crlf
" to compare it with a rival one you can put them to play against each other " crlf
" several matches and see which is the winner. You don't necessarily need an " crlf
" expert in rugby to judge, unless you want to assess other properties as well " crlf
" that do not depend on which robot wins the game. " crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-26))

(defrule ADVICE::4-27
  ?a4-27 <- (advice 4-27)
=>
  (printout t crlf crlf
" COMPARE RIVALS IN TERMS OF BEHAVIOUR " crlf
" It is very difficult to compare theoretically rival non-interactive systems or " crlf
" architectures. The best way is to do some theoretical research concerning the " crlf
" task, e.g. use mathematical tools or physics to model the task and then try to " crlf
" predict how each system will behave in different situations and compare the " crlf
" results. Any simplifying assumptions used should be justified and in the end " crlf
" you will have to test the real systems as well, as the theoretical analysis " crlf
" gives only predictions of the behaviour." crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-27))

(defrule ADVICE::4-28
  (advice 4-28)
=>
  (printout t crlf crlf
" TEST KNOWLEDGE IN TERMS OF BEHAVIOUR" crlf
" In order to test knowledge experimentally in terms of behaviour it is " crlf
" suggested to let an expert in the field , evaluate it. You may want to ask " crlf
" several experts to evaluate it so that you get different opinions. There are " crlf
" also some cases where you can use another method for evaluating the knowledge. " crlf
" You can assess the effect the system's knowledge has after a period of time. " crlf
" If e.g. this knowledge is intended to help students learn maths you can assess " crlf
" the knowledge by looking e.g. at their grades in maths after using the system " crlf
" for some time. This is a good method for evaluating the knowledge but it needs " crlf
" a lot of time as sometimes you may have to wait a lot of months or even years " crlf
" to get results. Behaviour of knowledge also includes the provision of additional " crlf
" information. E.g. explanation of why a solution is suggested, an advice " crlf
" given etc. " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-29
  (advice 4-29)
=>
  (printout t crlf crlf
" TEST KNOWLEDGE IN TERMS OF BEHAVIOUR " crlf
" To test the knowledge theoretically in terms of behaviour the only suggested " crlf
" way is to search the literature and try to find evidence to support the " crlf
" knowledge you've used. " crlf crlf
"Press enter to continue: ")
  (readline))



(defrule ADVICE::4-30
  (advice 4-30)
=>
  (printout t crlf crlf
" TEST KNOWLEDGE IN TERMS OF COVERAGE " crlf
" To test coverage of the represented knowledge experimentally you probably " crlf
" need to ask an expert to think of problems your knowledge doesn't tackle or " crlf
" make an experiment where you will use some example problems and see if your " crlf
" knowledge tackles these problems. You may find out that your knowledge deals " crlf
" better with a range of problems and worse with others. " crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-31
  (advice 4-31)
=>
  (printout t crlf crlf
" TEST KNOWLEDGE IN TERMS OF COVERAGE " crlf
" Sorry but it is beyond this system's functionality to give advice on how to " crlf
" theoretically test the knowledge of your system in terms of coverage. " crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-32
  (advice 4-32)
=>
  (printout t crlf crlf
" COMPARE THE KNOWLEDGE OF RIVALS IN TERMS OF BEHAVIOUR " crlf
" In order to compare experimentally your system's/technique's knowledge with " crlf
" rivals in terms of behaviour, it is suggested to let an expert in the field " crlf
" judge. You may want to ask several experts to evaluate it so that you get " crlf
" different opinions. The experts should judge \"blindly\" i.e. they should not " crlf
" know which knowledge belongs to which system/technique each time, so that the " crlf
" results are not biased. " crlf crlf
" There are also some cases where you can use another method for evaluating the " crlf
" knowledge. You can do a survey after the rival systems have been used for " crlf
" some time and ask which knowledge contributed more (in the field it was intended " crlf
" to contribute). However, this has the risk that the people's answers may be " crlf
" biased and a specific system/technique is favoured for a specific reason. When " crlf
" a new system is suggested there are always people that for certain reasons will " crlf
" not want it. Special analysis of social factors may be needed in order to use " crlf
" this method. " crlf
" Behaviour of knowledge also includes the provision of additional information. " crlf
" E.g. one system may provide explanation of why a solution is suggested, an " crlf
" advice given etc. while the other not." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-33
  (advice 4-33)
=>
  (printout t crlf crlf
" COMPARE THE KNOWLEDGE OF RIVALS IN TERMS OF BEHAVIOUR" crlf
" To compare the knowledge represented by two rival systems/techniques " crlf
" theoretically in terms of behaviour is difficult and it is beyond the " crlf
" scope of this system to give advice for that." crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-34
  (advice 4-34)
=>
  (printout t crlf crlf
" COMPARE THE KNOWLEDGE OF RIVALS IN TERMS OF COVERAGE " crlf
" To compare experimentally two rival systems/techniques in terms of coverage " crlf
" of the represented knowledge you probably need to do an experiment using " crlf
" certain test examples and see which system's/technique's knowledge deals " crlf
" correctly with the most examples. You may need to have an expert in the " crlf
" field to assess that the results are indeed correct e.g. if the system solves " crlf
" correctly a mathematical problem, if it finds the fastest way through a maze. " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-35
  (advice 4-35)
=>
  (printout t crlf crlf
" COMPARE THE KNOWLEDGE OF RIVALS IN TERMS OF COVERAGE " crlf
" Sorry but it is beyond this system's functionality to give advice on how to " crlf
" compare theoretically the knowledge of two rival systems in terms of coverage." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-36
  (advice 4-36)
=>
  (printout t crlf crlf
" You should have a formal representation of your technique (either an " crlf
" algorithm or some formal model) in order to be able to evaluate it. " crlf
" Otherwise it is not possible to model the behaviour of the technique for " crlf
" different test examples. There should be a way to get the output/result of " crlf
" your technique for different inputs and this cannot be done if your " crlf
" technique is not represented in a formal way in terms of the input it " crlf
" gets and the output it produces. I'm sorry but otherwise I cannot help you." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-37
  (advice 4-37)
=>
  (printout t crlf crlf
" For an experimental evaluation of your technique, that exists only in " crlf
" theory, you can try to perform a hand simulation (or structured walk-through)," crlf
" by explaining how the technique works at each step (going through it step by " crlf
" step) and justifying, that way, the behaviour you predict it will produce " crlf
" for certain inputs. " crlf crlf
" Otherwise, you will need to build a system based on your theory or make " crlf
" simulation of a system, in order to perform expermental evaluation. Then " crlf
" you can empirically verify your theory by testing the properties of your " crlf
" system. However, even in this case testing that the system really produces " crlf
" a certain behaviour is not enough. You also need to prove that:" crlf crlf
"- The system really depends upon that theory."crlf
"- The aspects of the system that depend on that theory are really crucial " crlf
"  at producing that behaviour." crlf crlf
"Press enter to continue: ")
  (readline)
  (printout t crlf crlf
" However, if your research is theoretical then it doesn't help to do an " crlf
" experimental evaluation.It will be more helpful to do a theoretical evaluation." crlf crlf
))


(defrule ADVICE::4-38
  (advice 4-38)
=>
  (printout t crlf crlf
" As your theory involves some algorithms, you can prove some behavioural " crlf
" properties by claiming that your algorithm(s) produces a certain range of " crlf
" outputs for a certain range of inputs and then justify this claim by " crlf
" proving the correctness of your algorithm in producing that output for " crlf
" the specific input. Details about the correctness of algorithms can be " crlf
" found in ALG-chapter 5. You can then prove that the output produced is " crlf
" correct or of good quality, or has whatever property you wanted to test. " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-39
  (advice 4-39)
=>
  (printout t crlf crlf
" As your theory involves a formal model of your technique then you may be " crlf
" able to prove some behavioural properties, like e.g. correctness for some " crlf
" well defined range of application, but there may be others like quality, " crlf
" that are usually harder to test when there is no implementation. Usually " crlf
" it depends on whether the property can be formally characterized. Also, if " crlf
" your technique is modeled using a modelling technique (like those discussed " crlf
" in MS) then you can easily predict some behavioural properties of the system " crlf
" that is modeled to implement the technique. You should though make sure that " crlf
" the behaviour predicted is due to aspects of the model that are based on the " crlf
" technique." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-40
  (advice 4-40)
=>
  (printout t crlf crlf
" As your theory involves some algorithms you may be able to test the efficiency " crlf
" of your technique using complexity analysis of algorithms. You can do " crlf
" complexity analysis to measure efficiency in terms of space or time. You can " crlf
" calculate the efficiency of one algorithm, or of a specific process, i.e. a " crlf
" combination of algorithms. Complexity analysis measures efficiency in terms " crlf
" of an order of magnitude estimate, e.g. that the time is a linear, quadratic " crlf
" or exponential function of the size of input to a process. Any polynomial " crlf
" function is usually considered acceptable, but exponential or worse is " crlf
" considered unacceptably inefficient. Most estimates are for worse case " crlf
" performance, but you can also do an average case analysis. " crlf
" More details on complexity analysis of algorithms can be found in ALG-chapter 6." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-41
  (advice 4-41)
=>
  (printout t crlf crlf
" As there exists a formal model for your technique you can measure efficiency " crlf
" using the formal model of your technique. You may be able to work out the " crlf
" time or space as a function of the size of input you will feed into your " crlf
" model. You can also exploit the theory about modelling that already exists." crlf
" There are a lot of ways to derive performance measures from a model, like " crlf
" response time, service rate or waiting time, to measure efficiency. One " crlf
" example is the operational laws (for details see MS-section 2). " crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-42
  (advice 4-42)
=>
  (printout t crlf crlf
" As your theory involves some algorithms you can test coverage by claiming " crlf
" that your algorithm(s) produces correct output, i.e. the output it is " crlf
" supposed to produce, for a certain, well defined range of inputs and you" crlf
" can justify this claim by proving the correctness of your algorithm in " crlf
" producing that output for the specific input. Details about the " crlf
" correctness of algorithms can be found in ALG-chapter 5." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-43
  (advice 4-43)
=>
  (printout t crlf crlf
" As there exists a formal model for your technique you can test coverage " crlf
" by feeding inputs into your model and predicting the results the technique " crlf
" is going to produce. That way you can prove in what range of inputs your " crlf
" technique gives correct results and also identify patterns in changes of " crlf
" behaviour or efficiency in different situations. E.g. you may find out that " crlf
" your technique behaves very well for a range of applications and not so well," crlf
" i.e. reduced quality or efficiency, in some other range. You should have well" crlf
" formed definitions of what correctness or quality means for your technique. " crlf
" You may also have to prove a theorem in order to justify your claims." crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-44
  ?a4-44 <- (advice 4-44)
=>
  (printout t crlf crlf
" For an experimental evaluation of a technique that exists only in theory " crlf
" you can try to perform a hand simulation (or structured walk-through)," crlf
" by explaining how the technique works at each step (going through it " crlf
" step by step) and justifying, that way, that the behaviour you predict " crlf
" for your technique is better than the one predicted for the rival " crlf
" techniques for certain inputs." crlf crlf
" Otherwise, you will need to build a system based on your theory or make " crlf
" simulation of a system, in order to perform experimental evaluation. Then " crlf
" you can empirically compare your technique with rival techniques, which " crlf
" will probalbly have already established properties. However, even in this " crlf
" case, testing that the system based on your technique/theory really " crlf
" produces a \"better\" behaviour than a rival technique is not enough. You also " crlf
" need to prove that:" crlf crlf
"- The system really depends upon that theory."crlf 
"- The aspects of the system that depend on that theory are really crucial " crlf
"  at producing that behaviour." crlf crlf
"Press enter to continue: ")
  (readline)
  (printout t crlf crlf
" However, if your research is theoretical then it doesn't help to do an" crlf
" experimental evaluation. It will be more helpful to do a theoretical " crlf
" evaluation." crlf crlf)
  (retract ?a4-44))

(defrule ADVICE::4-45
  ?a4-45 <- (advice 4-45)
=>
  (printout t crlf crlf
" As there exists some algorithm for your technique you can predict some " crlf
" behavioural properties of it by testing what output the algorithm will produce " crlf
" for certain input and then justify this by proving the correctness of  your " crlf
" algorithm. You can do that for the rival technique as well unless it already " crlf
" has established properties. More for the correctness of algorithms you can " crlf
" find in ALG." crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-45))

(defrule ADVICE::4-46
  ?a4-46 <- (advice 4-46)
=>
  (printout t crlf crlf
" As there exists a formal model for your technique you can try to predict what " crlf
" kind of results your technique produces in different situations and compare " crlf
" those results with the ones the rival technique produces for the same situation. " crlf
" You may want to use some techniques for deriving performance measures like " crlf
" those described in MS, if you have used one of the modelling technique " crlf
" described there." crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-46))


(defrule ADVICE::4-45-6
  ?a4-45-6 <- (advice 4-45-6)
=>
  (printout t crlf crlf
" When the results for both techniques are taken, you can either find a " crlf
" theoretical method of comparing the results or you can use empirical " crlf
" methods like statistical or graphical techniques. " crlf crlf
" One suggested way is to do a scatter plot for each property of interest." crlf
" Each point will represent a different situation, with its x value being the " crlf
" property of interest of the X system for this example and its y value being " crlf
" the property of interest of the Y system for this example. Then you can fit " crlf
" a line to these points and measure the angle. If it is 45 degrees then the " crlf
" techniques exhibit the same behaviour across the range of these examples, if " crlf
" it is less than 45 degrees then X is better at this property than Y and if " crlf
" it is bigger than 45 then X is worse at this property than Y. It is very " crlf
" difficult to tell what causes the difference, why e.g. X is better than Y " crlf
" as usually complete systems differ in more than one features from each other. " crlf
" If your claim is that e.g. the X system behaves better because of a specific " crlf
" feature, then you have to model the systems in a way so that they only differ " crlf
" in that feature, but still represent the technique correctly. Note however that " crlf
" the behaviour is only a prediction. An empirical test with the real systems " crlf
" has to take place." crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-45-6))

(defrule ADVICE::4-47
  ?a4-47 <- (advice 4-47)
=>
  (printout t crlf crlf
" As some algorithm is involved you can use complexity analysis to compare " crlf
" your technique with some rival technique in terms of efficiency. You can " crlf
" calculate the efficiency of a specific algorithm or of a whole process. " crlf
" Complexity analysis measures efficiency in terms of an order of magnitude " crlf
" estimate, e.g. that the cpu time is a linear, quadratic or exponential " crlf
" function of the size of input. Thus, you can calculate the complexity in " crlf
" terms of an order of magnitude estimate for each rival technique. Then, " crlf
" you can draw conclusions like e.g. if the complexity is linear for system X " crlf
" and quadratic for system Y then system X is more efficient than system Y. " crlf
" However, you should note that complexity analysis is usually for worst case," crlf
" i.e. very large inputs. An average case analysis may show different results." crlf
" You should also note that in order to claim that your technique is more " crlf
" efficient than another technique because of a specific algorithm or part of" crlf
" an algorithm the techniques should only differ in that algorithm (or part of" crlf
" the algorithm) and be otherwise identical. If that is not the case you may " crlf
" need to make some more analysis to specify the cause of the difference " crlf
" between the techniques, e.g. prove that a specific part of the algorithm that " crlf
" only exists in the rival technique causes the slowness of that technique. " crlf
" Thus, this can be the cause that yours is more efficient. " crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-47))

(defrule ADVICE::4-48
  ?a4-48 <- (advice 4-48)
=>
  (printout t crlf crlf
" As a formal model exists for your technique you can compare the efficiency" crlf
" of your technique with the efficiency of a rival one by deriving performance " crlf
" measures like response time or waiting time and comparing them with the " crlf
" corresponding measures of the rival techniques for the same test examples. " crlf
" You can use e.g. operational laws (described in MS) to calculate the " crlf
" throughput for each of the techniques or use other mathematical tools, prove " crlf
" some theorems, etc. It depends on the way you've modeled your technique and " crlf
" the kind of technique , which method you'll choose. The other rival technique " crlf
" will probably have already established properties, otherwise you will need " crlf
" to model the rival technique as well. You should note that you have to use " crlf
" the same test examples for all the techniques you are comparing. You would " crlf
" probably need to test the rival technique in some other test examples than " crlf
" the ones that it has already been tested on, as those were probably chosen " crlf
" to favour that technique. You should also be careful not to attribute the " crlf
" difference in efficiency of the techniques to a specific feature that exists " crlf
" in one of them, unless the techniques only differ in that feature." crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-48))


(defrule ADVICE::4-49
  ?a4-49 <- (advice 4-49)
=>
  (printout t crlf crlf
" As some algorithm is involved in your theory/technique you can compare " crlf
" the techniques in terms of coverage by claiming that your algorithm(s) " crlf
" produces correct output, i.e. the output it is supposed to produce, for " crlf
" a certain, well defined range of inputs and you can justify this claim by " crlf
" proving the correctness of your algorithm in producing that output for the" crlf
" specific input. Details about the correctness of algorithms can be found " crlf
" in ALG-chapter. Then you will have to prove that this range, is a superset" crlf
" of the range where the rival technique's algorithm produces correct results." crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-49))

(defrule ADVICE::4-50
  ?a4-50 <- (advice 4-50)
=>
  (printout t crlf crlf
" As a formal model exists for your theory/technique you can prove that the " crlf
" behaviour of your technique, predicted from the model, is correct for some " crlf
" well defined range of inputs. Then you will have to prove that this range, " crlf
" is a superset of the range where the rival theory/technique behaves correctly." crlf crlf
"Press enter to continue: ")
  (readline)
  (retract ?a4-50))


(defrule ADVICE::4-51
  (advice 4-51)
=>
  (printout t crlf crlf
" If you have improved a lot of aspects of the technique then to prove that " crlf
" the technique is \"better\" than it used to be is like proving that your " crlf
" technique (improved version) is \"better\" than a rival technique (previous " crlf
" version). Thus, I will give you the advice I would give you if you were to" crlf
" compare two rival techniques." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-52
  (advice 4-52)
=>
  (printout t crlf crlf
" If you have extended your technique by changing a lot of parameters in it" crlf
" then to prove that the technique covers indeed a wider range of applications " crlf
" than it used to cover is like proving that your technique (extended version) " crlf
" is \"better\" than a rival technique (previous version) in terms of coverage. " crlf
" Thus, I will give you the advice I would give you if you were to compare two " crlf
" rival techniques in terms of coverage." crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-53
  (advice 4-53)
=>
  (printout t crlf crlf
" As the technique is applied to a system in order to test the technique " crlf
" you can test the system implementing the technique. You should, though, " crlf
" prove either that the system is completely based on the technique or that" crlf
" the aspects of the system based on the technique are crucial for the system " crlf
" to produce the specific behaviour." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-54
  (advice 4-54)
=>
  (printout t crlf crlf
" As the technique is improved by changing only one parameter the best way to " crlf
" do evaluation and prove that the technique has indeed been improved at a " crlf
" specific property is to prove that the parameter you've changed in the " crlf
" technique causes this improvement. You may e.g. have changed an algorithm " crlf
" of the technique and be able to prove that this algorithm caused e.g. the " crlf
" slowness of the technique before been changed, but is now very efficient." crlf
" However, you can still use the same techniques as when comparing your technique " crlf
" with a rival one (the rival technique being the previous version of the " crlf
" technique you've improved). Maybe the properties are already established for" crlf
" the previous version of the technique, but you have to take into consideration " crlf
" that the test examples used for the previous version have to be the same as" crlf
" those for the improved version. You may want to do some more tests with other" crlf
" test examples for the previous version of the technique in order to be sure " crlf
" that the test examples used were not chosen to favour the technique." crlf 
" The methods for evaluation that will be suggested will be almost the same as those " crlf
" used for comparing rival techniques. The improved version will be your technique " crlf
" and the previous version the rival technique " crlf crlf 
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-55
  (advice 4-55)
=>
  (printout t crlf crlf
" As the technique is extended by changing only one parameter the best way " crlf
" to do evaluation and prove that the technique has indeed been extended is " crlf
" to prove that the parameter you've changed in the technique causes it to " crlf
" cover a wider range of applications (or produce good bahaviour or efficiency " crlf
" for a wider range of application). You may e.g. have changed an algorithm of " crlf
" the technique and be able to prove that this algorithm caused e.g. the previous " crlf
" version not to behave correctly for a specific range of applications and thus "  crlf
" now that is changed the technique covers this range. However, you can still " crlf
" use the same techniques as in comparing rival techniques, with the rival " crlf
" technique being the previous version of the technique you've improved. Maybe " crlf
" the coverage is already been established for the previous version of the " crlf
" technique, but you have to take into consideration that the test examples " crlf
" used for the previous version have to be the same as those for the improved " crlf
" version. You may want to do some more tests with other test examples, for the " crlf
" previous version of the technique, in order to be sure that the test examples " crlf
" used were not chosen to favour the technique. " crlf 
" The methods for evaluation that will be suggested will be almost the same as those " crlf
" used for comparing rival techniques. The extended version will be your technique " crlf
" and the previous version the rival technique " crlf crlf 
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-56a
  (advice 4-56a)
=>
  (printout t crlf crlf
" As the technique is applied to a system, in order to test the technique you " crlf
" can test the system implementing the technique. You should, though, prove " crlf
" either that the system is completely based on the technique or that the aspects " crlf
" of the system based on the technique are crucial for the system to produce a " crlf
" specific behaviour. Testing will involve the system implementing the improved " crlf
" technique and the system implementing the previous version of the technique " crlf
" (rival technique). " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-56b
  (advice 4-56b)
=>
  (printout t crlf crlf
" As the technique is applied to a system, in order to test the technique " crlf
" theoretically you can, as well, base your study on the system implementing " crlf
" the technique. You should, though, prove either that the system is completely " crlf
" based on the technique or that the aspects of the system based on the technique " crlf
" are crucial for the system to produce a specific behaviour. Testing will " crlf
" involve the system implementing the improved technique and the system " crlf
" implementing the previous version of the technique (rival technique). " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-57
  (advice 4-57)
=>
  (printout t crlf crlf
" TEST BEHAVIOUR" crlf
" In order to use empirical evaluation to support your claims that the system " crlf
" has an improved behaviour because of improving the technique applied in the " crlf
" system you need to conduct a controlled experiment for each claim you make. " crlf
" Any experiment has the same basic form. First you choose a hypothesis to test," crlf
" which can be determined by measuring some attribute of subject behaviour. " crlf
" A number of experimental conditions should be considered which differ only in " crlf
" the values of certain control variables, which here will be sets of test " crlf
" examples. Every other aspect should stay the same! (If that is not possible " crlf
" you should find ways to counteract the changes in other conditions or make " crlf
" sure that their influence on the results has been taken into account). The " crlf
" subject can have a session with both rival systems with the same sets of " crlf
" examples and the property of behaviour tested should be measured. There are " crlf
" two experimental methods:" crlf crlf
"- Between groups (or randomized) design, where each subject is assigned to a " crlf
"  different condition. It has the advantage that any learning effect resulting " crlf
"  from the user performing in one condition and then the other is controlled " crlf
"  and the disadvantage that a greater number of subjects is required and that " crlf
"  individual differences between users can bias the results." crlf crlf
"- Within groups, where each subject is assigned to all different conditions. " crlf
"  This design can suffer from transfer of learning effects, but this can be " crlf
"  lessened if the order in which conditions are tackled is varied between users. " crlf
"  Within-groups is less costly than between groups, since fewer users are " crlf
"  required, and there is also less chance of effects from variation between " crlf
"  subjects." crlf crlf
"Press enter to continue: ")
  (readline)
  (printout t crlf crlf
" Within the basic form there are a number of factors that are important to the " crlf
" overall reliability of the experimental design. Thus, you should note that: " crlf crlf
"- Subjects should be chosen to match the expected user population as closely " crlf
"  as possible. Ideally this would involve experimental testing with the actual" crlf
"  users but this is not always possible. Also the sample size should be large " crlf
"  enough to be representative of the population." crlf crlf
"- Differences in experimental evidence may only be due to chance. Statistical " crlf
"  analysis may be required to show that these differences are significant " crlf
"  (e.g. t-test, described in GAGP-appendix A). " crlf crlf
"- The property of interest must be measurable in some way." crlf crlf
"Press enter to continue: ")
  (readline)
  (printout t crlf crlf
" When the results for both systems are taken, a suggested way to compare the " crlf
" results is to do a scatter plot for each property of interest. Each point " crlf
" will represent a test example, with its x value being the property of interest " crlf
" of the X system for this example and its y value being the property of interest " crlf
" of the Y system for this example. Then you can fit a line to these points and " crlf
" measure the angle. If it is 45 degrees then the systems exhibit the same " crlf
" behaviour across the range of these examples, if it is less than 45 degrees " crlf
" then X is better at this property than Y and if it is bigger than 45 then X " crlf
" is worse at this property than Y. " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-58
  (advice 4-58)
=>
  (printout t crlf crlf
" TEST BEHAVIOUR" crlf
" It is very difficult to compare the behaviour of two systems theoretically." crlf
" The only suggestion is to build a formal model for each system and try to " crlf
" predict the behaviour of each one (using mathematical tools or other modelling" crlf
" techniques) for some sets of test examples. " crlf crlf
" When the results for both techniques are taken, you can either find a " crlf
" theoretical method of comparing the results or you can use empirical methods " crlf
" like statistical or graphical techniques. " crlf
" One suggested way is to do a scatter plot for each property of interest. Each " crlf
" point will represent a test example, with its x value being the property of " crlf
" interest of the X system for this example and its y value being the property " crlf
" of interest of the Y system for this example. Then you can fit a line to these " crlf
" points and measure the angle. If it is 45 degrees then the systems exhibit the " crlf
" same behaviour across the range of these examples, if it is less than 45 degrees " crlf
" then X is better at this property than Y and if it is bigger than 45 then X is " crlf
" worse at this property than Y. Note however that the behaviour is only a " crlf
" prediction. An empirical test with the real systems has to take place. " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-59
  (advice 4-59)
=>
  (printout t crlf crlf
" TEST EFFICIENCY" crlf
" In order to do an experimental evaluation to compare two rival systems in " crlf
" terms of efficiency you need to conduct an experiment where you will measure " crlf
" the time or space for each system by feeding a different input size each time." crlf
" When the results for both systems are taken, a suggested way to compare the " crlf
" results is to do a scatter plot. Each point will represent an input size you " crlf
" fed into each system, with its x value being the time for system X for this " crlf
" example and its y value being the time for system Y for this example. " crlf
" Then you can fit a line to these points and measure the angle. If it is " crlf
" 45 degrees then the systems are equally efficient across the range of these " crlf
" examples, if it is less than 45 degrees then X is more efficient than Y and if" crlf
" it is bigger than 45 then X is less efficient than Y." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-60a
  (advice 4-60a)
=>
  (printout t crlf crlf
" TEST EFFICIENCY" crlf
" In order to compare the two systems theoretically, in terms of efficiency " crlf
" and given that programming is involved you can use complexity analysis. You " crlf
" can do complexity analysis to measure efficiency in terms of space or cpu " crlf
" time. You can calculate the efficiency of all the program or of a specific " crlf
" procedure. Complexity analysis measures efficiency in terms of an order of " crlf
" magnitude estimate, e.g. that the cpu time is a linear, quadratic or " crlf
" exponential function of the size of input to a procedure. Thus, you can " crlf
" calculate the complexity in terms of an order of magnitude estimate for each " crlf
" rival system. Then, you can draw conclusions like e.g. if the complexity is " crlf
" linear for system X and quadratic for system Y then system X is more efficient " crlf
" than system Y. However, you should note that complexity analysis is usually " crlf
" for worst case, i.e. very large inputs. An average case analysis may show " crlf
" different results. " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-60b
  (advice 4-60b)
=>
  (printout t crlf crlf
" Another way to compare two interactive systems in terms of efficiency is " crlf
" using the GOMS model at the keystroke level. This model is described in " crlf
" HCI-chapter 6, and although it is not reliable for absolute measures of " crlf
" efficiency it can be very useful for relative measures, i.e. for testing " crlf
" if a system is more efficient than another one. " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-61a
  (advice 4-61a)
=>
  (printout t crlf crlf
" TEST EFFICIENCY" crlf
" One way to compare the systems theoretically in terms of efficiency is to " crlf
" find a formal model for each system and use mathematical tools or other " crlf
" modelling techniques to compare their efficiency. For example you can use " crlf
" operational laws to calculate the throughput for each of the two rival systems," crlf
" for different test examples. Then you can compare their average throughput and " crlf
" draw conclusions about which is more efficient." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-61b
  (advice 4-61b)
=>
  (printout t crlf crlf
" Another way to compare two interactive systems in terms of efficiency is " crlf
" by using the GOMS model at the Keystroke level. This model is described in " crlf
" HCI-chapter 6, and although it is not reliable for absolute measures of " crlf
" efficiency it can be very useful for relative measures, i.e. for testing if " crlf
" a system is more efficient than another one." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-62
  (advice 4-62)
=>
  (printout t crlf crlf
" TEST COVERAGE" crlf
" In order to compare two rival systems experimentally in terms of the coverage " crlf
" you need to conduct experiments where you feed input into each system and see" crlf
" how well each system behaves, if at all. Each input represents a different " crlf
" situation that the system may be requested to deal with. First you have to " crlf
" identify the range of applications where each system behaves correctly, " crlf
" i.e. does the task and produces correct results. That will define the coverage " crlf
" of each system. But usually that is not enough. You have to identify patterns " crlf
" in changes of behaviour or efficiency in different situations. E.g. you may " crlf
" find out that system X behaves better for a range of applications and worse " crlf
" for another range, i.e. has worse quality or efficiency than system Y in " crlf
" some other range. " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-63
  (advice 4-63)
=>
  (printout t crlf crlf
" TEST COVERAGE" crlf
" In order to compare the rival systems theoretically in terms of coverage you " crlf
" need to be able to represent the systems in a formal way using mathematical " crlf
" tools or other. Then you will have to prove that the behaviour of the system " crlf
" is correct for some well defined range. Then you can prove that this range," crlf
" is a superset of the range where the rival system behaves correctly." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-64
  (advice 4-64)
=>
  (printout t crlf crlf
" TEST BEHAVIOUR" crlf
" For non-interactive systems or architectures the suggested way to compare " crlf
" them in terms of behaviour empirically, is by letting experts in the task say " crlf
" which system behaves better. The expert should judge \"blindly\", i.e. the human " crlf
" expert should not know which system s/he judges each time, so that s/he " crlf
" doesn't favour one of them. In that way you can compare rivals in terms of " crlf
" behavioural properties like correctness, quality, simplicity etc. Not always" crlf
" is an expert necessary as a judge. E.g. If you've built a robot to play rugby " crlf
" and you want to compare it with a rival one you can put them to play against " crlf
" each other several matches and see which is the winner. You don't necessarily " crlf
" need an expert in rugby to judge, unless you want to assess other properties " crlf
" as well that do not depend on which robot wins the game." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-65
  (advice 4-65)
=>
  (printout t crlf crlf
" TEST BEHAVIOUR" crlf
" It is very difficult to compare theoretically rival non-interactive systems " crlf
" or architectures. The best way is to do some theoretical research concerning " crlf
" the task, e.g. use mathematical tools or physics to model the task and then " crlf
" try to predict how each system will behave in different situations and compare " crlf
" the results. In the end you will have to test the real systems as well, as " crlf
" the theoretical analysis gives only predictions of the behaviour." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-66
  (advice 4-66)
=>
  (printout t crlf crlf
" In order to do experimental evaluation you need to build a system based on " crlf
" your theory or make simulation of a system. Then you can empirically prove " crlf
" that the system behaves better when it implements the improved version of " crlf
" the technique." crlf
" Another way would be to perform a hand simulation (or structured walk-through), " crlf
" by explaining how the technique works at each step (going through it step by " crlf
" step) and justifying, that way, that your technique has improved behaviour " crlf
" for a specific range of inputs." crlf crlf
"Press enter to continue: ")
  (readline)
  (printout t crlf crlf
" However, if your research is theoretical then it doesn't help to do an" crlf
" experimental evaluation. It will be more helpful to do a theoretical " crlf
" evaluation." crlf crlf))


(defrule ADVICE::4-67
  (advice 4-67)
=>
  (printout t crlf crlf
" In order to do experimental evaluation you need to build a system based on " crlf
" your theory or make simulation of a system. Then you can empirically prove " crlf
" that the system covers a wider range of examples, when it implements the " crlf
" improved version of the technique. " crlf
" Another way would be to perform a hand simulation (or structured walk-through)," crlf
" by explaining how the technique works at each step (going through it step by " crlf
" step) and justifying, that way, that your technique covers a wider range of " crlf
" applications (produces satisfactory results for a wider range of inputs)." crlf crlf
"Press enter to continue: ")
  (readline)
  (printout t crlf crlf
" However, if your research is theoretical then it doesn't help to do an" crlf
" experimental evaluation. It will be more helpful to do a theoretical " crlf
" evaluation." crlf crlf))


(defrule ADVICE::4-68
  (advice 4-68)
=>
  (printout t crlf crlf
" As there exists a formal model for your technique you can try to predict " crlf
" what kind of results your technique produces in different situations and " crlf
" compare those results with the ones the rival technique produces for the " crlf
" same situation. You may want to use some techniques for deriving performance " crlf
" measures like those described in MS, if you have used one of the modelling " crlf
" techniques described there." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-69
  (advice 4-69)
=>
  (printout t crlf crlf
" As there exists some algorithm for your technique you can also predict some " crlf
" behaviour by testing what output the algorithm will produce for certain input " crlf
" and then justify this by proving the correctness of your algorithm. More for " crlf
" the correctness of algorithms you can find in ALG." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-68-9
  (advice 4-68-9)
=>
  (printout t crlf crlf
" When the results for both techniques are taken, you can either find a " crlf
" theoretical method of comparing the results or you can use empirical methods " crlf
" like statistical or graphical techniques." crlf
" One suggested way is to do a scatter plot for each property of interest. " crlf
" Each point will represent a different situation, with its x value being the " crlf
" property of interest of the X system for this example and its y value being " crlf
" the property of interest of the Y system for this example. Then you can fit " crlf
" a line to these points and measure the angle. If it is 45 degrees then the " crlf
" techniques exhibit the same behaviour across the range of these examples, if " crlf
" it is less than 45 degrees then X is better at this property than Y and if it " crlf
" is bigger than 45 then X is worse at this property than Y. " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-70
  (advice 4-70)
=>
  (printout t crlf crlf
" As some algorithm is involved you can use complexity analysis to compare " crlf
" your technique with the rival technique in terms of efficiency. You can " crlf
" calculate the efficiency of a specific algorithm or of a whole process. " crlf
" Complexity analysis measures efficiency in terms of an order of magnitude " crlf
" estimate, e.g. that the cpu time is a linear, quadratic or exponential " crlf
" function of the size of input to a procedure. Thus, you can calculate the " crlf
" complexity in terms of an order of magnitude for each rival technique. " crlf
" Then, you can draw conclusions like e.g. if the complexity is linear for " crlf
" system X and quadratic for system Y then system X is more efficient than " crlf
" system Y. However, you should note that complexity analysis is usually for " crlf
" worst case, i.e. very large inputs. An average case analysis may show " crlf
" different results. " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-71
  (advice 4-71)
=>
  (printout t crlf crlf
" As a formal model exists for your technique you can compare the efficiency " crlf
" of your technique with the efficiency of a rival one by deriving performance " crlf
" measures like response time or waiting time and comparing them with the " crlf
" corresponding measures of the rival techniques for the same test examples. " crlf
" You can use e.g. operational laws (described in MS) to calculate the " crlf
" throughput for each of the techniques or use other mathematical tools, prove " crlf
" some theorems, etc. It depends on the way you've modelled your technique and " crlf
" the kind of technique ,which method you'll choose." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-72
  (advice 4-72)
=>
  (printout t crlf crlf
" As a formal model exists for your theory/technique you can prove that the " crlf
" behaviour of your technique, predicted from the model, is correct for some " crlf
" well defined range of inputs. Then you will have to prove that this range, " crlf
" is a superset of the range where the rival theory/technique behaves correctly." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-73
  (advice 4-73)
=>
  (printout t crlf crlf
" As some algorithm is involved in your theory/technique you can compare the " crlf
" techniques in terms of coverage by claiming that your algorithm(s) produces " crlf
" correct output, i.e. the output it is supposed to produce, for a certain, well " crlf
" defined range of inputs and you can justify this claim by proving the " crlf
" correctness of your algorithm in producing that output for the specific " crlf
" input. Details about the correctness of algorithms can be found in ALG-chapter." crlf
" Then you will have to prove that this range, is a superset of the range where " crlf
" the rival technique's algorithm produces correct results." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-74
  (advice 4-74)
=>
  (printout t crlf crlf
" When you have proved that your technique has indeed been extended to cover " crlf
" a wider range of applications then you can attribute this to the parameter " crlf
" you've changed in the technique." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-75
  (advice 4-75)
=>
  (printout t crlf crlf
" When you have proved that your technique has indeed been improved in terms of " crlf
" some specific properties, you can attribute this improvement to the" crlf
" parameter you've changed in the technique." crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-76
  (advice 4-76)
=>
  (printout t crlf crlf
" TEST THAT BEHAVIOUR DEPENDS ON THE RANGE OF INPUTS " crlf
" In order to use empirical evaluation to support your claims that the " crlf
" behaviour of the system depends on the range of inputs, you need to" crlf
" conduct a controlled experiment for each claim you make. Any experiment " crlf
" has the same basic form. First you choose a hypothesis to test, which can " crlf
" be determined by measuring some attribute of subject behaviour. A number " crlf
" of experimental conditions should be considered which differ only in the " crlf
" values of certain control variables. The control variables here will be " crlf
" different example sessions that the subject should go through. Every other " crlf
" condition should stay the same! (If that is not possible you should find " crlf
" ways to counteract the changes in other conditions or make sure that their " crlf
" influence on the results has been taken into account). Then the properties " crlf
" of behaviour you want to test will be measured across the range of different " crlf
" examples. You will then have to identify patterns in changes of behaviour " crlf
" between the different examples. You may for example find that your system " crlf
" behaves good for a certain range of examples and outside that range the " crlf
" quality is poor. It is often informative to use graphical and statistical " crlf
" methods to explore, hypothesise and test such patterns." crlf crlf
"Press enter to continue: ")
  (readline)
(printout t crlf crlf
" There are two experimental methods (according to HCI-chapter 11):" crlf crlf
"- Between-groups (or randomized) design, where each subject is assigned to " crlf
"  a different condition. It has the advantage that any learning effect " crlf
"  resulting from the user performing in one condition and then the other " crlf
"  is controlled and the disadvantage that a greater number of subjects are " crlf
"  required and that individual differences between users can bias the results." crlf crlf
"- Within-groups, where each user performs under each different condition. This " crlf
"  design can suffer from transfer of learning effects, but this can be lessened " crlf
"  if the order in which conditions are tackled is varied between users. " crlf
"  Within-groups is less costly than between-groups, since fewer users are " crlf
"  required and there is also less chance of effects from variation between " crlf
"  subjects." crlf crlf
"Press enter to continue: ")
  (readline)
(printout t crlf crlf 
" Within the basic form there are a number of factors that are important to " crlf
" the overall reliability of the experimental design. Thus, you should note that:" crlf crlf
"- Subjects should be chosen to match the expected user population as closely " crlf
"  as possible. Ideally this would involve experimental testing with the actual " crlf
"  users but this is not always possible. Also the sample size should be large " crlf
"  enough to be representative of the population." crlf crlf
"- Differences in experimental evidence may only be due to chance. Statistical " crlf
"  analysis may be required to show that these differences are significant " crlf
"  (e.g. t-test described in GAGP-appendix A)." crlf crlf
"- Experimental conditions should only differ in the control variable and be " crlf
"  otherwise identical." crlf crlf
"- The property of interest must be measurable in some way." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-77
  (advice 4-77)
=>
  (printout t crlf crlf
" TEST THAT BEHAVIOUR DEPENDS ON THE RANGE OF INPUTS" crlf
" You can use Model-based evaluation to theoretically test that the system's " crlf
" behaviour depends on the range of inputs. Model-based evaluation uses a model " crlf
" of the system to derive performance measures. Certain cognitive and design " crlf
" models provide a means of combining design specification and evaluation into " crlf
" the same framework. One example is the GOMS model (described in HCI-chapter 6)." crlf crlf
" Some aspects of the behaviour of the system are not covered by cognitive " crlf
" models. For these aspects the only way to test them theoretically is not to " crlf
" use a cognitive model, but model your system or technique using mathematical " crlf
" tools or other. You can also exploit the theory about modelling that already " crlf
" exists. There are a lot of ways to model a system and derive performance " crlf
" measures, e.g. the operational laws are simple equations which may be used " crlf
" as an abstract representation or model of the average behaviour of almost any" crlf
" system. You may then be able to identify patterns in changes of behaviour " crlf
" between different test examples. You may for example find that your system " crlf
" behaves good for a certain range of inputs you feed into the model and outside " crlf
" that range the quality is poor. It is often informative to use graphical and " crlf
" statistical methods to explore,hypothesise and test such patterns." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-78
  (advice 4-78)
=>
  (printout t crlf crlf
" TEST THAT EFFICIENCY DEPENDS ON THE RANGE OF INPUTS" crlf
" In order to empirically prove that the efficiency of the system depends on " crlf
" the range of inputs, you just have to measure how much memory it uses (if " crlf
" space is considered) or how much CPU time it uses (if time is considered) in " crlf
" different sets of test examples. Each example will represent a point P(x,y) " crlf
" where x is the size of input and y is the space or time. Then you can plot the" crlf
" graph of the space or time at y-axis and the size of input at x-axis by " crlf
" connecting these points. You can also transform dots before plotting e.g. by " crlf
" using logarithmic graph paper. Then, you can identify patterns in changes of " crlf
" efficiency for different test examples. You can e.g. find out that the system " crlf
" is very efficient for some well defined range of examples and very inefficient " crlf
" for some other range. " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-79
  (advice 4-79)
=>
  (printout t crlf crlf
" TEST THAT EFFICIENCY DEPENDS ON THE RANGE OF INPUTS" crlf
" In order to prove theoretically that the efficiency of the system depends on " crlf
" the range of inputs you can use complexity analysis. You can do complexity " crlf
" analysis to measure efficiency in terms of space or time. You can calculate " crlf
" the efficiency of all the program, which is very hard to do theoretically, or " crlf
" of a specific procedure. Complexity analysis measures efficiency in terms of " crlf
" an order of magnitude estimate. e.g. that the cpu time is a linear, quadratic " crlf
" or exponential function of the size of input to a procedure. Complexity " crlf
" analysis in its own provides a proof that the efficiency depends on the range " crlf
" of inputs. Thus, unless you calculated a constant complexity function, your " crlf
" claim will be justified. " crlf 
" More details on complexity analysis can be found in ALG-chapter 6." crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-80
  (advice 4-80)
=>
  (printout t crlf crlf
" TEST THAT EFFICIENCY DEPENDS ON THE RANGE OF INPUTS" crlf
" In order to prove theoretically that the efficiency of the system depends " crlf
" on the range of inputs when a program is not involved, or you don't have " crlf
" access to it, then the only way is to find a way to model your system or " crlf
" technique in a formal way using mathematical tools or other. You may then be" crlf
" able to work out the time or space as a function of the size of input. You " crlf
" can also exploit the theory about modelling that already exists. There are " crlf
" a lot of ways to model a system and derive performance measures like response " crlf
" time, service rate or waiting time, to measure efficiency. One example is the" crlf
" operational laws (for details see MS-section 2), which are simple equations " crlf
" that may be used as an abstract representation or model of the average " crlf
" behaviour of almost any system. That way you can prove that efficiency depends " crlf
" on the range input by e.g. showing that the predicted throughput differs a lot " crlf
" as different inputs are fed into the model. On the other hand, you may find out" crlf
" that the throughput is independent from the input and is always the same." crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-81
  (advice 4-81)
=>
  (printout t crlf crlf
" TEST THAT BAHAVIOUR DEPENDS ON THE RANGE OF INPUTS" crlf
" For a non-interactive system the suggested way to empirically test if " crlf
" its behaviour depends on the range of inputs the best way is to conduct " crlf
" an experiment, where you will feed different inputs in the system and " crlf
" observe the produced behaviour. Most of the times an expert in the task " crlf
" should be there to evaluate the behaviour of the system i.e. to say when " crlf
" the behaviour is correct or of good quality. Sometimes, you may find other " crlf
" ways of assessing the correctness or quality of your system and an expert " crlf
" may not be necessary. E.g. If you've built a robot to find its way through " crlf
" a maze then you can test the robot in different test examples (probably " crlf
" different configurations of the maze). Observing that the robot finds its way" crlf
" through the maze is enough to know that the behaviour is correct, you don't " crlf
" need an expert to tell you that. Even for testing if it found the simplest " crlf
" way, an expert is not necessary. There are some times that a behaviour cannot " crlf
" be 100% correct. In these cases you need to measure accuracy, i.e. how far " crlf
" from 100% your result in each case is. This can be evaluated in some cases " crlf
" with the help of an expert and in others you can estimate it yourself when " crlf
" you know the absolute correct behaviour. You can then see if accuracy is " crlf
" different for different inputs and if there is difference find out at what " crlf
" range inputs the system produces the more accurate behaviour. Thus, with such " crlf
" an analysis you will be able to identify patterns in changes of behaviour " crlf
" between different test examples. You may for example find that your system " crlf
" behaves good for a certain range of examples and outside that range the " crlf
" quality is poor. It is often informative to use graphical and statistical " crlf
" methods to explore, hypothesise and test such patterns." crlf crlf
"Press enter to continue: ")
  (readline))



(defrule ADVICE::4-82
  (advice 4-82)
=>
  (printout t crlf crlf
" TEST THAT BAHAVIOUR DEPENDS ON THE RANGE OF INPUTS" crlf
" It is difficult to test, theoretically, behavioural properties. Some of them " crlf
" are easier to test than others, but all of them have to be formally " crlf
" characterized in order to be tested theoretically. You need to have built a " crlf
" formal model for your system, using mathematical tools or other (some " crlf
" modelling techniques are described in MS). Then you may be able to predict " crlf
" behavioural properties by feeding different inputs in your model. You can " crlf
" also do some theoretical research concerning the task your system performs. " crlf
" E.g. If you have built a robot arm to grasp objects you do some theoretical" crlf
" research into the task by using a combination of geometry and mechanics to " crlf
" investigate :optimal places and forces to hold an object, optimal trajectories " crlf
" for a robot hand/arm to traverse, required accuracies in motors and joints " crlf
" for acceptable error rates etc. This analysis can then be used to predict " crlf
" how the robot arm will behave in different ranges of input. You might predict " crlf
" that the robot's behaviour would be of sufficiently high quality in terms of" crlf
" its success rate and accuracy, for a certain range but of poor quality for " crlf
" another range. Thus, with such an analysis you will be able to identify " crlf
" patterns in changes of behaviour for your system, between different test " crlf
" examples. You may, for example, predict that your system behaves good for a" crlf
" certain range of examples and outside that range the quality is poor. It is " crlf
" often informative to use graphical and statistical methods to explore, " crlf
" hypothesise and test such patterns. Of course you will eventually have to " crlf
" show that these theoretical predictions are borne out in practice by testing " crlf
" the real robot. " crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-83
  (advice 4-83)
=>
  (printout t crlf crlf
" For an experimental evaluation of a technique that exists only in theory " crlf
" you can try to perform a hand simulation (or structured walk-through)," crlf
" by explaining how the technique works at each step (going through it " crlf
" step by step) and justifying, that way, that the behaviour you predict " crlf
" for your technique is better than the one predicted for the rival " crlf
" techniques for certain inputs." crlf crlf
" Otherwise, you will need to build a system based on your theory or make " crlf
" simulation of a system, in order to perform experimental evaluation. Then " crlf
" you can empirically prove that the behaviour or efficiency of your technique" crlf
" depends on the range of inputs. However, even in this case, testing that the" crlf
" system based on your technique/theory really produces a behaviour that depends" crlf
" on the range of inputs, is not enough. You also need to prove that:" crlf crlf
"- The system really depends upon that theory."crlf 
"- The aspects of the system that depend on that theory are really crucial " crlf
"  at producing that behaviour." crlf crlf
"Press enter to continue: ")
  (readline)
  (printout t crlf crlf
" However, if your research is theoretical then it doesn't help to do an" crlf
" experimental evaluation. It will be more helpful to do a theoretical " crlf
" evaluation." crlf crlf))


(defrule ADVICE::4-84
  (advice 4-84)
=>
  (printout t crlf crlf
" As your theory involves a formal model of your technique then you may be " crlf
" able to prove that some behavioural properties, like e.g. correctness depends " crlf
" on the range of inputs. Some properties like quality are usually harder to " crlf
" test when there is no implementation. Usually it depends on whether the " crlf
" property can be formally characterized. Also, if your technique is modeled " crlf
" using a modelling technique (like those discussed in MS) then you can easily " crlf
" predict some behavioural properties of the system that is modeled to implement " crlf
" the technique. You should though make sure that the behaviour predicted is " crlf
" due to aspects of the model that are based on the technique. That way you " crlf
" can feed different inputs into your model and identify patterns in changes " crlf
" of behaviour. It will be useful to use graphical or statistical methods to " crlf
" explore, hypothesise and test such patterns." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-85
  (advice 4-85)
=>
  (printout t crlf crlf
" As your theory involves some algorithms, you can prove that some behavioural " crlf
" properties depend on the range of inputs by claiming that your algorithm(s) " crlf
" produces a certain range of outputs for a certain range of inputs and then " crlf
" justify this claim by proving the correctness of your algorithm in producing " crlf
" that output for the specific input. Details about the correctness of " crlf
" algorithms can be found in ALG-chapter 5. You can then prove that the output " crlf
" produced is different for different inputs e.g. produce results of better " crlf
" quality for a specific well defined range of applications and worse for some" crlf
" other range." crlf crlf
"Press enter to continue: ")
  (readline))

(defrule ADVICE::4-86
  (advice 4-86)
=>
  (printout t crlf crlf
" You may be able to prove that the efficiency of your technique depends on " crlf
" the range of inputs by using complexity analysis of algorithms. You can do " crlf
" complexity analysis to measure efficiency in terms of space or time. You can" crlf
" calculate the efficiency of one algorithm, or of a specific process, i.e. a " crlf
" combination of algorithms. Complexity analysis measures efficiency in terms " crlf
" of an order of magnitude estimate. e.g. that the cpu time is a linear, " crlf
" quadratic or exponential function of the size of input to a procedure. " crlf
" Complexity analysis in its own provides a proof that the efficiency depends " crlf
" on the range of inputs. Thus, unless you calculated a constant complexity" crlf
" function, your claim will be justified." crlf crlf
"Press enter to continue: ")
  (readline))


(defrule ADVICE::4-87
  (advice 4-87)
=>
  (printout t crlf crlf
" You can prove that the efficiency of your technique depends on the range " crlf
" of inputs by using the formal model of your technique. You may be able to " crlf
" work out the time or space as a function of the size of input you will feed " crlf
" into your model, and that way you have already a proof that the efficiency " crlf
" depends on the range of inputs, unless the function is a constant one." crlf
" You can also exploit the theory about modelling that already exists. There " crlf
" are a lot of ways to derive performance measures from a model, like response " crlf
" time, service rate or waiting time, to measure efficiency. One example is " crlf
" the operational laws (for details see MS-section 2). That way you can prove " crlf
" that efficiency depends on the range input by e.g. showing that the predicted" crlf
" throughput differs a lot as different inputs are fed into the model. On the " crlf
" other hand, you may find out that the throughput is independent from the " crlf
" input and is always the same. " crlf crlf
"Press enter to continue: ")
  (readline))




;;********************************************************************************
;;General question asked when a topic is completed so that the user can return to* 
;;the main menu or exit the program.                                             *
;;********************************************************************************

(defrule ADVICE::question-general
 (declare (salience -200))
  ?qg <- (question general)
=>
 (retract ?qg)
 (printout t crlf crlf
   " You reached the end of the questions and advice for this topic. " crlf
   " Do you want ... " crlf crlf
   "              1.)a review of the advice given? " crlf
   "              2.)exit the program?  " crlf
   "              3.)return to the main menu? " crlf crlf
   "        Choice: ")
  (bind ?response (read))
  (assert (answer general ?response))
  (assert (answer-check ?response general 3))
  (focus ERROR))

(defrule ADVICE::advice-review
  (advice ?no)                              
  (answer general 1)
  =>
  (assert (question end))  
  (refresh ?no)                ;;all rules that have any advice already
                               ;;in the fact list(given) fire again.
 )
   
   
(defrule ADVICE::end-question
(declare (salience -100))
   (or (question end)
       (answer general 2))
   =>
   (printout t crlf crlf
   " Do you want to ... " crlf crlf
   "              1.) exit the program?" crlf
   "              2.) return to the main-menu? " crlf crlf
   "        Choice: ")
  (bind ?response (read))
  (printout t crlf crlf)
  (assert (answer end ?response))
  (assert (answer-check ?response end 2))
  (focus ERROR))

(defrule ADVICE::answer-end
 (or (answer general 2)
     (answer end 1))
=>
  (exit))



